﻿/*
 Copyright (c) 2012-2014 EU Outsourcing, Inc
All rights reserved.  http://www.euoutsourcing.com
Version: 2.1.0
Disclaimer: 
Any redistribution, modification and use of this software either in source or binary forms is permitted provided that the following terms are met: 
1. This software’s source code is not modified to avoid the use of EasyXLS™. 
2.  All servers and computers, whether physical or virtual, using this software or its derivatives must be properly licensed to use EasyXLS™.
3. Any derivatives of this source code that is redistributed must include the above copyright notice, these five conditions and the included disclaimer. 
4. Any redistributions in binary form must include the above copyright notice, these five conditions and the included disclaimer in any materials that are provided with the distribution. 
5. EasyXLS™ and the name of its parent organization may not be used to endorse or promote any software products or derivative works without specific prior written permission from the copyright holders.
DISCLAIMER.  THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using EasyXLS;
using System.Drawing;
using EasyXLS.Drawings;

namespace EasyXLSDemoApp.EasyXLSGrid
{
    // class for storing cell style, width and height
    public class CellStyleData
    {
        // cell style
        private ExcelStyle _xlsStyle = null;
        // cell data type
        private string _xlsDataType;
        // cell width
        private int _xlsWidth;
        // cell height auto size
        private bool _xlsAutoSizeHeight;
        // cell height
        private int _xlsHeight;
        // unformatted cell Value
        private bool _formatRed;
        // pattern color
        private Color _patternColor;
        // font color
        private Color _fontColor;
        // comments
        private ExcelComment _xlsComments;
        // html values
        private string _htmlValues;


        // geter/seter for cell style
        public ExcelStyle XlsStyle
        {
            get { return _xlsStyle; }
            set { _xlsStyle = value; }
        }

        // geter/seter for cell data type
        public string XlsDataType
        {
            get { return _xlsDataType; }
            set { _xlsDataType = value; }
        }

        

        // geter/seter for cell width
        public int XlsWidth
        {
            get { return _xlsWidth; }
            set { _xlsWidth = value; }
        }

        // geter/seter for cell auto size height
        public bool XlsAutoSizeHeight
        {
            get { return _xlsAutoSizeHeight; }
            set { _xlsAutoSizeHeight = value; }
        }

        // geter/seter for cell height
        public int XlsHeight
        {
            get { return _xlsHeight; }
            set { _xlsHeight = value; }
        }

        // geter/seter for cell height
        public bool IsFormatRed
        {
            get { return _formatRed; }
            set { _formatRed = value; }
        }

        // geter/seter for cell pattern color
        public Color PatternColor
        {
            get { return _patternColor; }
            set { _patternColor = value; }
        }

        // geter/seter for cell pattern color
        public Color FontColor
        {
            get { return _fontColor; }
            set { _fontColor = value; }
        }

        // geter/seter for cell pattern color
        public ExcelComment XlsComments
        {
            get { return _xlsComments; }
            set { _xlsComments = value; }
        }

        // geter/seter for cell html value
        public string HtmlValue
        {
            get { return _htmlValues; }
            set { _htmlValues = value; }
        }
        

        // constructor for CellStyleData class
        public CellStyleData(ExcelStyle xlsStyle, string xlsDataType, int xlsWidth, bool xlsAutoSizeHeight, int xlsHeight, ExcelComment xlsComment, string htmlValue)
        {
            XlsStyle = xlsStyle;
            XlsDataType = xlsDataType;
            XlsWidth = xlsWidth;
            XlsAutoSizeHeight = xlsAutoSizeHeight;
            XlsHeight = xlsHeight;
            XlsComments = xlsComment;
            HtmlValue = htmlValue;
        }

        // constructor for CellStyleData class
        public CellStyleData()
        {
            XlsStyle = null;
            XlsDataType = "";
            XlsWidth = 0;
            XlsAutoSizeHeight = true;
            XlsHeight = 0;
        }

        public void PopulateValues(ExcelStyle xlsStyle, string xlsDataType, int xlsWidth, bool xlsAutoSizeHeight, int xlsHeight, bool numericFormatRed, Color patternCol, ExcelComment xlsComment, string htmlValue)
        {
            XlsStyle = xlsStyle;
            XlsDataType = xlsDataType;
            XlsWidth = xlsWidth;
            XlsAutoSizeHeight = xlsAutoSizeHeight;
            XlsHeight = xlsHeight;
            IsFormatRed = numericFormatRed;
            PatternColor = patternCol;
            XlsComments = xlsComment;
            HtmlValue = htmlValue;
        }

        /// <summary>
        /// IsEmptyStyleCell - check if cell has no style defined
        /// </summary>
        /// <param name="cellStyle"></param>
        /// <returns></returns>
        public static bool IsEmptyStyleCell(CellStyleData cellStyle)
        {
            bool isEmpty = true;
            if (cellStyle.XlsStyle != null)
            {

                ExcelStyle excelStyle = cellStyle.XlsStyle.Clone();

                // check left border color
                if (excelStyle.getLeftColor_Border().A != 0 || excelStyle.getLeftColor_Border().R != 0 || excelStyle.getLeftColor_Border().G != 0 || excelStyle.getLeftColor_Border().B != 0)
                {
                    isEmpty = false;
                    return isEmpty;
                }

                // check right border color
                if (excelStyle.getRightColor_Border().A != 0 || excelStyle.getRightColor_Border().R != 0 || excelStyle.getRightColor_Border().G != 0 || excelStyle.getRightColor_Border().B != 0)
                {
                    isEmpty = false;
                    return isEmpty;
                }

                // check top border color
                if (excelStyle.getTopColor_Border().A != 0 || excelStyle.getTopColor_Border().R != 0 || excelStyle.getTopColor_Border().G != 0 || excelStyle.getTopColor_Border().B != 0)
                {
                    isEmpty = false;
                    return isEmpty;
                }

                // check bottom border color
                if (excelStyle.getBottomColor_Border().A != 0 || excelStyle.getBottomColor_Border().R != 0 || excelStyle.getBottomColor_Border().G != 0 || excelStyle.getBottomColor_Border().B != 0)
                {
                    isEmpty = false;
                    return isEmpty;
                }

                // check if merged cell
                if (excelStyle.IsMergeCell())
                {
                    isEmpty = false;
                    return isEmpty;
                }

                // check background color
                if (excelStyle.getBackground().A != 0 || excelStyle.getBackground().R != 0 || excelStyle.getBackground().G != 0 || excelStyle.getBackground().B != 0)
                {
                    isEmpty = false;
                    return isEmpty;
                }

                //pattern color
                if (cellStyle.PatternColor.A != 0 || cellStyle.PatternColor.R != 0 || cellStyle.PatternColor.G != 0 || cellStyle.PatternColor.B != 0)
                {
                    isEmpty = false;
                    return isEmpty;
                }
            }
            return isEmpty;
        }

    }
}
