﻿/*
 Copyright (c) 2012-2014 EU Outsourcing, Inc
All rights reserved.  http://www.euoutsourcing.com
Version: 2.1.0
Disclaimer: 
Any redistribution, modification and use of this software either in source or binary forms is permitted provided that the following terms are met: 
1. This software’s source code is not modified to avoid the use of EasyXLS™. 
2.  All servers and computers, whether physical or virtual, using this software or its derivatives must be properly licensed to use EasyXLS™.
3. Any derivatives of this source code that is redistributed must include the above copyright notice, these five conditions and the included disclaimer. 
4. Any redistributions in binary form must include the above copyright notice, these five conditions and the included disclaimer in any materials that are provided with the distribution. 
5. EasyXLS™ and the name of its parent organization may not be used to endorse or promote any software products or derivative works without specific prior written permission from the copyright holders.
DISCLAIMER.  THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using EasyXLS;
using System.Drawing;
using System.Data;
using EasyXLSDemoApp.EasyXLSGrid;

namespace EasyXLSDemoApp.EasyXLSData
{
    // class for storing cell style, width and height
    public class ChartData
    {
        // chart row position
        private int _rowPosition = 0;
        // chart column position
        private int _colPosition = 0;
        // chart picture URL
        private string _chartURL;
        // chart size
        private double _chartWidth;
        // chart size
        private double _chartHeight;
        // col span
        private int _colSpan;
        // row span
        private int _rowSpan;

        private const int emptyCellWidth = 65;
        private const int emptyCellHeight = 21;

        // geter/seter for row position
        public int RowPosition
        {
            get { return _rowPosition; }
            set { _rowPosition = value; }
        }

        // geter/seter for column position
        public int ColPosition
        {
            get { return _colPosition; }
            set { _colPosition = value; }
        }

        // geter/seter for chart URL
        public string ChartURL
        {
            get { return _chartURL; }
            set { _chartURL = value; }
        }

        // geter/seter for chart width
        public double ChartWidth
        {
            get { return _chartWidth; }
            set { _chartWidth = value; }
        }

        // geter/seter for chart height
        public double ChartHeight
        {
            get { return _chartHeight; }
            set { _chartHeight = value; }
        }

        // geter/seter for row span
        public int RowSpan
        {
            get { return _rowSpan; }
            set { _rowSpan = value; }
        }

        // geter/seter for column span
        public int ColSpan
        {
            get { return _colSpan; }
            set { _colSpan = value; }
        }

        // constructor for ChartData class
        public ChartData(string chartUrl, int rowPos, int colPos, double chartWidth, double chartHeight)
        {
            ChartURL = chartUrl;
            RowPosition = rowPos;
            ColPosition = colPos;
            ChartWidth = chartWidth;
            ChartHeight = chartHeight; 
        }

        // constructor for ChartData class
        public ChartData()
        {
            ChartURL = "";
            RowPosition = -1;
            ColPosition = -1;
            ChartWidth = -1;
            ChartHeight = -1; 
        }

        /// <summary>
        /// FindTheLastRowAndColumn
        /// </summary>
        /// <param name="dtCharts"></param>
        static public int [] FindTheLastRowAndColumn(DataTable dtCharts)
        {
            // array to keep the maximum no of columns and no of rows
            int[] rowCol = new int[6];

            // Maximum Row Position
            int maxRow = -1;
            int rMaxRowPostitionIdx = -1;
            int cMaxRowPostitionIdx = -1;

            // Maximum Col Position
            int maxCol = -1;
            int rMaxColPositionIdx = -1;
            int cMaxColPositionIdx = -1;

            // for all rows in the table
            for (int rowIdx = 0; rowIdx < dtCharts.Rows.Count; rowIdx++)
            {
                // for all columns in the table
                for (int colIdx = 0; colIdx < dtCharts.Columns.Count; colIdx++)
                {
                    if (dtCharts.Rows[rowIdx][colIdx].ToString() != "")
                    {
                        // get the chart info
                        ChartData chrtData = (ChartData)dtCharts.Rows[rowIdx][colIdx];

                        // get the maximum no row and maximum no of cols
                        if (maxRow < chrtData.RowPosition)
                        {
                            maxRow = chrtData.RowPosition;
                            rMaxRowPostitionIdx = rowIdx;
                            cMaxRowPostitionIdx = colIdx;
                        }
                        if (maxCol < chrtData.ColPosition)
                        {
                            maxCol = chrtData.ColPosition;
                            rMaxColPositionIdx = rowIdx;
                            cMaxColPositionIdx = colIdx;
                        }
                    }
                }
            }

            // add the max row and max column to an array
            rowCol[0] = maxRow;
            rowCol[1] = rMaxRowPostitionIdx;
            rowCol[2] = cMaxRowPostitionIdx;
            rowCol[3] = maxCol;
            rowCol[4] = rMaxColPositionIdx;
            rowCol[5] = cMaxColPositionIdx;

            return rowCol;

        }

        /// <summary>
        /// FindTheLastRowAndColumn
        /// </summary>
        /// <param name="dtCharts"></param>
        static public int[] FindTheMaximumSpan(DataTable dtCharts, int chartRowIdxForRowSpan, int chartColIdxForRowSpan, int chartRowIdxForColSpan, int chartColIdxForColSpan)
        {
            // array to keep the maximum no of columns and no of rows
            int maximumColSpan = 0;
            int maximumRowSpan = 0;
            int[] array = new int[2];

            maximumRowSpan = ((ChartData)dtCharts.Rows[chartRowIdxForRowSpan][chartColIdxForRowSpan]).RowSpan;
            maximumColSpan = ((ChartData)dtCharts.Rows[chartRowIdxForColSpan][chartColIdxForColSpan]).ColSpan;

            array[0] = maximumRowSpan;
            array[1] = maximumColSpan;
            return array;
        }

        /// <summary>
        /// CheckIfChartAtPosition
        /// </summary>
        /// <param name="idxI"></param>
        /// <param name="idxJ"></param>
        /// <param name="dtChart"></param>
        /// <returns></returns>
        static public string CheckIfChartAtPosition(int idxI, int idxJ, DataTable dtChart)
        {
            string chartURL = "";
            bool found = false;

            foreach (DataRow dr in dtChart.Rows)
            {
                for (int cols = 0; cols < dtChart.Columns.Count; cols++)
                {
                    if (dr[cols].ToString() != "")
                    {
                        ChartData chDat = ((ChartData)(dr[cols]));
                        int positionRow = chDat.RowPosition;
                        int positionCol = chDat.ColPosition;
                        if (positionRow == idxI && positionCol == idxJ)
                        {
                            chartURL = chDat.ChartURL;
                            found = true;
                            break;
                        }
                    }
                }

                if (found) break;
            }

            return chartURL;

        }

        /// <summary>
        /// CheckIfChartAtPosition
        /// </summary>
        /// <param name="idxI"></param>
        /// <param name="idxJ"></param>
        /// <param name="dtChart"></param>
        /// <returns></returns>
        static public ChartData FindChartByURL(string URL, DataTable dtChart)
        {
            ChartData chart = new ChartData();
            bool found = false;

            foreach (DataRow dr in dtChart.Rows)
            {
                for (int cols = 0; cols < dtChart.Columns.Count; cols++)
                {
                    if (dr[cols].ToString() != "")
                    {
                        ChartData chDat = ((ChartData)(dr[cols]));
                        string chartURL = chDat.ChartURL;
                        if (chartURL == URL)
                        {
                            chart.ChartHeight = chDat.ChartHeight;
                            chart.ChartURL = chDat.ChartURL;
                            chart.ChartWidth = chDat.ChartWidth;
                            chart.ColPosition = chDat.ColPosition;
                            chart.RowPosition = chDat.RowPosition;
                            chart.ColSpan = chDat.ColSpan;
                            chart.RowSpan = chDat.RowSpan;
                            found = true;
                            break;
                        }
                    }
                }

                if (found) break;
            }

            return chart;

        }

        /// <summary>
        /// FindTheLastRowAndColumn
        /// </summary>
        /// <param name="dtCharts"></param>
        static public DataTable CreateOne(DataTable dtValue, DataTable dtCharts, int cellLeftCornerX, int cellLeftCornerY)
        {
            DataTable dtTogether = new DataTable();

            int[] RowColLast = ChartData.FindTheLastRowAndColumn(dtCharts);
            int chartRowIdxForRowSpan = RowColLast[1];
            int chartColIdxForRowSpan = RowColLast[2];
            int chartRowIdxForColSpan = RowColLast[4];
            int chartColIdxForColSpan = RowColLast[5];
            int[] RowColSpan = ChartData.FindTheMaximumSpan(dtCharts, chartRowIdxForRowSpan, chartColIdxForRowSpan, chartRowIdxForColSpan, chartColIdxForColSpan);

            int maxChartRow = RowColLast[0];
            int maxChartCol = RowColLast[3];
            int maxChartRowSpan = RowColSpan[0];
            int maxChartColSpan = RowColSpan[1];

            //if (dtValue.Columns.Count > 0)
            //{
                maxChartCol += maxChartColSpan;
                maxChartRow += maxChartRowSpan;
            //}
            int maxValueRow = dtValue.Rows.Count;
            int maxValueCol = dtValue.Columns.Count;
            int maxCols = (maxChartCol > maxValueCol ? maxChartCol : maxValueCol);
            int maxRows = (maxChartRow > maxValueRow ? maxChartRow : maxValueRow);

            // create an empty datatable 
            for (int idx = cellLeftCornerX; idx < maxCols; idx++)
            {
                //Create a new column for each column in the excel file
                DataColumn dcol = new DataColumn();  // give header
                dtTogether.Columns.Add(dcol);
            }

            for (int i = cellLeftCornerY; i < maxRows; i++)
            {
                DataRow drRow = dtTogether.NewRow();
                for (int j = cellLeftCornerX; j < maxCols; j++)
                {
                    string chartURL = "";
                    chartURL = CheckIfChartAtPosition(i, j, dtCharts);
                    if (chartURL != "")
                    {
                        drRow[j - cellLeftCornerX] = chartURL;
                    }
                    else
                    {
                        if (i - cellLeftCornerY < dtValue.Rows.Count && j - cellLeftCornerX < dtValue.Columns.Count)
                        { drRow[j - cellLeftCornerX] = dtValue.Rows[i - cellLeftCornerY][j - cellLeftCornerX]; }
                        else
                        { drRow[j - cellLeftCornerX] = ""; }
                    }

                }

                dtTogether.Rows.Add(drRow);
            }

            return dtTogether;

        }

        /// <summary>
        /// GetChartColSpan
        /// </summary>
        /// <returns></returns>
        public int GetChartColSpan(DataTable dtStyles, int rowNo, int colNo, ExcelTable xlsTable)
        {
            double chrtWidth = this.ChartWidth;
            int idx = 0;
            int noOfMergedCols = 0;

            while (chrtWidth >= 0)
            {
                int cellWidth = emptyCellWidth;

                if (colNo + idx < dtStyles.Columns.Count && rowNo < dtStyles.Rows.Count)
                {
                    cellWidth = ((CellStyleData)dtStyles.Rows[rowNo][(colNo + idx)]).XlsWidth;
                }

                chrtWidth -= cellWidth;
                noOfMergedCols++;
                idx++;
            }

            int getNextCellWidth = emptyCellWidth;
            if (colNo + idx -1 < dtStyles.Columns.Count && rowNo < dtStyles.Rows.Count)
            {
                getNextCellWidth = ((CellStyleData)dtStyles.Rows[rowNo][(colNo + idx - 1)]).XlsWidth;
            }

            if((-1) * chrtWidth <getNextCellWidth - 0.8*getNextCellWidth)
            {
                noOfMergedCols++;
            }

            if (rowNo < dtStyles.Rows.Count && colNo < dtStyles.Columns.Count)
            {
                // check the next row cell
                for (int rangeIdx = 0; rangeIdx < xlsTable.MergeCellRangesCount(); rangeIdx++)
                {
                    if (xlsTable.easy_getCellMergingFirstCol(rangeIdx) == colNo && xlsTable.easy_getCellMergingFirstRow(rangeIdx) == rowNo)
                        return xlsTable.easy_getCellMergingLastCol(rangeIdx) - xlsTable.easy_getCellMergingFirstCol(rangeIdx) + 1;
                }
            }

           

            return noOfMergedCols-1;
        }

        /// <summary>
        /// GetChartRowSpan
        /// </summary>
        /// <returns></returns>
        public int GetChartRowSpan(DataTable dtStyles, int rowNo, int colNo, ExcelTable xlsTable)
        {
            double chrtHeight = this.ChartHeight;
            int idx = 0;
            int noOfMergedRows = 0;

            while (chrtHeight > 0)
            {
                int cellHeight = emptyCellHeight;

                if (rowNo + idx < dtStyles.Rows.Count && colNo < dtStyles.Columns.Count)
                {
                    cellHeight = ((CellStyleData)dtStyles.Rows[rowNo + idx][0]).XlsHeight;
                }

                chrtHeight -= cellHeight;
                noOfMergedRows++;
                idx++;
            }

            //int getNextCellHeight = emptyCellHeight;
            //if (rowNo + idx - 1 < dtStyles.Rows.Count && colNo < dtStyles.Columns.Count)
            //{
            //    getNextCellHeight = ((CellStyleData)dtStyles.Rows[rowNo + idx - 1][0]).XlsHeight;
            //}

            //if ((-1) * chrtHeight < getNextCellHeight - 0.95 * getNextCellHeight)
            //{
            //    noOfMergedRows++;
            //}

            if (rowNo < dtStyles.Rows.Count && colNo < dtStyles.Columns.Count)
            {
                // check the next row cell
                for (int rangeIdx = 0; rangeIdx < xlsTable.MergeCellRangesCount(); rangeIdx++)
                {
                    if (xlsTable.easy_getCellMergingFirstCol(rangeIdx) == colNo && xlsTable.easy_getCellMergingFirstRow(rangeIdx) == rowNo)
                        return xlsTable.easy_getCellMergingLastRow(rangeIdx) - xlsTable.easy_getCellMergingFirstRow(rangeIdx) + 1;
                }
            }
 
            return noOfMergedRows;
        }
    }
}
