﻿/*
 Copyright (c) 2012-2014 EU Outsourcing, Inc
All rights reserved.  http://www.euoutsourcing.com
Version: 2.1.0
Disclaimer: 
Any redistribution, modification and use of this software either in source or binary forms is permitted provided that the following terms are met: 
1. This software’s source code is not modified to avoid the use of EasyXLS™. 
2.  All servers and computers, whether physical or virtual, using this software or its derivatives must be properly licensed to use EasyXLS™.
3. Any derivatives of this source code that is redistributed must include the above copyright notice, these five conditions and the included disclaimer. 
4. Any redistributions in binary form must include the above copyright notice, these five conditions and the included disclaimer in any materials that are provided with the distribution. 
5. EasyXLS™ and the name of its parent organization may not be used to endorse or promote any software products or derivative works without specific prior written permission from the copyright holders.
DISCLAIMER.  THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using EasyXLS;
using EasyXLS.Constants;
using System.Drawing;
using System.Data;
using EasyXLS.Util;
using System.Web.UI.WebControls;
using EasyXLSDemoApp.EasyXLSGrid;
using EasyXLS.Themes;

namespace EasyXLSDemoApp.EasyXLSData
{
    // class for storing cell border styles, width, alignment, font settings methods
    public class EasyXLSGridBorder
    {

        #region [ Color vs Theme Color ]

        /// <summary>
        /// getObjectThemeVSForeground
        /// </summary>
        /// <returns></returns>
        public static Color getObjectThemeVSNormalColor(ThemeColor themeColor, Color color, ExcelTheme excelTheme)
        {
            Color automaticColor = Color.Black;

            Color xlsColor = automaticColor;

            // if theme color does not exist
            if (themeColor == null)
            {
                // get normal foreground
                xlsColor = color;

                // if foreground is empty return black
                if (xlsColor.IsEmpty) xlsColor = automaticColor;
            }
            else
            {
                // get the color from the theme
                xlsColor = themeColor.getColorForTheme(excelTheme);
            }

            return xlsColor;
        }

        /// <summary>
        /// getObjectThemeVSForeground
        /// </summary>
        /// <returns></returns>
        public static Color getFillBorderThemeVSNormalColor(ThemeColor themeColor, Color color, ExcelTheme excelTheme)
        {
            Color xlsColor = Color.Empty;

            // if theme color does not exist
            if (themeColor == null)
            {
                // get normal foreground
                xlsColor = color;
            }
            else
            {
                // get the color from the theme
                xlsColor = themeColor.getColorForTheme(excelTheme);
            }

            return xlsColor;
        }


        #endregion

        //not keep
        #region [ Set Border Color and Style ]

        /// <summary>
        /// GetTheBorderStyle
        /// </summary>
        /// <returns></returns>
        static string GetTheBorderStyle(int styleBorder, string placement)
        {

            // init border style
            string borderStyle = "";

            // convert xls border style int to BorderStyle object
            switch (styleBorder)
            {
                // Solid
                case 0: borderStyle = "solid"; break;
                case 1: borderStyle = "solid; border-" + placement + "-width: 1px"; break;
                case 2: borderStyle = "solid; border-" + placement + "-width: 2px"; break;  // border medium
                // Dashed
                case 3: borderStyle = "dashed; border-" + placement + "-width: 2px"; break;
                case 8: borderStyle = "dashed; border-" + placement + "-width: 3px"; break;
                case 9: borderStyle = "dashed; border-" + placement + "-width: 2px"; break;
                case 11: borderStyle = "dashed; border-" + placement + "-width: 2px"; break;
                // Dotted
                case 4: borderStyle = "dotted; border-" + placement + "-width: 2px"; break;
                case 7: borderStyle = "dotted; border-" + placement + "-width: 1px"; break;
                // Solid
                case 5: borderStyle = "solid; border-" + placement + "-width: 3px"; break;  //border thick
                // Double
                case 6: borderStyle = "double; border-" + placement + "-width: 3px"; break;
                // default
                default: borderStyle = "dashed; border-" + placement + "-width: 3px"; break;
            }

            //return
            return borderStyle;
        }

        /// <summary>
        /// Set Border Color and Style
        /// </summary>
        /// <param name="grvXLS"></param>
        /// <param name="grRow"></param>
        /// <param name="style"></param>
        /// <param name="cellIdx"></param>
        /// <param name="rowNo"></param>
        /// <param name="rowOnTable"></param>
        /// <param name="mergeCellsNo"></param>
        public static string SetBorderColorStyle(GridView grvXLS, GridViewRow grRow, ExcelStyle style, ExcelStyle beforeMergeStyle, int cellIdx, int rowNo, int rowOnTable, int mergeCellsNo, int thisMergeCol, int thisMergeRow, DataTable dtStyles, int rowMergeCell, Dictionary<int, List<EasyXLSDemoApp.EasyXLSData.EasyXLSGridData.CellSpan>> rowMergedCells, ExcelTheme xlsTheme)
        {
            // set attributes for all border colors and styles
            string cellAttributes = "";

            // no merging cells or rows
            if (thisMergeCol <= 0 && thisMergeRow <= 0)
            {
                // border color (left color and style will be copied for the whole grid view cell)
                Color basicLeftColorBorder = beforeMergeStyle.getLeftColor_Border();
                ThemeColor themeLeftColorBorder = beforeMergeStyle.getLeftThemeColor_Border();
                Color leftColorBorder = getObjectThemeVSNormalColor(themeLeftColorBorder, basicLeftColorBorder, xlsTheme);

                Color basicRightColorBorder = style.getRightColor_Border();
                ThemeColor themeRightColorBorder = style.getRightThemeColor_Border();
                Color rightColorBorder = getObjectThemeVSNormalColor(themeRightColorBorder, basicRightColorBorder, xlsTheme);

                Color basicTopColorBorder = beforeMergeStyle.getTopColor_Border();
                ThemeColor themeTopColorBorder = beforeMergeStyle.getTopThemeColor_Border();
                Color topColorBorder = getObjectThemeVSNormalColor(themeTopColorBorder, basicTopColorBorder, xlsTheme);

                Color basicBottomColorBorder = beforeMergeStyle.getBottomColor_Border();
                ThemeColor themeBottomColorBorder = beforeMergeStyle.getBottomThemeColor_Border();
                Color bottomColorBorder = getObjectThemeVSNormalColor(themeBottomColorBorder, basicBottomColorBorder, xlsTheme);

                Color fillColor = getFillBorderThemeVSNormalColor(style.getBackgroundThemeColor(), style.getBackground(), xlsTheme);

                string leftBorderStyle = GetTheBorderStyle(beforeMergeStyle.getLeftStyle_Border(), "left");
                string rightBorderStyle = GetTheBorderStyle(style.getRightStyle_Border(), "right");
                string topBorderStyle = GetTheBorderStyle(beforeMergeStyle.getTopStyle_Border(), "top");
                string bottomBorderStyle = GetTheBorderStyle(beforeMergeStyle.getBottomStyle_Border(), "bottom");

                // check if first cell in the row and left style set
                if (cellIdx > 0 && beforeMergeStyle.getLeftStyle_Border() != 0)
                {
                    // check the left cell to see if can rewrite the style
                    CheckLeftCell(grvXLS, grRow, rowNo, cellIdx - 1, rowOnTable, mergeCellsNo, leftColorBorder, GetTheBorderStyle(beforeMergeStyle.getLeftStyle_Border(), "right"), thisMergeRow, thisMergeCol, dtStyles, xlsTheme);
                }
                else
                {
                    if (beforeMergeStyle.getLeftStyle_Border() == 0 && !fillColor.IsEmpty && cellIdx > 0)
                    {
                        // get left cell style based on cell and offset
                        ExcelStyle previousLeftStyle = ((CellStyleData)dtStyles.Rows[rowNo + rowOnTable][(cellIdx -1 + mergeCellsNo - thisMergeCol)]).XlsStyle;
                        if (previousLeftStyle != null)
                        {
                            if (previousLeftStyle.getRightStyle_Border() == 0)
                            {

                                string previousCellAttributes = grRow.Cells[cellIdx - 1].Attributes["style"];

                                previousCellAttributes += "border-right-color:rgb(" + fillColor.R + "," + fillColor.G + "," + fillColor.B + ");border-right-style:1px;";

                                // add rest of attributes
                                grRow.Cells[cellIdx - 1].Attributes["style"] = previousCellAttributes;
                            }
                        }
                    }
                }

                // check if first row in the gridview and top style set
                if (rowNo > 0 && beforeMergeStyle.getTopStyle_Border() != 0)
                {
                    // check the top cell to see if can rewrite the style
                    CheckTopCell(grvXLS, rowNo - 1, cellIdx, rowOnTable, mergeCellsNo, topColorBorder, GetTheBorderStyle(beforeMergeStyle.getTopStyle_Border(), "bottom"), thisMergeCol, dtStyles, rowMergedCells[rowNo - 1], xlsTheme);
                }
                else
                {
                    if (beforeMergeStyle.getTopStyle_Border() == 0 && !fillColor.IsEmpty && rowNo - 1 > 0 && rowNo - 1 < grvXLS.Rows.Count && cellIdx < grvXLS.Rows[rowNo - 1].Cells.Count)
                    {
                         // get left cell style based on cell and offset
                        ExcelStyle previousTopStyle = ((CellStyleData)dtStyles.Rows[rowNo + rowOnTable -1][(cellIdx + mergeCellsNo - thisMergeCol)]).XlsStyle;
                        if (previousTopStyle != null)
                        {
                            string previousTopCellAttributes = grvXLS.Rows[rowNo - 1].Cells[cellIdx].Attributes["style"];
                            if (previousTopCellAttributes != null)
                            {
                                if (previousTopStyle.getBottomStyle_Border() == 0 && !previousTopCellAttributes.Contains("border-bottom"))
                                {
                                    previousTopCellAttributes += "border-bottom-color:rgb(" + fillColor.R + "," + fillColor.G + "," + fillColor.B + ");border-bottom-style:1px;";

                                    // add rest of attributes
                                    grvXLS.Rows[rowNo - 1].Cells[cellIdx].Attributes["style"] = previousTopCellAttributes;
                                }
                            }
                        }
                    }
                }

                // if no style then set a grey border on each part

                // left border color & style
                if (beforeMergeStyle.getLeftStyle_Border() != 0)
                {
                    cellAttributes += "border-left-color:rgb(" + leftColorBorder.R + "," + leftColorBorder.G + "," + leftColorBorder.B + ");border-left-style:" + leftBorderStyle + ";";
                }
                else
                {
                    if (!fillColor.IsEmpty)
                    {
                        cellAttributes += "border-left-color:rgb(" + fillColor.R + "," + fillColor.G + "," + fillColor.B + ");border-left-style:1px;";
                    }
                }

                // right border color & style
                if (style.getRightStyle_Border() != 0)
                {
                    cellAttributes += "border-right-color:rgb(" + rightColorBorder.R + "," + rightColorBorder.G + "," + rightColorBorder.B + ");border-right-style:" + rightBorderStyle + ";";
                }
                else
                {
                    if (!fillColor.IsEmpty)
                    {
                        cellAttributes += "border-right-color:rgb(" + fillColor.R + "," + fillColor.G + "," + fillColor.B + ");border-right-style:1px;";
                    }
                }

                // top border color & style
                if (beforeMergeStyle.getTopStyle_Border() != 0)
                {
                    cellAttributes += "border-top-color:rgb(" + topColorBorder.R + "," + topColorBorder.G + "," + topColorBorder.B + ");border-top-style:" + topBorderStyle + ";";
                }
                else
                {
                    if (!fillColor.IsEmpty)
                    {
                        cellAttributes += "border-top-color:rgb(" + fillColor.R + "," + fillColor.G + "," + fillColor.B + ");border-top-style:1px;";
                    }
                }

                // bottom border color & style
                if (beforeMergeStyle.getBottomStyle_Border() != 0)
                {
                    cellAttributes += "border-bottom-color:rgb(" + bottomColorBorder.R + "," + bottomColorBorder.G + "," + bottomColorBorder.B + ");border-bottom-style:" + bottomBorderStyle + ";";
                }
                else
                {
                    if (!fillColor.IsEmpty)
                    {
                        cellAttributes += "border-bottom-color:rgb(" + fillColor.R + "," + fillColor.G + "," + fillColor.B + ");border-bottom-style:1px;";
                    }
                }
            }
            else
            {
                //get style for left border
                if (rowMergeCell > 0) { thisMergeCol = 0; thisMergeRow = 0; }

                if (
                    dtStyles.Rows.Count > rowNo + rowOnTable + thisMergeRow && rowNo + rowOnTable + thisMergeRow >= 0
                    && 
                    dtStyles.Columns.Count > cellIdx + mergeCellsNo - thisMergeCol && cellIdx + mergeCellsNo - thisMergeCol >= 0
                    )
                {
                    ExcelStyle leftTopStyle = ((CellStyleData)dtStyles.Rows[rowNo + rowOnTable][cellIdx + mergeCellsNo - thisMergeCol]).XlsStyle;
                    ExcelStyle rightStyle = ((CellStyleData)dtStyles.Rows[rowNo + rowOnTable][cellIdx + mergeCellsNo]).XlsStyle;
                    ExcelStyle bottomStyle = ((CellStyleData)dtStyles.Rows[rowNo + rowOnTable + thisMergeRow][cellIdx + mergeCellsNo]).XlsStyle;

                    // border color (left color and style will be copied for the whole grid view cell)
                    Color basicLeftColorBorder = leftTopStyle.getLeftColor_Border();
                    ThemeColor themeLeftColorBorder = leftTopStyle.getLeftThemeColor_Border();
                    Color leftColorBorder = getObjectThemeVSNormalColor(themeLeftColorBorder, basicLeftColorBorder, xlsTheme);

                    Color basicRightColorBorder = rightStyle.getRightColor_Border();
                    ThemeColor themeRightColorBorder = rightStyle.getRightThemeColor_Border();
                    Color rightColorBorder = getObjectThemeVSNormalColor(themeRightColorBorder, basicRightColorBorder, xlsTheme);

                    Color basicTopColorBorder = leftTopStyle.getTopColor_Border();
                    ThemeColor themeTopColorBorder = leftTopStyle.getTopThemeColor_Border();
                    Color topColorBorder = getObjectThemeVSNormalColor(themeTopColorBorder, basicTopColorBorder, xlsTheme);

                    Color basicBottomColorBorder = bottomStyle.getBottomColor_Border();
                    ThemeColor themeBottomColorBorder = bottomStyle.getBottomThemeColor_Border();
                    Color bottomColorBorder = getObjectThemeVSNormalColor(themeBottomColorBorder, basicBottomColorBorder, xlsTheme);

                    Color fillColor = getFillBorderThemeVSNormalColor(style.getBackgroundThemeColor(), style.getBackground(), xlsTheme);

                    // check if first row in the gridview and top style set
                    if (rowNo > 0 && leftTopStyle.getTopStyle_Border() != 0)
                    {
                        // check the top cell to see if can rewrite the style
                        CheckTopCell(grvXLS, rowNo - 1, cellIdx, rowOnTable, mergeCellsNo, topColorBorder, GetTheBorderStyle(leftTopStyle.getTopStyle_Border(), "bottom"), thisMergeCol, dtStyles, rowMergedCells[rowNo - 1], xlsTheme);
                    }
                    else
                    {
                        if (beforeMergeStyle.getTopStyle_Border() == 0 && !fillColor.IsEmpty && rowNo > 0)
                        {
                            // get left cell style based on cell and offset
                            ExcelStyle previousTopStyle = ((CellStyleData)dtStyles.Rows[rowNo + rowOnTable - 1][(cellIdx + mergeCellsNo - thisMergeCol)]).XlsStyle;
                            if (previousTopStyle != null)
                            {
                                if (previousTopStyle.getBottomStyle_Border() == 0)
                                {
                                    string previousTopCellAttributes = "";
                                    if (grvXLS.Rows[rowNo - 1].Cells[cellIdx].RowSpan > 1)
                                    {
                                        previousTopCellAttributes = grvXLS.Rows[rowNo - 1].Cells[cellIdx + mergeCellsNo - thisMergeCol].Attributes["style"];

                                        previousTopCellAttributes += "border-bottom-color:rgb(" + fillColor.R + "," + fillColor.G + "," + fillColor.B + ");border-bottom-style:1px;";

                                        // add rest of attributes
                                        grvXLS.Rows[rowNo - 1].Cells[cellIdx + mergeCellsNo - thisMergeCol].Attributes["style"] = previousTopCellAttributes;
                                    }
                                    else
                                    {
                                        previousTopCellAttributes = grvXLS.Rows[rowNo - 1].Cells[cellIdx].Attributes["style"];

                                        previousTopCellAttributes += "border-bottom-color:rgb(" + fillColor.R + "," + fillColor.G + "," + fillColor.B + ");border-bottom-style:1px;";

                                        // add rest of attributes
                                        grvXLS.Rows[rowNo - 1].Cells[cellIdx].Attributes["style"] = previousTopCellAttributes;
                                    }
                                }
                            }
                        }
                    }

                    // check if first cell in the row and left style set
                    if (cellIdx > 0 && leftTopStyle.getLeftStyle_Border() != 0)
                    {
                        // check the left cell to see if can rewrite the style
                        CheckLeftCell(grvXLS, grRow, rowNo, cellIdx - 1, rowOnTable, mergeCellsNo, leftColorBorder, GetTheBorderStyle(leftTopStyle.getLeftStyle_Border(), "right"), thisMergeRow, thisMergeCol, dtStyles, xlsTheme);
                    }
                    else
                    {
                        if (beforeMergeStyle.getLeftStyle_Border() == 0 && !fillColor.IsEmpty && cellIdx > 0)
                        {
                            // get left cell style based on cell and offset
                            ExcelStyle previousLeftStyle = ((CellStyleData)dtStyles.Rows[rowNo + rowOnTable][(cellIdx - 1 + mergeCellsNo - thisMergeCol)]).XlsStyle;
                            if (previousLeftStyle != null)
                            {
                                if (previousLeftStyle.getRightStyle_Border() == 0 && !previousLeftStyle.IsMergeCell())
                                {

                                    string previousCellAttributes = grRow.Cells[cellIdx - 1].Attributes["style"];

                                    previousCellAttributes += "border-right-color:rgb(" + fillColor.R + "," + fillColor.G + "," + fillColor.B + ");border-right-style:1px;";

                                    // add rest of attributes
                                    grRow.Cells[cellIdx - 1].Attributes["style"] = previousCellAttributes;
                                }
                            }
                        }
                    }

                    ExcelStyle nextCellLeftStyle = new ExcelStyle();

                    if (cellIdx + mergeCellsNo + 1 < dtStyles.Columns.Count)
                    {
                        // (if merged cell)check right cell style
                        nextCellLeftStyle = ((CellStyleData)dtStyles.Rows[rowNo + rowOnTable][cellIdx + mergeCellsNo + 1]).XlsStyle;
                    }

                    // left border color & style
                    if (leftTopStyle.getLeftStyle_Border() != 0)
                    {
                        cellAttributes += "border-left-color:rgb(" + leftColorBorder.R + "," + leftColorBorder.G + "," + leftColorBorder.B + ");border-left-style:" + GetTheBorderStyle(leftTopStyle.getLeftStyle_Border(), "left") + ";";
                    }
                    else
                    {
                        if (!fillColor.IsEmpty)
                        {
                            cellAttributes += "border-left-color:rgb(" + fillColor.R + "," + fillColor.G + "," + fillColor.B + ");border-left-style:1px;";
                        }
                    }

                    // right border color & style
                    if (rightStyle.getRightStyle_Border() != 0)
                    {
                        cellAttributes += "border-right-color:rgb(" + rightColorBorder.R + "," + rightColorBorder.G + "," + rightColorBorder.B + ");border-right-style:" + GetTheBorderStyle(rightStyle.getRightStyle_Border(), "right") + ";";
                    }
                    else
                    {
                        bool rightBorderSet = false;
                        if (nextCellLeftStyle != null)
                        {
                            if (nextCellLeftStyle.getLeftStyle_Border() != 0)
                            {
                                rightColorBorder = getObjectThemeVSNormalColor(nextCellLeftStyle.getLeftThemeColor_Border(), nextCellLeftStyle.getLeftColor_Border(), xlsTheme);
                                cellAttributes += "border-right-color:rgb(" + rightColorBorder.R + "," + rightColorBorder.G + "," + rightColorBorder.B + ");border-right-style:" + GetTheBorderStyle(nextCellLeftStyle.getLeftStyle_Border(), "right") + ";";
                                rightBorderSet = true;
                            }
                        }
                       if(!rightBorderSet)
                       {
                            if (!fillColor.IsEmpty)
                            {
                                cellAttributes += "border-right-color:rgb(" + fillColor.R + "," + fillColor.G + "," + fillColor.B + ");border-right-style:1px;";
                            }
                        }
                    }

                    // top border color & style
                    if (leftTopStyle.getTopStyle_Border() != 0)
                    {
                        cellAttributes += "border-top-color:rgb(" + topColorBorder.R + "," + topColorBorder.G + "," + topColorBorder.B + ");border-top-style:" + GetTheBorderStyle(leftTopStyle.getTopStyle_Border(), "top") + ";";
                    }
                    else
                    {
                        if (!fillColor.IsEmpty)
                        {
                            cellAttributes += "border-top-color:rgb(" + fillColor.R + "," + fillColor.G + "," + fillColor.B + ");border-top-style:1px;";
                        }
                    }

                    // bottom border color & style
                    if (bottomStyle.getBottomStyle_Border() != 0)
                    {
                        cellAttributes += "border-bottom-color:rgb(" + bottomColorBorder.R + "," + bottomColorBorder.G + "," + bottomColorBorder.B + ");border-bottom-style:" + GetTheBorderStyle(bottomStyle.getBottomStyle_Border(), "bottom") + ";";
                    }
                    else
                    {
                        if (!fillColor.IsEmpty)
                        {
                            cellAttributes += "border-bottom-color:rgb(" + fillColor.R + "," + fillColor.G + "," + fillColor.B + ");border-bottom-style:1px;";
                        }
                    }
                }
            }

            // set attributes
            return cellAttributes;
        }

        /// <summary>
        /// Check the cell in the left
        /// </summary>
        /// <param name="grRow"></param>
        /// <param name="rowNo"></param>
        /// <param name="i"></param>
        /// <param name="rowOnGridView"></param>
        /// <param name="formatNo"></param>
        /// <param name="rightColorBorder"></param>
        /// <param name="rightBorderStyle"></param>
        static void CheckLeftCell(GridView grvXLS, GridViewRow grRow, int rowNo, int colIdx, int rowOnGridView, int formatNo, Color rightColorBorder, string rightBorderStyle, int thisMergeRow, int thisMergeCol, DataTable dtStyles, ExcelTheme xlsTheme)
        {

            if (thisMergeRow == 0)
            {
                // get left cell style based on cell and offset
                ExcelStyle style = ((CellStyleData)dtStyles.Rows[rowNo + rowOnGridView][(colIdx + formatNo - thisMergeCol)]).XlsStyle;
                if (style != null)
                {
                    if (style.getRightStyle_Border() == 0 && !style.IsMergeCell())
                    {
                        if (grRow.Cells.Count > colIdx)
                        {

                            string cellAttributes = grRow.Cells[colIdx].Attributes["style"];

                            cellAttributes += "border-right-color:rgb(" + rightColorBorder.R + "," + rightColorBorder.G + "," + rightColorBorder.B + ");border-right-style:" + rightBorderStyle + ";";

                            // add rest of attributes
                            grRow.Cells[colIdx].Attributes["style"] = cellAttributes;
                        }
                    }
                }
            }
            else
            {
                
                    // get topcell based on cell and offse
                    ExcelStyle currentStyle = ((CellStyleData)dtStyles.Rows[rowNo + rowOnGridView][(colIdx + formatNo - thisMergeCol)]).XlsStyle;
                    if (currentStyle.getRightStyle_Border() == 0)
                    {
                        if (grRow.Cells.Count > colIdx)
                        {
                            string cellAttributes = grRow.Cells[colIdx].Attributes["style"];

                            cellAttributes += "border-right-color:rgb(" + rightColorBorder.R + "," + rightColorBorder.G + "," + rightColorBorder.B + ");border-right-style:" + rightBorderStyle + ";";

                            if (colIdx < grvXLS.Columns.Count)
                            {
                                // add rest of attributes
                                grRow.Cells[colIdx].Attributes["style"] = cellAttributes;
                            }
                        }
                    }
                    
            }
           
        }


        /// <summary>
        /// Check the cell above 
        /// </summary>
        /// <param name="gridView"></param>
        /// <param name="rowNo"></param>
        /// <param name="i"></param>
        /// <param name="rowOnGridView"></param>
        /// <param name="formatNo"></param>
        /// <param name="bottomColorBorder"></param>
        /// <param name="bottomBorderStyle"></param>
        static void CheckTopCell(GridView gridView, int rowNo, int colIdx, int rowOnGridView, int formatNo, Color bottomColorBorder, string bottomBorderStyle, int thisMergeCol, DataTable dtStyles, List<EasyXLSDemoApp.EasyXLSData.EasyXLSGridData.CellSpan> rowMergedCells, ExcelTheme xlsTheme)
        {
            int noOfPreviousRowMergedCells = 0;
            int noOfPreviousRowMergedStartCellIdx = 0;
            int prevCellIdx = rowMergedCells.FindLast(x => x.cellIndex <= colIdx).cellIndex;
            noOfPreviousRowMergedCells = rowMergedCells.FindLast(x => x.cellIndex <= colIdx).columnSpan;

            if (prevCellIdx < 0 || colIdx > prevCellIdx) prevCellIdx = colIdx;

            noOfPreviousRowMergedStartCellIdx += prevCellIdx;

            int noOfCurrentRowMergedCells = 0;
            noOfCurrentRowMergedCells = formatNo - thisMergeCol;

            int noOfPreviousRowSpan = 0;
            int prevRowSpan = rowMergedCells.FindLast(x => x.cellIndex <= colIdx).rowSpan;
            if (prevRowSpan > 0) prevRowSpan--;

            noOfPreviousRowSpan = prevRowSpan;
            rowNo -= noOfPreviousRowSpan;

            int actualColumnNo = noOfPreviousRowMergedStartCellIdx + (noOfCurrentRowMergedCells - noOfPreviousRowMergedCells);

            if (actualColumnNo + noOfPreviousRowMergedCells < dtStyles.Columns.Count && actualColumnNo + noOfPreviousRowMergedCells >= 0 && rowNo + rowOnGridView < dtStyles.Rows.Count)
            {
                if (thisMergeCol == 0)
                {
                    // get topcell based on cell anf offse
                    ExcelStyle style = ((CellStyleData)dtStyles.Rows[rowNo + rowOnGridView][(actualColumnNo + noOfPreviousRowMergedCells)]).XlsStyle;
                    if (style.getBottomStyle_Border() == 0)
                    {
                        if (gridView.Rows.Count > rowNo)
                        {
                            if (gridView.Rows[rowNo].Cells.Count > actualColumnNo && actualColumnNo >= 0)
                            {
                                string cellAttributes = gridView.Rows[rowNo].Cells[actualColumnNo].Attributes["style"];

                                cellAttributes += "border-bottom-color:rgb(" + bottomColorBorder.R + "," + bottomColorBorder.G + "," + bottomColorBorder.B + ");border-bottom-style:" + bottomBorderStyle + ";";

                                if (rowNo < gridView.Rows.Count)
                                {
                                    if (actualColumnNo < gridView.Rows[rowNo].Cells.Count)
                                    {
                                        // add rest of attributes
                                        gridView.Rows[rowNo].Cells[actualColumnNo].Attributes["style"] = cellAttributes;
                                    }
                                }
                            }
                        }
                    }
                }
                else
                {
                    int getNoOfMergedCells = thisMergeCol;
                    while (getNoOfMergedCells >= 0)

                    {
                        // get topcell based on cell anf offse
                        ExcelStyle currentStyle = ((CellStyleData)dtStyles.Rows[rowNo + rowOnGridView][(actualColumnNo + noOfPreviousRowMergedCells + getNoOfMergedCells)]).XlsStyle;
                        if (currentStyle.getBottomStyle_Border() == 0)
                        {
                            if (gridView.Rows.Count > rowNo)
                            {
                                if (gridView.Rows[rowNo].Cells.Count > actualColumnNo + getNoOfMergedCells && actualColumnNo + getNoOfMergedCells >= 0)
                                {

                                    string cellAttributes = gridView.Rows[rowNo].Cells[actualColumnNo + getNoOfMergedCells].Attributes["style"];

                                    cellAttributes += "border-bottom-color:rgb(" + bottomColorBorder.R + "," + bottomColorBorder.G + "," + bottomColorBorder.B + ");border-bottom-style:" + bottomBorderStyle + ";";

                                    if (actualColumnNo + getNoOfMergedCells < gridView.Rows[rowNo].Cells.Count)
                                    {
                                        // add rest of attributes
                                        gridView.Rows[rowNo].Cells[actualColumnNo + getNoOfMergedCells].Attributes["style"] = cellAttributes;
                                    }
                                }
                            }
                        }
                        getNoOfMergedCells--;
                    }
                }
            }
        }

        #endregion

        //not keep
        #region [ Set Alignment ]

        /// <summary>
        /// Set Horizontal and Vertical Alignment
        /// </summary>
        /// <param name="grRow"></param>
        /// <param name="style"></param>
        /// <param name="cellIdx"></param>
        public static string SetAlignment(GridViewRow grRow, ExcelStyle style, int cellIdx, string dataType)
        {
            string attributes = "text-align:right;vertical-align:bottom;";

            // horizontal alignment
            string horAlign = style.getHorizontalAlignment();

            // Center
            if (horAlign.ToLower() == HorizontalAlign.Center.ToString().ToLower())
            {
                attributes = "text-align:center;";
                // grRow.Cells[cellIdx].HorizontalAlign = HorizontalAlign.Center;
            }

            // Justify
            if (horAlign.ToLower() == HorizontalAlign.Justify.ToString().ToLower())
            {
                attributes = "text-align:justify;";
                //grRow.Cells[cellIdx].HorizontalAlign = HorizontalAlign.Justify;
            }

            //Left
            if (horAlign.ToLower() == HorizontalAlign.Left.ToString().ToLower() || horAlign.ToLower() == "general")
            {
                attributes = "text-align:left;";
                //grRow.Cells[cellIdx].HorizontalAlign = HorizontalAlign.Left;
            }

            // Right
            if (horAlign.ToLower() == HorizontalAlign.Right.ToString().ToLower())
            {
                attributes = "text-align:right;";
                //   grRow.Cells[cellIdx].HorizontalAlign = HorizontalAlign.Right;
            }


            if (horAlign.ToLower() == "general")
            {
                if (dataType == "numeric" || dataType == "date")
                {
                    // align text to the right
                    attributes = "text-align:right;";
                    //grRow.Cells[cellIdx].HorizontalAlign = HorizontalAlign.Right;
                }

                string format = style.getFormat();

                if (format.Contains("%") || format.Contains("$"))
                {
                    // align text to the right
                    attributes = "text-align:right;";
                    //grRow.Cells[cellIdx].HorizontalAlign = HorizontalAlign.Right;
                }
                
                if (format.Contains("0")) attributes = "text-align:right;";

                int intResult; double doubleResult;
                if (int.TryParse(grRow.Cells[cellIdx].Text, out intResult) || double.TryParse(grRow.Cells[cellIdx].Text, out doubleResult))
                {
                    // align text to the right
                    attributes = "text-align:right;";
                }

                 if (dataType == "string")
                {
                    // align text to the right
                    attributes = "text-align:left;";
                    //grRow.Cells[cellIdx].HorizontalAlign = HorizontalAlign.Right;
                }
            }
            // vertical alignment
            string vertAlign = style.getVerticalAlignment();

            //Bottom
            if (vertAlign.ToLower() == VerticalAlign.Bottom.ToString().ToLower())
            {
                attributes += "vertical-align:bottom;";
                //grRow.Cells[cellIdx].VerticalAlign = VerticalAlign.Bottom;
            }


            //Middle
            if (vertAlign.ToLower() == VerticalAlign.Middle.ToString().ToLower())
            {
                attributes += "vertical-align:middle;";
                //grRow.Cells[cellIdx].VerticalAlign = VerticalAlign.Middle;
            }

            // Not Set
            if (vertAlign.ToLower() == VerticalAlign.NotSet.ToString().ToLower())
            {
                attributes += "vertical-align:notset;";
                //grRow.Cells[cellIdx].VerticalAlign = VerticalAlign.NotSet;
            }

            // Top
            if (vertAlign.ToLower() == VerticalAlign.Top.ToString().ToLower())
            {
                attributes += "vertical-align:top;";
                //grRow.Cells[cellIdx].VerticalAlign = VerticalAlign.Top;
            }

            return attributes;
        }

        #endregion

        //not keep
        #region [ Set Font Attributes ]

        /// <summary>
        /// SetFontAttributes
        /// </summary>
        /// <param name="grRow"></param>
        /// <param name="style"></param>
        /// <param name="cellIdx"></param>
        public static string SetFontAttributes(GridViewRow grRow, ExcelStyle style, int cellIdx, bool isNumericRed, ExcelTheme xlsTheme)
        {
            string cellAttributes = "";
            // font name
            string fontName = style.getFont();
            if (style.IsBodyFont())
            {
                 // get appropriate font
                fontName = xlsTheme.getFontScheme().getBodyFont();
            }
            // if font name is HEADING scheme
            else if (style.IsHeadingsFont())
            {
                // get appropriate font
                fontName = xlsTheme.getFontScheme().getHeadingFont();
            }

            //grRow.Cells[cellIdx].Font.Name = fontName;
            cellAttributes += "font-family: " + fontName + ";";

            // font size
            int fontSize = style.getFontSize();
            //grRow.Cells[cellIdx].Font.Size = fontSize;
            cellAttributes += "font-size: " + fontSize + "pt;";

            // forecolor
            Color foreground = EasyXLSGridData.getObjectThemeVSNormalColor(style.getForegroundThemeColor(), style.getForeground(), xlsTheme, "foreground");
            //grRow.Cells[cellIdx].ForeColor = foreground;
            cellAttributes += "color: rgb(" + foreground.R + "," + foreground.G + "," + foreground.B + ");" ;

            // check if any negative numbers formatted with red
            if (isNumericRed)
            {
                //set the foreground color
                //grRow.Cells[cellIdx].ForeColor = Color.Red;
                cellAttributes += "color: rgb(" + Color.Red.R + "," + Color.Red.G + "," + Color.Red.B + ");";
            }


            // underline
            int underline = style.getUnderlineStyle();
            if (underline == 1)
            {
                //grRow.Cells[cellIdx].Font.Underline = true;
                cellAttributes += "text-decoration: underline;";
            }

            // Bold
            bool isBold = style.IsBold();
            if (isBold)
            {
                //grRow.Cells[cellIdx].Font.Bold = isBold;
                cellAttributes += "font-weight: bold;";
            }
            else
            {
                cellAttributes += "font-weight: normal;";
            }

            // Italic
            bool IsItalic = style.IsItalic();
            if (IsItalic)
            {
                //grRow.Cells[cellIdx].Font.Italic = IsItalic;
                cellAttributes += "font-style: italic;";
            }
            else
            {
                cellAttributes += "font-style: normal;";
            }

            // JustifyDistributed 
            //bool IsJustifyDistributed = style.IsJustifyDistributed();
            //if (IsJustifyDistributed)
            //    grRow.Cells[cellIdx].HorizontalAlign = HorizontalAlign.Justify;

            // Is Strikethrough
            bool IsStrikethrough = style.IsStrikethrough();
            if (IsStrikethrough)
            {
                //grRow.Cells[cellIdx].Font.Strikeout = IsStrikethrough;
                cellAttributes += "text-decoration: line-through;";
            }
            

            // Is Wrap
            bool IsWrap = style.IsWrap();
            if (IsWrap)
            {
                //grRow.Cells[cellIdx].Font.Strikeout = IsStrikethrough;
                cellAttributes += "white-space: normal;";
            }
            else
            {
                cellAttributes += "white-space: nowrap;";
            }
            
            //grRow.Cells[cellIdx].Wrap = IsWrap;

            return cellAttributes;
        }

        #endregion
    }
}