﻿/*
 Copyright (c) 2012-2014 EU Outsourcing, Inc
All rights reserved.  http://www.euoutsourcing.com
Version: 2.1.0
Disclaimer: 
Any redistribution, modification and use of this software either in source or binary forms is permitted provided that the following terms are met: 
1. This software’s source code is not modified to avoid the use of EasyXLS™. 
2.  All servers and computers, whether physical or virtual, using this software or its derivatives must be properly licensed to use EasyXLS™.
3. Any derivatives of this source code that is redistributed must include the above copyright notice, these five conditions and the included disclaimer. 
4. Any redistributions in binary form must include the above copyright notice, these five conditions and the included disclaimer in any materials that are provided with the distribution. 
5. EasyXLS™ and the name of its parent organization may not be used to endorse or promote any software products or derivative works without specific prior written permission from the copyright holders.
DISCLAIMER.  THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using EasyXLS;
using EasyXLS.Constants;
using System.Drawing;
using System.Data;
using EasyXLS.Util;
using System.Web.UI.WebControls;
using EasyXLSDemoApp.EasyXLSGrid;
using EasyXLS.Charts;
using EasyXLS.Themes;
using System.Web.UI.DataVisualization.Charting;
using EasyXLS.Drawings.Formatting;
using System.Web.UI.DataVisualization.Charting.Utilities;
using EasyXLS.Drawings;
using Org.BouncyCastle.Utilities.Collections;
using System.Drawing.Drawing2D;
using System.IO;

namespace EasyXLSDemoApp.EasyXLSData
{
    // class for storing chart methods
    public class EasyXLSGridChart
    {
        bool isXLSXFile = false;

        // axis label space in pixels
        const double AXIS_LABEL_SPACE_PX = 30;

        // constant array of posible automatic interval values
        int[] automaticInterval = new int[3] { 1, 2, 5};

        // constant size for automatic chart title
        const int CHART_TITLE_AUTOMATIC_FONT_SIZE = 18;

        // constant style for automatic chart title
        const FontStyle CHART_TITLE_AUTOMATIC_FONT_STYLE = FontStyle.Bold;

        // constants for width and height of empty cells
        public const int emptyCellWidth = 65;
        public const int emptyCellHeight = 21;

        public string UserSession = "";

        #region [ Get the EXCEL Document ]

        /// <summary>
        /// getExcelDocument
        /// </summary>
        /// <param name="pathToFile"></param>
        /// <returns></returns>
        public ExcelDocument getExcelDocument(string pathToFile, int SheetNumber)
        {
            // Load the xls file
            ExcelDocument myDocument = new ExcelDocument();
            // Load the xlsx file
            ExcelDocument myXLSXDocument = new ExcelDocument();

            isXLSXFile = false;
            try
            {
                // check if the file is a xls/xlsx document
                if (myDocument.easy_IsExcel(pathToFile) == true)
                {
                    // get the xls document
                    myDocument.easy_LoadXLSFile(pathToFile);
                }
                else
                {
                    // get the xlsx document
                    isXLSXFile = true;
                    myXLSXDocument.easy_LoadXLSXFile(pathToFile);
                }
            }
            catch (Exception){}

            // overwrite the documents
            if (isXLSXFile)
            { myDocument = myXLSXDocument; }

            return myDocument;
        }


        #endregion

        #region [ Settings Easy XLS to .NET Charts ]

        /// <summary>
        /// getChartType
        /// </summary>
        /// <param name="ExcelChartType"></param>
        /// <returns></returns>
        public SeriesChartType getChartType(int ExcelChartType)
        {
            SeriesChartType chartType = new SeriesChartType();

            switch (ExcelChartType)
            {
                case EasyXLS.Constants.Chart.CHART_TYPE_AREA: { chartType = SeriesChartType.Area; break; }                   /*CHART_TYPE_AREA*/
                case EasyXLS.Constants.Chart.CHART_TYPE_BUBBLE: { chartType = SeriesChartType.Bubble; break; }              /*CHART_TYPE_BUBBLE*/
                case EasyXLS.Constants.Chart.CHART_TYPE_BUBBLE_3D_EFFECT: { chartType = SeriesChartType.Bubble; break; }              /*CHART_TYPE_BUBBLE_3D*/
                case EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_CLUSTERED: { chartType = SeriesChartType.Column; break; }              /*CHART_TYPE_COLUMN_CLUSTERED */
                case EasyXLS.Constants.Chart.CHART_TYPE_DOUGHNUT: { chartType = SeriesChartType.Doughnut; break; }            /*CHART_TYPE_DOUGHNUT  */
                case EasyXLS.Constants.Chart.CHART_TYPE_DOUGHNUT_EXPLODED: { chartType = SeriesChartType.Doughnut; break; }            /*CHART_TYPE_DOUGHNUT_EXPLODED  */
                case EasyXLS.Constants.Chart.CHART_TYPE_PIE: { chartType = SeriesChartType.Pie; break; }                 /*CHART_TYPE_PIE*/
                case EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_COLUMN: { chartType = SeriesChartType.Column; break; }              /*CHART_TYPE_PYRAMID_COLUMN*/
                case EasyXLS.Constants.Chart.CHART_TYPE_RADAR: { chartType = SeriesChartType.Radar; break; }               /*CHART_TYPE_RADAR */
                case EasyXLS.Constants.Chart.CHART_TYPE_RADAR_MARKERS: { chartType = SeriesChartType.Radar; break; }               /*CHART_TYPE_RADAR_MARKERS */
                case EasyXLS.Constants.Chart.CHART_TYPE_RADAR_FILLED: { chartType = SeriesChartType.Radar; break; }               /*CHART_TYPE_RADAR_FILLED */
                case EasyXLS.Constants.Chart.CHART_TYPE_AREA_STACKED: { chartType = SeriesChartType.StackedArea; break; }         /*CHART_TYPE_AREA_STACKED*/
                case EasyXLS.Constants.Chart.CHART_TYPE_AREA_100_PERCENT_STACKED: { chartType = SeriesChartType.StackedArea100; break; }      /*CHART_TYPE_AREA_100_PERCENT_STACKED*/
                case EasyXLS.Constants.Chart.CHART_TYPE_CONE_BAR: { chartType = SeriesChartType.Bar; break; }                 /*CHART_TYPE_CONE_BAR*/
                case EasyXLS.Constants.Chart.CHART_TYPE_CONE_BAR_STACKED: { chartType = SeriesChartType.StackedBar; break; }          /*CHART_TYPE_CONE_BAR_STACKED*/
                case EasyXLS.Constants.Chart.CHART_TYPE_CONE_BAR_100_PERCENT_STACKED: { chartType = SeriesChartType.StackedBar100; break; }       /*CHART_TYPE_CONE_BAR_100_PERCENT_STACKED*/
                case EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_STACKED: { chartType = SeriesChartType.StackedColumn; break; }       /*CHART_TYPE_COLUMN_STACKED*/
                case EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_100_PERCENT_STACKED: { chartType = SeriesChartType.StackedColumn100; break; }    /*CHART_TYPE_COLUMN_100_PERCENT_STACKED*/

                case EasyXLS.Constants.Chart.CHART_TYPE_PIE_EXPLODED: { chartType = SeriesChartType.Pie; break; }             /* CHART_TYPE_PIE_EXPLODED */
                case EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_COLUMN: { chartType = SeriesChartType.Column; break; }          /* CHART_TYPE_CYLINDER_COLUMN */
                case EasyXLS.Constants.Chart.CHART_TYPE_LINE_3D: { chartType = SeriesChartType.Line; break; }            /* CHART_TYPE_LINE_3D */
                case EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_3D_CLUSTERED: { chartType = SeriesChartType.Column; break; }          /* CHART_TYPE_COLUMN_3D_CLUSTERED */
                case EasyXLS.Constants.Chart.CHART_TYPE_CONE_COLUMN: { chartType = SeriesChartType.Column; break; }          /* CHART_TYPE_CONE_COLUMN */
                case EasyXLS.Constants.Chart.CHART_TYPE_LINE: { chartType = SeriesChartType.Line; break; }            /* CHART_TYPE_LINE */
                case EasyXLS.Constants.Chart.CHART_TYPE_LINE_STACKED: { chartType = SeriesChartType.StackedArea; break; }     /* CHART_TYPE_LINE_STACKED */
                case EasyXLS.Constants.Chart.CHART_TYPE_SCATTER: { chartType = SeriesChartType.Point; break; }           /* CHART_TYPE_SCATTER */
                case EasyXLS.Constants.Chart.CHART_TYPE_LINE_100_PERCENT_STACKED: { chartType = SeriesChartType.StackedArea100; break; }  /* CHART_TYPE_LINE_100_PERCENT_STACKED */
                case EasyXLS.Constants.Chart.CHART_TYPE_LINE_MARKERS: { chartType = SeriesChartType.Line; break; }            /*CHART_TYPE_LINE_MARKERS*/
                case EasyXLS.Constants.Chart.CHART_TYPE_LINE_STACKED_MARKERS: { chartType = SeriesChartType.StackedArea; break; }     /*CHART_TYPE_LINE_STACKED_MARKERS*/
                case EasyXLS.Constants.Chart.CHART_TYPE_LINE_100_PERCENT_STACKED_MARKERS: { chartType = SeriesChartType.StackedArea100; break; }  /*CHART_TYPE_LINE_100_PERCENT_STACKED_MARKERS*/
                case EasyXLS.Constants.Chart.CHART_TYPE_SCATTER_SMOOTHED_LINES_MARKERS: { chartType = SeriesChartType.Spline; break; }          /*CHART_TYPE_SCATTER_SMOOTHED_LINES_MARKERS*/
                case EasyXLS.Constants.Chart.CHART_TYPE_SCATTER_SMOOTHED_LINES: { chartType = SeriesChartType.Spline; break; }          /*CHART_TYPE_SCATTER_SMOOTHED_LINES*/
                case EasyXLS.Constants.Chart.CHART_TYPE_SCATTER_LINES_MARKERS: { chartType = SeriesChartType.Line; break; }            /*CHART_TYPE_SCATTER_LINES_MARKERS*/
                case EasyXLS.Constants.Chart.CHART_TYPE_SCATTER_LINES: { chartType = SeriesChartType.Line; break; }            /*CHART_TYPE_SCATTER_LINES*/

                case EasyXLS.Constants.Chart.CHART_TYPE_BAR_CLUSTERED: { chartType = SeriesChartType.Bar; break; }             /*CHART_TYPE_BAR_CLUSTERED*/
                case EasyXLS.Constants.Chart.CHART_TYPE_BAR_STACKED: { chartType = SeriesChartType.StackedBar; break; }      /*CHART_TYPE_BAR_STACKED*/
                case EasyXLS.Constants.Chart.CHART_TYPE_BAR_100_PERCENT_STACKED: { chartType = SeriesChartType.StackedBar100; break; }   /*CHART_TYPE_BAR_100_PERCENT_STACKED*/
                case EasyXLS.Constants.Chart.CHART_TYPE_BAR_3D_CLUSTERED: { chartType = SeriesChartType.Bar; break; }             /*CHART_TYPE_BAR_3D_CLUSTERED*/
                case EasyXLS.Constants.Chart.CHART_TYPE_BAR_3D_STACKED: { chartType = SeriesChartType.StackedBar; break; }      /*CHART_TYPE_BAR_3D_STACKED*/
                case EasyXLS.Constants.Chart.CHART_TYPE_BAR_3D_100_PERCENT_STACKED: { chartType = SeriesChartType.StackedBar100; break; }   /*CHART_TYPE_BAR_3D_100_PERCENT_STACKED*/

                case EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_BAR: { chartType = SeriesChartType.Bar; break; }             /*CHART_TYPE_CYLINDER_BAR*/
                case EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_BAR_STACKED: { chartType = SeriesChartType.StackedBar; break; }      /*CHART_TYPE_CYLINDER_BAR_STACKED*/
                case EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_BAR_100_PERCENT_STACKED: { chartType = SeriesChartType.StackedBar100; break; }   /*CHART_TYPE_CYLINDER_BAR_100_PERCENT_STACKED*/
                case EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_COLUMN_3D: { chartType = SeriesChartType.Column; break; }          /*CHART_TYPE_CYLINDER_COLUMN_3D*/

                case EasyXLS.Constants.Chart.CHART_TYPE_BAR_OF_PIE: { chartType = SeriesChartType.Pie; break; } /* CHART_TYPE_BAR_OF_PIE */
                case EasyXLS.Constants.Chart.CHART_TYPE_PIE_OF_PIE: { chartType = SeriesChartType.Pie; break; } /* CHART_TYPE_PIE_OF_PIE */
                case EasyXLS.Constants.Chart.CHART_TYPE_PIE_3D: { chartType = SeriesChartType.Pie; break; } /* CHART_TYPE_PIE_3D */
                case EasyXLS.Constants.Chart.CHART_TYPE_PIE_3D_EXPLODED: { chartType = SeriesChartType.Pie; break; } /* CHART_TYPE_PIE_3D_EXPLODED */

                case EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_BAR: { chartType = SeriesChartType.Bar; break; }             /*CHART_TYPE_PYRAMID_BAR*/
                case EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_BAR_STACKED: { chartType = SeriesChartType.StackedBar; break; }      /*CHART_TYPE_PYRAMID_BAR_STACKED*/
                case EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_BAR_100_PERCENT_STACKED: { chartType = SeriesChartType.StackedBar100; break; }   /*CHART_TYPE_PYRAMID_BAR_100_PERCENT_STACKED*/

                case EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_3D_STACKED: { chartType = SeriesChartType.StackedColumn; break; }       /* CHART_TYPE_COLUMN_3D_STACKED */
                case EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_3D_100_PERCENT_STACKED: { chartType = SeriesChartType.StackedColumn100; break; }    /* CHART_TYPE_COLUMN_3D_100_PERCENT_STACKED */
                case EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_3D: { chartType = SeriesChartType.Column; break; }              /* CHART_TYPE_COLUMN_3D */

                case EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_COLUMN_STACKED: { chartType = SeriesChartType.StackedColumn; break; }       /* CHART_TYPE_CYLINDER_COLUMN_STACKED */
                case EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_COLUMN_100_PERCENT_STACKED: { chartType = SeriesChartType.StackedColumn100; break; }    /* CHART_TYPE_CYLINDER_COLUMN_100_PERCENT_STACKED */

                case EasyXLS.Constants.Chart.CHART_TYPE_CONE_COLUMN_STACKED: { chartType = SeriesChartType.StackedColumn; break; }       /* CHART_TYPE_CONE_COLUMN_STACKED */
                case EasyXLS.Constants.Chart.CHART_TYPE_CONE_COLUMN_100_PERCENT_STACKED: { chartType = SeriesChartType.StackedColumn100; break; }    /* CHART_TYPE_CONE_COLUMN_100_PERCENT_STACKED */
                case EasyXLS.Constants.Chart.CHART_TYPE_CONE_COLUMN_3D: { chartType = SeriesChartType.Column; break; }              /* CHART_TYPE_CONE_COLUMN_3D */
                case EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_COLUMN_STACKED: { chartType = SeriesChartType.StackedColumn; break; }       /*CHART_TYPE_PYRAMID_COLUMN_STACKED*/
                case EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_COLUMN_100_PERCENT_STACKED: { chartType = SeriesChartType.StackedColumn100; break; }    /*CHART_TYPE_PYRAMID_COLUMN_100_PERCENT_STACKED*/
                case EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_COLUMN_3D: { chartType = SeriesChartType.Column; break; }              /*CHART_TYPE_PYRAMID_COLUMN_3D*/

                /* STOCK */
                case EasyXLS.Constants.Chart.CHART_TYPE_STOCK_HIGH_LOW_CLOSE: { chartType = SeriesChartType.Stock; break; }   /* CHART_TYPE_STOCK_HIGH_LOW_CLOSE */
                case EasyXLS.Constants.Chart.CHART_TYPE_STOCK_OPEN_HIGH_LOW_CLOSE: { chartType = SeriesChartType.Stock; break; }   /* CHART_TYPE_STOCK_OPEN_HIGH_LOW_CLOSE */
                case EasyXLS.Constants.Chart.CHART_TYPE_STOCK_VOLUME_HIGH_LOW_CLOSE: { chartType = SeriesChartType.Stock; break; }   /* CHART_TYPE_STOCK_VOLUME_HIGH_LOW_CLOSE */
                case EasyXLS.Constants.Chart.CHART_TYPE_STOCK_VOLUME_OPEN_HIGH_LOW_CLOSE: { chartType = SeriesChartType.Stock; break; }   /* CHART_TYPE_STOCK_VOLUME_OPEN_HIGH_LOW_CLOSE */

                /* AREA 3D */
                case EasyXLS.Constants.Chart.CHART_TYPE_AREA_3D: { chartType = SeriesChartType.Area; break; }            /*CHART_TYPE_AREA_3D*/
                case EasyXLS.Constants.Chart.CHART_TYPE_AREA_3D_STACKED: { chartType = SeriesChartType.StackedArea; break; }     /*CHART_TYPE_AREA_3D_STACKED*/
                case EasyXLS.Constants.Chart.CHART_TYPE_AREA_3D_100_PERCENT_STACKED: { chartType = SeriesChartType.StackedArea100; break; }  /*CHART_TYPE_AREA_3D_100_PERCENT_STACKED*/

                case EasyXLS.Constants.Chart.CHART_TYPE_SURFACE_3D: { chartType = SeriesChartType.Point; break; }           /* CHART_TYPE_SURFACE */
                case EasyXLS.Constants.Chart.CHART_TYPE_SURFACE_3D_WIREFRAME: { chartType = SeriesChartType.Point; break; }           /* CHART_TYPE_SURFACE_3D_WIREFRAME */
                case EasyXLS.Constants.Chart.CHART_TYPE_SURFACE_CONTOUR: { chartType = SeriesChartType.SplineArea; break; }      /* CHART_TYPE_SURFACE_CONTOUR */
                case EasyXLS.Constants.Chart.CHART_TYPE_SURFACE_CONTOUR_WIREFRAME: { chartType = SeriesChartType.SplineArea; break; }      /* CHART_TYPE_SURFACE_CONTOUR_WIREFRAME */
                default: { chartType = SeriesChartType.Area; break; }
            }
            return chartType;
        }

        #endregion

        #region [ Theme Color vs Color ]

        /// <summary>
        /// getSeriesMarker
        /// </summary>
        /// <param name="markerStyle"></param>
        /// <returns></returns>
        public GradientStyle getGradientStyle(int gradientStyle)
        {
            GradientStyle webGradient = new GradientStyle();

            switch (gradientStyle)
            {
                case FillFormat.GRADIENT_DIRECTION_FROM_CENTER: { webGradient = GradientStyle.Center; break; }       /*GRADIENT_DIAGONAL_DOWN_BOTTOM_LEFT_TO_UPPER_RIGHT*/
                case FillFormat.GRADIENT_DIRECTION_FROM_CORNER_BOTTOM_LEFT_TO_UPPER_RIGHT: { webGradient = GradientStyle.DiagonalRight; break; }     /*GRADIENT_DIAGONAL_DOWN_CORNERS_TO_CENTER*/
                case FillFormat.GRADIENT_DIRECTION_FROM_CORNER_BOTTOM_RIGHT_TO_UPPER_LEFT: { webGradient = GradientStyle.DiagonalLeft; break; }    /*GRADIENT_DIAGONAL_DOWN_UPPER_RIGHT_TO_BOTTOM_LEFT*/
                case FillFormat.GRADIENT_DIRECTION_FROM_CORNER_UPPER_RIGHT_TO_BOTTOM_LEFT: { webGradient = GradientStyle.DiagonalLeft; break; }      /*GRADIENT_DIAGONAL_UP_CORNERS_TO_CENTER*/
                case FillFormat.GRADIENT_DIRECTION_LINEAR_DIAGONAL_135_DEGREES: { webGradient = GradientStyle.LeftRight; break; }      /*GRADIENT_DIAGONAL_UP_UPPER_LEFT_TO_BOTTOM_RIGHT*/
                case FillFormat.GRADIENT_DIRECTION_LINEAR_DIAGONAL_225_DEGREES: { webGradient = GradientStyle.DiagonalRight; break; }     /*GRADIENT_FROM_BOTTOM_LEFT_CORNER*/
                case FillFormat.GRADIENT_DIRECTION_LINEAR_DIAGONAL_315_DEGREES: { webGradient = GradientStyle.DiagonalLeft; break; }      /*GRADIENT_FROM_BOTTOM_RIGHT_CORNER*/
                case FillFormat.GRADIENT_DIRECTION_LINEAR_DIAGONAL_45_DEGREES: { webGradient = GradientStyle.LeftRight; break; }     /*GRADIENT_FROM_CENTER*/
                case FillFormat.GRADIENT_DIRECTION_LINEAR_DOWN_90_DEGREES: { webGradient = GradientStyle.TopBottom; break; }      /*GRADIENT_FROM_UPPER_LEFT_CORNER*/
                case FillFormat.GRADIENT_DIRECTION_LINEAR_LEFT_180_DEGREES: { webGradient = GradientStyle.DiagonalRight; break; }      /*GRADIENT_FROM_UPPER_RIGHT_CORNER*/
                case FillFormat.GRADIENT_DIRECTION_LINEAR_RIGHT_0_DEGREES: { webGradient = GradientStyle.LeftRight; break; }      /*GRADIENT_HORIZONTAL_BOTTOM_TO_TOP*/
                case FillFormat.GRADIENT_DIRECTION_LINEAR_UP_270_DEGREES: { webGradient = GradientStyle.TopBottom; break; }      /*GRADIENT_HORIZONTAL_EDGES_TO_CENTER*/

                default: { webGradient = GradientStyle.None; break; }
            }
            return webGradient;
        }

        /// <summary>
        /// getObjectFontName
        /// </summary>
        /// <param name="objFontFormat"></param>
        /// <param name="excelTheme"></param>
        /// <returns></returns>
        string getObjectFontName(FontFormat objFontFormat, ExcelTheme excelTheme)
        {
            // keeps the font name
            string fontName = "";

            // initialize it with the font form EasyXLS
            fontName = objFontFormat.getFont();
           
            // if font name is BODY scheme
            if (fontName == FontSettings.BODY_FONT)
            {
                // get appropriate font
                fontName = excelTheme.getFontScheme().getBodyFont();
            }
            // if font name is HEADING scheme
            else if (fontName == FontSettings.HEADINGS_FONT)
            {
                // get appropriate font
                fontName = excelTheme.getFontScheme().getHeadingFont();
            }

            //return font
            return fontName;
        }

        /// <summary>
        /// getObjectForegroundColor
        /// </summary>
        /// <param name="objFontFormat"></param>
        /// <param name="excelTheme"></param>
        /// <returns></returns>
        Color getObjectForegroundColor(FontFormat objFontFormat, ExcelTheme excelTheme)
        {
            // keeps the font name
            Color foreground = Color.Black;

            // get the theme color
            ThemeColor themeForegroundColor = objFontFormat.getForegroundThemeColor();

            // if theme color does not exist
            if (themeForegroundColor == null)
            {
                // get normal foreground
                foreground = objFontFormat.getForeground();

                // if foreground is empty return black
                if (foreground.IsEmpty) foreground = Color.Black;
            }
            else
            {
                // get the color from the theme
                foreground = themeForegroundColor.getColorForTheme(excelTheme);
            }

            //return foreground
            return foreground;
        }

        /// <summary>
        /// getObjectLineColor
        /// </summary>
        /// <param name="objFontFormat"></param>
        /// <param name="excelTheme"></param>
        /// <returns></returns>
        Color getObjectLineColor(LineColorFormat objLineColorFormat, ExcelTheme excelTheme, int seriesId)
        {
            // keeps the font name
            Color lineColor = Color.Black;

            // get the theme color
            ThemeColor themeLineColor = new ThemeColor();

            if (objLineColorFormat.IsSolidLine())
            {
                themeLineColor = objLineColorFormat.getLineThemeColor();

                // if theme color does not exist
                if (themeLineColor == null)
                {
                    // get normal foreground
                    lineColor = objLineColorFormat.getLineColor();

                    // if foreground is empty return black
                    if (lineColor.IsEmpty) lineColor = Color.LightGray;
                }
                else
                {
                    // get the color from the theme
                    lineColor = themeLineColor.getColorForTheme(excelTheme);
                }
            }

            if (objLineColorFormat.IsAutomatic())
            {
                // check if theme is not null
                if (excelTheme != null)
                {
                    // check if series involved
                    if (seriesId >= 0)
                    {
                        // constant for theme colors
                        const int SERIES_START_COLORS = 4;
                        // constant for theme colors
                        const int SERIES_NOOF_COLORS = 6;
                        double tintColor = 0;
                        int basicColorIdx = seriesId;

                        tintColor += (seriesId / SERIES_NOOF_COLORS) * 0.2;
                        if (seriesId >= SERIES_NOOF_COLORS)
                        {
                            basicColorIdx = seriesId - SERIES_NOOF_COLORS * (seriesId / SERIES_NOOF_COLORS);
                        }

                        // get the automatic theme color
                        themeLineColor = new ThemeColor(SERIES_START_COLORS + basicColorIdx, tintColor);
                        // set back color
                        lineColor = themeLineColor.getColorForTheme(excelTheme);
                    }
                }
            }

            //return foreground
            return lineColor;
        }

        /// <summary>
        /// getObjectBackColor
        /// </summary>
        /// <param name="objFontFormat"></param>
        /// <param name="excelTheme"></param>
        /// <returns></returns>
        Color getObjectBackColor(FillFormat objBackColorFormat, ExcelTheme excelTheme, int seriesId)
        {
            // keeps the font name
            Color backColor = Color.White;

             // get the theme back color
            ThemeColor themeBackColor = new ThemeColor();

            if (objBackColorFormat.IsSolidFill())
            {
                // get the theme back color
                themeBackColor = objBackColorFormat.getBackgroundThemeColor();


                // if theme back color does not exist
                if (themeBackColor == null)
                {
                    // get normal back color
                    backColor = objBackColorFormat.getBackground();

                    // if back color is empty return black
                    if (backColor.IsEmpty) backColor = Color.White;
                }
                else
                {
                    // get the back color from the theme
                    backColor = themeBackColor.getColorForTheme(excelTheme);
                }
            }

            // check if color is automatic
            if (objBackColorFormat.IsAutomatic())
            {
                // check if theme is not null
                if (excelTheme != null)
                {
                    // check if series involved
                    if(seriesId >= 0)
                    {
                        // constant for theme colors
                        const int SERIES_START_COLORS = 4;
                        // constant for theme colors
                        const int SERIES_NOOF_COLORS = 6;
                        double tintColor = 0;
                        int basicColorIdx = seriesId;

                        tintColor += (seriesId/SERIES_NOOF_COLORS) * 0.2;
                        if(seriesId >= SERIES_NOOF_COLORS) 
                        {
                            basicColorIdx = seriesId-SERIES_NOOF_COLORS*(seriesId/SERIES_NOOF_COLORS);
                        }

                        // get the automatic theme color
                        themeBackColor = new ThemeColor(SERIES_START_COLORS + basicColorIdx, tintColor);
                        // set back color
                        backColor = themeBackColor.getColorForTheme(excelTheme);
                    }
                }
            }

            return backColor;
        }

        /// <summary>
        /// getObjectWidth
        /// </summary>
        /// <param name="objLineColorFormat"></param>
        /// <param name="objLineStyleFormat"></param>
        /// <returns></returns>
        double getObjectWidth(LineColorFormat objLineColorFormat, LineStyleFormat objLineStyleFormat, string obj)
        {
            double lineWidth = objLineStyleFormat.getWidth();
            bool bIsLineAutomatic = objLineColorFormat.IsAutomatic();
            bool bIsNoLine = objLineColorFormat.IsNoLine();

            //Check if the line width is automatic
            if (
                    (bIsLineAutomatic || !bIsNoLine)
                    && lineWidth == 0
                    && objLineStyleFormat.getCompoundType().Equals(LineStyleFormat.COMPOUND_TYPE_NONE))
            {
                
                //add automatic width for axis
                switch(obj)
                {
                    case "axis": lineWidth = 0.75; break;
                    case "tickMarks": lineWidth = 0.75; break;
                    case "gridLines": lineWidth = 0.75; break;
                    case "series": lineWidth = 2.25; break;
                    case "border": lineWidth = 1.0; break;
                    case "legend": lineWidth = 0; break;
                    case "markers": lineWidth = 0.75; break;
                    default: lineWidth = 0.75; break;
                }
            }

            return lineWidth;
        }

        #endregion

        #region [ Series Settings ]

        /// <summary>
        /// GetSeriesName
        /// </summary>
        /// <param name="seriesName"></param>
        /// <returns></returns>
        private string GetSeriesName(string seriesName, ExcelDocument excelDoc, Formula formula, ExcelTable xlsTable, int seriesId)
        {
            string getName = seriesName;
            if (seriesName != "" && seriesName.Contains("!"))
            {
                //ExcelTable xlsTable = new ExcelTable();
                string SheetName = "";
                int indexOfEqual = seriesName.IndexOf("=") + 1;
                int indexOfExclam = seriesName.IndexOf("!");
                SheetName = seriesName.Substring(indexOfEqual, indexOfExclam - indexOfEqual);

                // check if sheetName contins special characters
                if (SheetName.Contains("'"))
                    SheetName = SheetName.Replace("'", "");

                // get the document table
                xlsTable = ((ExcelWorksheet)excelDoc.easy_getSheet(SheetName)).easy_getExcelTable();

                if (seriesName.StartsWith("="))
                    seriesName = seriesName.Remove(0, 1);

                if (seriesName.Contains(":"))
                { getName = "No Name " + seriesId; }
                else
                {
                    object[] elementName = formula.get3DCellElements(seriesName);
                    // start col/row
                    int col = int.Parse(elementName[1].ToString());
                    int row = int.Parse(elementName[2].ToString());
                    getName = xlsTable.easy_getCellAt(row, col).getValue();
                }
            }

            return getName;
        }

        /// <summary>
        /// getSeriesMarker
        /// </summary>
        /// <param name="markerStyle"></param>
        /// <returns></returns>
        public MarkerStyle getSeriesMarker(int markerStyle)
        {
            MarkerStyle marker = new MarkerStyle();

            switch (markerStyle)
            {
                case EasyXLS.Constants.Chart.SERIES_MARKER_NONE: { marker = MarkerStyle.None; break; }       /*SERIES_MARKER_NONE*/
                case EasyXLS.Constants.Chart.SERIES_MARKER_SQUARE: { marker = MarkerStyle.Square; break; }     /*SERIES_MARKER_SQUARE*/
                case EasyXLS.Constants.Chart.SERIES_MARKER_DIAMOND: { marker = MarkerStyle.Diamond; break; }    /*SERIES_MARKER_DIAMOND*/
                case EasyXLS.Constants.Chart.SERIES_MARKER_TRIANGLE: { marker = MarkerStyle.Triangle; break; }   /*SERIES_MARKER_TRIANGLE*/
                case EasyXLS.Constants.Chart.SERIES_MARKER_X: { marker = MarkerStyle.Star6; break; }      /*SERIES_MARKER_X*/
                case EasyXLS.Constants.Chart.SERIES_MARKER_STAR: { marker = MarkerStyle.Star6; break; }      /*SERIES_MARKER_STAR*/
                case EasyXLS.Constants.Chart.SERIES_MARKER_DOW_JONES: { marker = MarkerStyle.Star10; break; }     /*SERIES_MARKER_DOW_JONES*/
                case EasyXLS.Constants.Chart.SERIES_MARKER_STANDARD_DEVIATION: { marker = MarkerStyle.Star4; break; }      /*SERIES_MARKER_STANDARD_DEVIATION*/
                case EasyXLS.Constants.Chart.SERIES_MARKER_CIRCLE: { marker = MarkerStyle.Circle; break; }     /*SERIES_MARKER_CIRCLE*/
                case EasyXLS.Constants.Chart.SERIES_MARKER_PLUS_SIGN: { marker = MarkerStyle.Cross; break; }      /*SERIES_MARKER_PLUS_SIGN*/
                default: { marker = MarkerStyle.None; break; }
            }
            return marker;
        }

        /// <summary>
        /// getPointWidthByGap
        /// </summary>
        /// <returns></returns>
        public double getPointWidthByGap(int excelGapValue)
        {
            double value = 0.0;

            if (excelGapValue >= 430 && excelGapValue <= 500) return 0.1;
            if (excelGapValue >= 360 && excelGapValue < 430) return 0.2;
            if (excelGapValue >= 290 && excelGapValue < 360) return 0.3;
            if (excelGapValue >= 250 && excelGapValue < 290) return 0.4;
            if (excelGapValue >= 200 && excelGapValue < 250) return 0.5;
            if (excelGapValue >= 150 && excelGapValue < 200) return 0.6;
            if (excelGapValue >= 80 && excelGapValue < 150) return 0.7;
            if (excelGapValue >= 10 && excelGapValue < 80) return 0.8;
            if (excelGapValue > 0 && excelGapValue < 10) return 0.9;
            if (excelGapValue == 0) return 1;

            return value;
        }

        /// <summary>
        /// getDataLabelStyle
        /// </summary>
        /// <returns></returns>
        public string getDataLabelStyle(ExcelDataLabels dataLabelStyle)
        {
            string value = "";
            if (dataLabelStyle.ShowCategoryName() && dataLabelStyle.ShowPercentage())
            {
                value = "#VALX, #PERCENT{P2}"; /* SERIES_DATA_LABELS_SHOW_LABEL_AND_PERCENT */
            }
            else if (dataLabelStyle.ShowValue())
            {
                value = "#VALY"; /* SERIES_DATA_LABELS_SHOW_VALUE */
            }
            else if (dataLabelStyle.ShowPercentage())
            {
                value = "#PERCENT{P2}"; /* SERIES_DATA_LABELS_SHOW_PERCENT */
            }
            else if (dataLabelStyle.ShowCategoryName())
            {
                value = "#VALX"; /* SERIES_DATA_LABELS_SHOW_LABEL */
            }
            else if (dataLabelStyle.ShowBubbleSize())
            {
                value = "SERIES_DATA_LABELS_SHOW_BUBBLE_SIZE"; /* SERIES_DATA_LABELS_SHOW_BUBBLE_SIZE */
            }
            else
            {
                value = "";
            }
            //switch (dataLabelStyle)
            //{
            //    case EasyXLS.Constants.Chart.SERIES_DATA_LABELS_NONE: { value = ""; break; }                                      /* SERIES_DATA_LABELS_NONE */
            //    case EasyXLS.Constants.Chart.SERIES_DATA_LABELS_SHOW_VALUE: { value = "#VALY"; break; }                                 /* SERIES_DATA_LABELS_SHOW_VALUE */
            //    case EasyXLS.Constants.Chart.SERIES_DATA_LABELS_SHOW_PERCENT: { value = "#PERCENT{P2}"; break; }                          /* SERIES_DATA_LABELS_SHOW_PERCENT */
            //    case EasyXLS.Constants.Chart.SERIES_DATA_LABELS_SHOW_LABEL: { value = "#VALX"; break; }                                 /* SERIES_DATA_LABELS_SHOW_LABEL */
            //    case EasyXLS.Constants.Chart.SERIES_DATA_LABELS_SHOW_LABEL_AND_PERCENT: { value = "#VALX, #PERCENT{P2}"; break; }                   /* SERIES_DATA_LABELS_SHOW_LABEL_AND_PERCENT */
            //    case EasyXLS.Constants.Chart.SERIES_DATA_LABELS_SHOW_BUBBLE_SIZE: { value = "SERIES_DATA_LABELS_SHOW_BUBBLE_SIZE"; break; }   /* SERIES_DATA_LABELS_SHOW_BUBBLE_SIZE */
            //    default: { value = ""; break; }
            //}

            return value;
        }

        /// <summary>
        /// getChartLineStyle
        /// </summary>
        /// <param name="chartLineStyle"></param>
        /// <returns></returns>
        public ChartDashStyle getChartLineStyle(string chartLineStyle)
        {
            ChartDashStyle lineStyle = new ChartDashStyle();

            switch (chartLineStyle)
            {
                case LineStyleFormat.DASH_TYPE_DASH: { lineStyle = ChartDashStyle.Dash; break; }         /*DASH_TYPE_DASH*/
                case LineStyleFormat.DASH_TYPE_DASH_DOT: { lineStyle = ChartDashStyle.DashDot; break; }      /*DASH_TYPE_DASH_DOT*/
                case LineStyleFormat.DASH_TYPE_ROUND_DOT: { lineStyle = ChartDashStyle.DashDot; break; }      /*DASH_TYPE_ROUND_DOT*/
                case LineStyleFormat.DASH_TYPE_SQUARE_DOT: { lineStyle = ChartDashStyle.Dash; break; }         /*DASH_TYPE_SQUARE_DOT*/
                case LineStyleFormat.DASH_TYPE_LONG_DASH_DOT_DOT: { lineStyle = ChartDashStyle.DashDotDot; break; }   /*DASH_TYPE_LONG_DASH_DOT_DOT*/
                case LineStyleFormat.DASH_TYPE_SOLID: { lineStyle = ChartDashStyle.Solid; break; }        /*DASH_TYPE_SOLID*/
                default: { lineStyle = ChartDashStyle.Solid; break; }        /*LINE_STYLE_SOLID*/
            }
            return lineStyle;
        }


        #endregion

        #region [ Axis Settings ]

        /// <summary>
        /// getFontStyle
        /// </summary>
        /// <param name="fontFormat"></param>
        /// <returns></returns>
        public FontStyle getFontStyle(FontFormat fontFormat)
        {
            FontStyle fontStyle = new FontStyle();
            if (fontFormat.IsBold()) fontStyle = FontStyle.Bold;
            if (fontFormat.IsItalic()) fontStyle = FontStyle.Italic;
            if (fontFormat.IsStrikethrough()) fontStyle = FontStyle.Strikeout;
            if (fontFormat.getUnderlineStyle() != "none") fontStyle = FontStyle.Underline;

            return fontStyle;
        }

        /// <summary>
        /// isLabelNextToAxis
        /// </summary>
        /// <param name="tickMarkType"></param>
        /// <returns></returns>
        public bool isLabelNextToAxis(int tickMarkLabel)
        {
            bool isNextToAxis = false;

            switch (tickMarkLabel)
            {
                case EasyXLS.Constants.Chart.AXIS_TICK_MARK_LABEL_HIGH: { isNextToAxis = false; break; }    /*AXIS_TICK_MARK_LABEL_HIGH*/
                case EasyXLS.Constants.Chart.AXIS_TICK_MARK_LABEL_NEXT_TO_AXIS: { isNextToAxis = true; break; }     /*AXIS_TICK_MARK_LABEL_NEXT_TO_AXIS*/
                case EasyXLS.Constants.Chart.AXIS_TICK_MARK_LABEL_LOW: { isNextToAxis = false; break; }    /*AXIS_TICK_MARK_LABEL_LOW*/
                case EasyXLS.Constants.Chart.AXIS_TICK_MARK_LABEL_NONE: { isNextToAxis = false; break; }    /*AXIS_TICK_MARK_LABEL_NONE */
            }
            return isNextToAxis;
        }

        /// <summary>
        /// getTickMarkStyle
        /// </summary>
        /// <param name="tickMarkType"></param>
        /// <returns></returns>
        public TickMarkStyle getTickMarkStyle(int tickMarkType)
        {
            TickMarkStyle markType = new TickMarkStyle();

            switch (tickMarkType)
            {
                case EasyXLS.Constants.Chart.AXIS_TICK_MARK_TYPE_CROSS: { markType = TickMarkStyle.AcrossAxis; break; }     /*AXIS_TICK_MARK_TYPE_CROSS*/
                case EasyXLS.Constants.Chart.AXIS_TICK_MARK_TYPE_INSIDE: { markType = TickMarkStyle.InsideArea; break; }     /*AXIS_TICK_MARK_TYPE_INSIDE*/
                case EasyXLS.Constants.Chart.AXIS_TICK_MARK_TYPE_NONE: { markType = TickMarkStyle.None; break; }           /*AXIS_TICK_MARK_TYPE_NONE*/
                case EasyXLS.Constants.Chart.AXIS_TICK_MARK_TYPE_OUTSIDE: { markType = TickMarkStyle.OutsideArea; break; }    /*AXIS_TICK_MARK_TYPE_OUTSIDE */
                default: { markType = TickMarkStyle.None; break; }           /*AXIS_TICK_MARK_TYPE_NONE*/
            }
            return markType;
        }


        #endregion

        #region [ Legend Settings ]

        /// <summary>
        /// getAlignment
        /// </summary>
        /// <param name="xlsAlignment"></param>
        /// <returns></returns>
        public ContentAlignment getAlignment(AlignmentFormat xlsAlignment)
        {
            string horAlign = xlsAlignment.getHorizontalAlignment();
            string verAlign = xlsAlignment.getVerticalAlignment();
            ContentAlignment contAlign = new ContentAlignment();
            switch (verAlign)
            {
                case AlignmentFormat.VERTICAL_ALIGNMENT_TOP: { contAlign = ContentAlignment.TopRight; break; }        /*VERTICAL_ALIGNMENT_TOP*/
                case AlignmentFormat.VERTICAL_ALIGNMENT_MIDDLE: { contAlign = ContentAlignment.MiddleRight; break; }   /*VERTICAL_ALIGNMENT_MIDDLE*/
                case AlignmentFormat.VERTICAL_ALIGNMENT_BOTTOM: { contAlign = ContentAlignment.BottomRight; break; }     /*VERTICAL_ALIGNMENT_BOTTOM*/
                case AlignmentFormat.VERTICAL_ALIGNMENT_TOP_CENTER: { contAlign = ContentAlignment.TopCenter; break; }      /*VERTICAL_ALIGNMENT_TOP_CENTER*/
                case AlignmentFormat.VERTICAL_ALIGNMENT_MIDDLE_CENTER: { contAlign = ContentAlignment.MiddleCenter; break; }   /*VERTICAL_ALIGNMENT_MIDDLE_CENTER*/
                case AlignmentFormat.VERTICAL_ALIGNMENT_BOTTOM_CENTER: { contAlign = ContentAlignment.BottomCenter; break; }   /*VERTICAL_ALIGNMENT_BOTTOM_CENTER*/
            }

            switch (horAlign)
            {
                case AlignmentFormat.HORIZONTAL_ALIGNMENT_LEFT: { contAlign = ContentAlignment.TopLeft; break; }        /*HORIZONTAL_ALIGNMENT_LEFT*/
                case AlignmentFormat.HORIZONTAL_ALIGNMENT_CENTER: { contAlign = ContentAlignment.TopCenter; break; }   /*HORIZONTAL_ALIGNMENT_CENTER*/
                case AlignmentFormat.HORIZONTAL_ALIGNMENT_RIGHT: { contAlign = ContentAlignment.TopRight; break; }       /*HORIZONTAL_ALIGNMENT_RIGHT*/
                case AlignmentFormat.HORIZONTAL_ALIGNMENT_LEFT_MIDDLE: { contAlign = ContentAlignment.MiddleLeft; break; }     /*HORIZONTAL_ALIGNMENT_LEFT_MIDDLE*/
                case AlignmentFormat.HORIZONTAL_ALIGNMENT_CENTER_MIDDLE: { contAlign = ContentAlignment.MiddleCenter; break; }   /*HORIZONTAL_ALIGNMENT_CENTER_MIDDLE*/
                case AlignmentFormat.HORIZONTAL_ALIGNMENT_RIGHT_MIDDLE: { contAlign = ContentAlignment.MiddleRight; break; }    /*HORIZONTAL_ALIGNMENT_RIGHT_MIDDLE*/
            }

            return contAlign;
        }

        /// <summary>
        /// getDocking
        /// </summary>
        /// <param name="dockType"></param>
        /// <returns></returns>
        public Docking getDocking(int dockType)
        {
            Docking dockingType = new Docking();

            switch (dockType)
            {
                case EasyXLS.Constants.Chart.LEGEND_BOTTOM: { dockingType = Docking.Bottom; break; }    /*LEGEND_BOTTOM*/
                case EasyXLS.Constants.Chart.LEGEND_LEFT: { dockingType = Docking.Left; break; }      /*LEGEND_LEFT*/
                case EasyXLS.Constants.Chart.LEGEND_RIGHT: { dockingType = Docking.Right; break; }     /*LEGEND_RIGHT*/
                case EasyXLS.Constants.Chart.LEGEND_TOP: { dockingType = Docking.Top; break; }       /*LEGEND_TOP */
                case EasyXLS.Constants.Chart.LEGEND_CORNER: { dockingType = Docking.Right; break; }       /*LEGEND_CORNER */
                default: { dockingType = Docking.Right; break; }     /*LEGEND_Right*/
            }
            return dockingType;
        }

        /// <summary>
        /// getAlignmentByPlacement
        /// </summary>
        /// <param name="dockType"></param>
        /// <returns></returns>
        public StringAlignment getAlignmentByPlacement(int dockType)
        {
            StringAlignment alignmentType = new StringAlignment();

            switch (dockType)
            {
                case EasyXLS.Constants.Chart.LEGEND_CORNER: { alignmentType = StringAlignment.Near; break; }    /**/
                default: { alignmentType = StringAlignment.Center; break; }     /*LEGEND_CENTER*/
            }
            return alignmentType;
        }
        #endregion

        #region [ Classification of charts ]

        /// <summary>
        /// IsChartToReverseLegend
        /// </summary>
        /// <param name="chartTp"></param>
        /// <returns></returns>
        protected bool IsChartToReverseLegend(int chartTp)
        {
            bool mustReverse = false;

            // have to reverse if scatter chart
            if (IsScatterChart(chartTp))
            {
                mustReverse = true;
                return true;
            }

            // have to reverse if doughnut chart
            if (IsDoughnutChart(chartTp))
            {
                mustReverse = true;
                return true;
            }

            // have to reverse if pie chart
            if (IsPieChart(chartTp))
            {
                mustReverse = true;
                return true;
            }

            // have to reverse if clustered BAR chart
            if (IsClusteredBarChart(chartTp))
            {
                mustReverse = true;
                return true;
            }

            // have to reverse if clustered BAR chart
            if (IsRadarChart(chartTp))
            {
                mustReverse = true;
                return true;
            }

            if (
                // CLUSTERED COLUMN
                chartTp == EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_CLUSTERED
                // CLUSTERED CYLINDER COLUMN
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_COLUMN
                // CLUSTERED PYRAMID COLUMN
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_COLUMN
                // STACKED CONE COLUMN
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CONE_COLUMN
                //CHART_TYPE_COLUMN_3D_CLUSTERED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_3D_CLUSTERED 

                // STACKED BARS
                || 
                chartTp == EasyXLS.Constants.Chart.CHART_TYPE_BAR_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_BAR_100_PERCENT_STACKED
                // STACKED 3D BAR
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_BAR_3D_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_BAR_3D_100_PERCENT_STACKED
                // STACKED CYLINDER BAR
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_BAR_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_BAR_100_PERCENT_STACKED
                // STCAKED PYRAMID BAR
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_BAR_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_BAR_100_PERCENT_STACKED
                // STACKED CONE BAR
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CONE_BAR_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CONE_BAR_100_PERCENT_STACKED

                // STACKED LINE
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_LINE
                //|| chartTp == EasyXLS.Constants.Chart.CHART_TYPE_LINE_100_PERCENT_STACKED
                //STACKED LINE MARKERS
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_LINE_MARKERS
                //|| chartTp == EasyXLS.Constants.Chart.CHART_TYPE_LINE_100_PERCENT_STACKED_MARKERS

                // STACKED AREA
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_AREA_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_AREA_100_PERCENT_STACKED
                // STACKED 3D AREA
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_AREA_3D
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_AREA_3D_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_AREA_3D_100_PERCENT_STACKED
                )
                mustReverse = true;

            return mustReverse;
        }

        /// <summary>
        /// IsSurfaceChart
        /// </summary>
        /// <param name="chartTp"></param>
        /// <returns></returns>
        protected bool IsSurfaceChart(int chartTp)
        {
            bool IsSurface = false;
            if (
                chartTp == EasyXLS.Constants.Chart.CHART_TYPE_SURFACE_3D
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_SURFACE_3D_WIREFRAME
                )
                IsSurface = true;

            return IsSurface;
        }

        /// <summary>
        /// IsDoughnutChart
        /// </summary>
        /// <param name="chartTp"></param>
        /// <returns></returns>
        protected bool IsDoughnutChart(int chartTp)
        {
            bool IsDoughnut = false;
            if (
                chartTp == EasyXLS.Constants.Chart.CHART_TYPE_DOUGHNUT
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_DOUGHNUT_EXPLODED
                )
                IsDoughnut = true;

            return IsDoughnut;
        }

        /// <summary>
        /// IsBubbleChart
        /// </summary>
        /// <param name="chartTp"></param>
        /// <returns></returns>
        protected bool IsBubbleChart(int chartTp)
        {
            bool IsBubble = false;
            if (
                chartTp == EasyXLS.Constants.Chart.CHART_TYPE_BUBBLE
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_BUBBLE_3D_EFFECT
                )
                IsBubble = true;

            return IsBubble;
        }

        /// <summary>
        /// IsStockChart
        /// </summary>
        /// <param name="chartTp"></param>
        /// <returns></returns>
        protected bool IsStockChart(int chartTp)
        {
            bool IsStock = false;
            if (
                chartTp == EasyXLS.Constants.Chart.CHART_TYPE_STOCK_HIGH_LOW_CLOSE
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_STOCK_OPEN_HIGH_LOW_CLOSE
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_STOCK_VOLUME_HIGH_LOW_CLOSE
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_STOCK_VOLUME_OPEN_HIGH_LOW_CLOSE
                )
                IsStock = true;

            return IsStock;
        }

        /// <summary>
        /// IsExplodedChart
        /// </summary>
        /// <param name="chartTp"></param>
        /// <returns></returns>
        protected bool IsExplodedChart(int chartTp)
        {
            bool IsExploded = false;
            if (
                chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PIE_EXPLODED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PIE_3D_EXPLODED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_DOUGHNUT_EXPLODED
                )
                IsExploded = true;

            return IsExploded;
        }

        /// <summary>
        /// Is3DChart
        /// </summary>
        /// <param name="chartTp"></param>
        /// <returns></returns>
        protected bool Is3DChart(int chartTp)
        {
            bool Is3D = false;
            if (
                // COLUMN CHARTS
                    chartTp == EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_3D
                    || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_3D_CLUSTERED
                    || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_3D_100_PERCENT_STACKED
                    || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_3D_STACKED
                    || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_COLUMN_3D
                    || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CONE_COLUMN_3D
                    || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_COLUMN_3D

                     // BAR CHARTS
                    || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_BAR_3D_100_PERCENT_STACKED
                    || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_BAR_3D_STACKED

                    // LINE CHARTS
                    || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_LINE_3D

                    // AREA CHARTS
                   || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_AREA_3D
                   || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_AREA_3D_STACKED
                   || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_AREA_3D_100_PERCENT_STACKED

                   // BUBBLE CHARTS
                   || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_BUBBLE_3D_EFFECT

                   // PIE CHARTS
                   || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PIE_3D
                   || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PIE_3D_EXPLODED

                   // SURFACE CHARTS
                   || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_SURFACE_3D
                //|| chartTp == EasyXLS.Constants.Chart.CHART_TYPE_SURFACE_3D_WIREFRAME
                )
                Is3D = true;
            return Is3D;
        }

        /// <summary>
        /// Is100StackedColumnChart
        /// </summary>
        /// <param name="chartTp"></param>
        /// <returns></returns>
        protected bool Is100StackedColumnChart(int chartTp)
        {
            bool Is100StackedColumn = false;
            if (
                chartTp == EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_100_PERCENT_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_3D_100_PERCENT_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_COLUMN_100_PERCENT_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CONE_COLUMN_100_PERCENT_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_COLUMN_100_PERCENT_STACKED
                )
                Is100StackedColumn = true;

            return Is100StackedColumn;
        }

        /// <summary>
        /// Is100StackedLineChart
        /// </summary>
        /// <param name="chartTp"></param>
        /// <returns></returns>
        protected bool Is100StackedLineChart(int chartTp)
        {
            bool Is100StackedLine = false;
            if (
                chartTp == EasyXLS.Constants.Chart.CHART_TYPE_LINE_100_PERCENT_STACKED_MARKERS
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_LINE_100_PERCENT_STACKED
                )
                Is100StackedLine = true;

            return Is100StackedLine;
        }

        /// <summary>
        /// IsAreaChart
        /// </summary>
        /// <param name="chartTp"></param>
        /// <returns></returns>
        protected bool IsAreaChart(int chartTp)
        {
            bool isArea = false;
            if (
                chartTp == EasyXLS.Constants.Chart.CHART_TYPE_AREA
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_AREA_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_AREA_100_PERCENT_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_AREA_3D
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_AREA_3D_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_AREA_3D_100_PERCENT_STACKED)
                isArea = true;

            return isArea;
        }

        /// <summary>
        /// IsCylinderChart
        /// </summary>
        /// <param name="chartTp"></param>
        /// <returns></returns>
        protected bool IsPieChart(int chartTp)
        {
            bool isPie = false;
            if (
                chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PIE
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PIE_3D
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PIE_OF_PIE
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PIE_EXPLODED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PIE_3D_EXPLODED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_BAR_OF_PIE)
                isPie = true;

            return isPie;
        }

        /// <summary>
        /// IsCylinderChart
        /// </summary>
        /// <param name="chartTp"></param>
        /// <returns></returns>
        protected bool IsCylinderChart(int chartTp)
        {
            bool isCylinder = false;
            if (
                chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_COLUMN
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_COLUMN_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_COLUMN_100_PERCENT_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_BAR
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_BAR_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_BAR_100_PERCENT_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_COLUMN_3D
                )
                isCylinder = true;

            return isCylinder;
        }


        /// <summary>
        /// IsBarChart
        /// </summary>
        /// <param name="chartTp"></param>
        /// <returns></returns>
        protected bool IsBarChart(int chartTp)
        {
            bool isBar = false;
            if (
                // BAR CHART
                chartTp == EasyXLS.Constants.Chart.CHART_TYPE_BAR_CLUSTERED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_BAR_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_BAR_100_PERCENT_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_BAR_3D_CLUSTERED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_BAR_3D_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_BAR_3D_100_PERCENT_STACKED

                // CONE BAR CHART
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CONE_BAR
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CONE_BAR_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CONE_BAR_100_PERCENT_STACKED

                // CYLINDER BAR CHART
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_BAR
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_BAR_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_BAR_100_PERCENT_STACKED

                // PYRAMID BAR CHART
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_BAR
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_BAR_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_BAR_100_PERCENT_STACKED
                )
                isBar = true;

            return isBar;
        }

        /// <summary>
        /// IsClusteredChart
        /// </summary>
        /// <param name="chartTp"></param>
        /// <returns></returns>
        protected bool IsClusteredChart(int chartTp)
        {
            bool isClustered = false;
            if (
                // BAR CHART
                chartTp == EasyXLS.Constants.Chart.CHART_TYPE_BAR_CLUSTERED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_BAR_3D_CLUSTERED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_3D_CLUSTERED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_CLUSTERED 

                // CONE BAR CHART
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CONE_BAR
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CONE_COLUMN 

                // CYLINDER BAR CHART
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_BAR
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_COLUMN 

                // PYRAMID BAR CHART
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_BAR
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_COLUMN

                
                )
                isClustered = true;

            return isClustered;
        }

        /// <summary>
        /// IsBarChart
        /// </summary>
        /// <param name="chartTp"></param>
        /// <returns></returns>
        protected bool IsClusteredBarChart(int chartTp)
        {
            bool isClusteredBar = false;
            if (
                // BAR CHART
                chartTp == EasyXLS.Constants.Chart.CHART_TYPE_BAR_CLUSTERED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_BAR_3D_CLUSTERED

                // CONE BAR CHART
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CONE_BAR

                // CYLINDER BAR CHART
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_BAR

                // PYRAMID BAR CHART
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_BAR
                )
                isClusteredBar = true;

            return isClusteredBar;
        }

        /// <summary>
        /// Is3DColumnChart
        /// </summary>
        /// <param name="chartTp"></param>
        /// <returns></returns>
        protected bool Is3DColumnChart(int chartTp)
        {
            bool is3DColumn = false;
            if (
               
                // 3D COLUMN CHARTS
                chartTp == EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_3D

                // CYLINDER COLUMN CHARTS
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_COLUMN_3D

                // CONE COLUMN CHARTS
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CONE_COLUMN_3D

                // PYRAMID COLUMN CHARTS
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_COLUMN_3D
                )
                is3DColumn = true;

            return is3DColumn;
        }

        /// <summary>
        /// IsColumnChart
        /// </summary>
        /// <param name="chartTp"></param>
        /// <returns></returns>
        protected bool IsColumnChart(int chartTp)
        {
            bool isColumn = false;
            if (
                // COLUMN CHARTS
                chartTp == EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_CLUSTERED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_100_PERCENT_STACKED

                // 3D COLUMN CHARTS
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_3D_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_3D_100_PERCENT_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_3D
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_3D_CLUSTERED

                // CYLINDER COLUMN CHARTS
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_COLUMN
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_COLUMN_3D
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_COLUMN_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_COLUMN_100_PERCENT_STACKED

                // CONE COLUMN CHARTS
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CONE_COLUMN
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CONE_COLUMN_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CONE_COLUMN_100_PERCENT_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CONE_COLUMN_3D

                // PYRAMID COLUMN CHARTS
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_COLUMN
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_COLUMN_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_COLUMN_100_PERCENT_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_COLUMN_3D
                )
                isColumn = true;

            return isColumn;
        }

        /// <summary>
        /// IsColumnChart
        /// </summary>
        /// <param name="chartTp"></param>
        /// <returns></returns>
        protected bool IsColumnStackedChart(int chartTp)
        {
            bool isColumnStacked = false;
            if (
                chartTp == EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_STACKED

                // 3D COLUMN CHARTS
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_3D_STACKED

                // CYLINDER COLUMN CHARTS
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_COLUMN_STACKED

                // CONE COLUMN CHARTS
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CONE_COLUMN_STACKED

                // PYRAMID COLUMN CHARTS
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_COLUMN_STACKED
                )
                isColumnStacked = true;

            return isColumnStacked;
        }

        /// <summary>
        /// Is100Stacked
        /// </summary>
        /// <param name="chartTp"></param>
        /// <returns></returns>
        protected bool Is100Stacked(int chartTp)
        {
            bool is100Stack = false;
            if (
                // 100% COLUMN CHARTS
                chartTp == EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_100_PERCENT_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_COLUMN_3D_100_PERCENT_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_COLUMN_100_PERCENT_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CONE_COLUMN_100_PERCENT_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_COLUMN_100_PERCENT_STACKED

                // 100% BAR CHARTS
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_BAR_100_PERCENT_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_BAR_3D_100_PERCENT_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CYLINDER_BAR_100_PERCENT_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_CONE_BAR_100_PERCENT_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_PYRAMID_BAR_100_PERCENT_STACKED

                // 100% LINE CHARTS
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_LINE_100_PERCENT_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_LINE_100_PERCENT_STACKED_MARKERS

                // 100% AREA CHARTS
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_AREA_100_PERCENT_STACKED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_AREA_3D_100_PERCENT_STACKED
                )
                is100Stack = true;

            return is100Stack;
        }

        /// <summary>
        /// IsScatterChart
        /// </summary>
        /// <param name="chartTp"></param>
        /// <returns></returns>
        protected bool IsScatterChart(int chartTp)
        {
            bool isScatter = false;

            if (
                chartTp == EasyXLS.Constants.Chart.CHART_TYPE_SCATTER_SMOOTHED_LINES_MARKERS
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_SCATTER_SMOOTHED_LINES
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_SCATTER_LINES_MARKERS
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_SCATTER_LINES
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_SCATTER 
                )
                isScatter = true;

            return isScatter;
        }

        /// <summary>
        /// IsScatterChart
        /// </summary>
        /// <param name="chartTp"></param>
        /// <returns></returns>
        protected bool IsRadarChart(int chartTp)
        {
            bool isRadar = false;

            if (
                chartTp == EasyXLS.Constants.Chart.CHART_TYPE_RADAR
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_RADAR_FILLED
                || chartTp == EasyXLS.Constants.Chart.CHART_TYPE_RADAR_MARKERS
                )
                isRadar = true;

            return isRadar;
        }


        /// <summary>
        /// getMaxValueFrom
        /// </summary>
        /// <param name="valY"></param>
        /// <returns></returns>
        protected double getMaxValueFrom(double[] valY)
        {
            double max = 0;
            for (int i = 0; i < valY.Count(); i++)
            {
                if (max < valY[i]) max = valY[i];
            }
            return max;

        }

        /// <summary>
        /// getMaxValuePositionFrom
        /// </summary>
        /// <param name="valY"></param>
        /// <returns></returns>
        protected int getMaxValuePositionFrom(double[] valY)
        {
            int maxPos = 0;
            double max = 0;
            for (int i = 0; i < valY.Count(); i++)
            {
                if (max < valY[i]) { max = valY[i]; maxPos = i; }
            }
            return maxPos;

        }

        #endregion

        #region [ Configure XAxis and YAxis]

        /// <summary>
        /// configureXAxis
        /// </summary>
        /// <param name="xlsXAxis"></param>
        /// <param name="ChartType"></param>
        /// <param name="excelChartArea"></param>
        /// <param name="excelChart"></param>
        /// <returns></returns>
        protected Axis configureXAxis(ExcelAxis xlsXAxis, int ChartType, ExcelChartArea excelChartArea, ExcelChart excelChart, ExcelTheme xlsTheme)
        {
            Axis xAxis = new Axis();

            //X-Axis Title
            ExcelTitle xAxisTitle = xlsXAxis.getAxisTitle();
            FontFormat xTitleFontFormat = xAxisTitle.getFontFormat();

            // set axis title attributes
            xAxis.Title = xAxisTitle.getText();
            xAxis.TitleFont = new Font(getObjectFontName(xTitleFontFormat, xlsTheme), xTitleFontFormat.getFontSize());
            xAxis.TitleForeColor = getObjectForegroundColor(xTitleFontFormat, xlsTheme);
            xAxis.Name = xAxisTitle.getText();
            xAxis.TitleAlignment = StringAlignment.Center;
            
            // X-Axis 
            xAxis.IsLogarithmic = xlsXAxis.IsLogarithmicScale();
            xAxis.IsLabelAutoFit = true;
            xAxis.IsMarginVisible = true;
            xAxis.IsMarksNextToAxis = isLabelNextToAxis(xlsXAxis.getTickMarkLabel());

            // set X-Axis the label style
            FontFormat xlsXFontFormat = xlsXAxis.getFontFormat();
            AlignmentFormat xlsXAxisAlign = xlsXAxis.getAlignmentFormat();
            LabelStyle lblStyle = new LabelStyle();

            // set axis font and angle attributes
            lblStyle.Font = new Font(getObjectFontName(xlsXFontFormat, xlsTheme), xlsXFontFormat.getFontSize());
            lblStyle.ForeColor = getObjectForegroundColor(xlsXFontFormat, xlsTheme);
            lblStyle.Angle = xlsXAxisAlign.getTextAngle();
            lblStyle.Enabled = true;
            lblStyle.IsEndLabelVisible = true;


            if (lblStyle.Angle != 0) xAxis.IsLabelAutoFit = false;

            //set X-Axis number format
            if (xlsXAxis.getNumberFormat() != "General")
            {
                string xFormat = EasyXLSGridFormat.FormatTheDateAndTime(xlsXAxis.getNumberFormat());
                lblStyle.Format = xFormat != ""?xFormat:xlsXAxis.getNumberFormat();
            }

            // set X-Axis Label Style
            xAxis.LabelStyle = lblStyle;
            xAxis.LabelAutoFitStyle = LabelAutoFitStyles.DecreaseFont;
            // get the interval of the gridlines
            if (!xlsXAxis.IsAutomaticMajorUnit())
            {
                xAxis.Interval = xlsXAxis.getMajorUnit();
            }
            else
                xAxis.Interval = 1;

            // get axis color
            LineColorFormat axisLnColor = xlsXAxis.getLineColorFormat();
            xAxis.LineColor = getObjectLineColor(axisLnColor, xlsTheme, -1);


            // get axis line style
            LineStyleFormat axisLnFormat = xlsXAxis.getLineStyleFormat();
            xAxis.LineDashStyle = getChartLineStyle(axisLnFormat.getDashType());
            xAxis.LineWidth = (int)Unit.Point((int)Math.Ceiling(getObjectWidth(axisLnColor, axisLnFormat, "axis"))).Value;

            xAxis.MajorTickMark.TickMarkStyle = getTickMarkStyle(xlsXAxis.getMajorTickMarkType());
            if (xAxis.MajorTickMark.TickMarkStyle != TickMarkStyle.None)
            {
                xAxis.MajorTickMark.Enabled = true;
                xAxis.MajorTickMark.LineColor = getObjectLineColor(axisLnColor, xlsTheme, -1);
                xAxis.MajorTickMark.LineWidth = (int)Math.Ceiling(getObjectWidth(axisLnColor, axisLnFormat, "tickMarks"));
                xAxis.MajorTickMark.LineDashStyle = getChartLineStyle(axisLnFormat.getDashType());
                // get the interval of the gridlines
                if (!xlsXAxis.IsAutomaticMajorUnit())
                {
                    xAxis.MajorTickMark.Interval = xlsXAxis.getMajorUnit(); ;
                }
            }
            
            // Minor Tickmark
            xAxis.MinorTickMark.TickMarkStyle = getTickMarkStyle(xlsXAxis.getMinorTickMarkType());
            if (xAxis.MajorTickMark.TickMarkStyle != TickMarkStyle.None)
            {
                xAxis.MinorTickMark.Enabled = true;
                xAxis.MinorTickMark.LineColor = getObjectLineColor(axisLnColor, xlsTheme, -1);
                xAxis.MinorTickMark.LineWidth = (int)Math.Ceiling(getObjectWidth(axisLnColor, axisLnFormat, "tickMarks"));
                xAxis.MinorTickMark.LineDashStyle = getChartLineStyle(axisLnFormat.getDashType());
                // get the interval of the gridlines
                if (!xlsXAxis.IsAutomaticMinorUnit())
                {
                    xAxis.MinorTickMark.Interval = xlsXAxis.getMinorUnit(); ;
                }
                else
                {
                    xAxis.MinorTickMark.Interval = 0.5;
                }
            }

            if (!xlsXAxis.IsCrossedBetweenCategories())
            {
                    xAxis.Crossing = xlsXAxis.getCrossValue();
            }

            // check if reversed order
            xAxis.IsReversed = xlsXAxis.IsReverseOrder();

            // set starting point
            if (!xlsXAxis.IsAutomaticMinimumValue())
            {
                xAxis.IsStartedFromZero = (xlsXAxis.getMinimumValue() == 0.0 ? true : false);
            }
            else
            {
                if (ChartType >= EasyXLS.Constants.Chart.CHART_TYPE_LINE && ChartType <= EasyXLS.Constants.Chart.CHART_TYPE_LINE_3D)
                {
                    xAxis.IsStartedFromZero = false;
                }
                else xAxis.IsStartedFromZero = true;
            }

            // check if maximum value is set or is automatic(=0)
            if (!xlsXAxis.IsAutomaticMaximumValue())
            {
                // set maximum value
                xAxis.Maximum = xlsXAxis.getMaximumValue();
            }
            if (!xlsXAxis.IsAutomaticMinimumValue())
            {
                // set minimum value
                xAxis.Minimum = xlsXAxis.getMinimumValue();
            }
            else
            {
                // check if charts are line charts
                if (ChartType >= EasyXLS.Constants.Chart.CHART_TYPE_LINE && ChartType <= EasyXLS.Constants.Chart.CHART_TYPE_LINE_3D)
                {
                    xAxis.Minimum = 1.0;
                }
                else xAxis.Minimum = 0.0;
            }

            // Major X-Grid
            ExcelChartGridlines xlsXAxisMajorGrid = excelChart.easy_getXAxisMajorGridlines();
            LineColorFormat xAxisMajColorFormat = xlsXAxisMajorGrid.getLineColorFormat();
            LineStyleFormat xAxisMajLineFormat = xlsXAxisMajorGrid.getLineStyleFormat();

            Grid majXGrid = new Grid();
            majXGrid.Enabled = xlsXAxisMajorGrid.IsVisible();
            majXGrid.LineColor = getObjectLineColor(xAxisMajColorFormat, xlsTheme, -1);
            majXGrid.LineDashStyle = getChartLineStyle(xAxisMajLineFormat.getDashType());
            majXGrid.LineWidth = (int)Math.Ceiling(getObjectWidth(xAxisMajColorFormat, xAxisMajLineFormat, "gridLines"));
            majXGrid.Interval = xAxis.Interval;
            xAxis.MajorGrid = majXGrid;

            // Minor X-Grid
            ExcelChartGridlines xlsXAxisMinorGrid = excelChart.easy_getXAxisMinorGridlines();
            LineColorFormat xAxisMinColorFormat = xlsXAxisMinorGrid.getLineColorFormat();
            LineStyleFormat xAxisMinLineFormat = xlsXAxisMinorGrid.getLineStyleFormat();
            double minorUnit = xlsXAxis.IsAutomaticMinorUnit() ? 2 : xlsXAxis.getMinorUnit();

            Grid minXGrid = new Grid();
            minXGrid.Enabled = xlsXAxisMinorGrid.IsVisible();
            minXGrid.LineColor = getObjectLineColor(xAxisMinColorFormat, xlsTheme, -1);
            minXGrid.LineDashStyle = getChartLineStyle(xAxisMinLineFormat.getDashType());
            minXGrid.LineWidth = (int)Math.Ceiling(getObjectWidth(xAxisMinColorFormat, xAxisMinLineFormat, "gridLines"));
            minXGrid.Interval = minorUnit;
            xAxis.MinorGrid = minXGrid;

            return xAxis;
        }


        /// <summary>
        /// configureYAxis
        /// </summary>
        /// <param name="xlsYAxis"></param>
        /// <param name="ChartType"></param>
        /// <param name="excelChartArea"></param>
        /// <param name="excelChart"></param>
        /// <returns></returns>
        protected Axis configureYAxis(ExcelAxis xlsYAxis, int ChartType, ExcelChartArea excelChartArea, ExcelChart excelChart, ExcelTheme xlsTheme)
        {
            Axis yAxis = new Axis();

            if (!xlsYAxis.IsAutomaticCategoryAxisCrossesValue())
            {
                    yAxis.Crossing = xlsYAxis.getCrossValue();
            }

            // Y-Axis Title
            ExcelTitle yAxisTitle = xlsYAxis.getAxisTitle();
            FontFormat yTitleFontFormat = yAxisTitle.getFontFormat();

            yAxis.Title = yAxisTitle.getText();
            yAxis.TitleFont = new Font(getObjectFontName(yTitleFontFormat, xlsTheme), yTitleFontFormat.getFontSize());
            yAxis.TitleForeColor = getObjectForegroundColor(yTitleFontFormat, xlsTheme);

            // Y-Axis Features
            yAxis.IsLabelAutoFit = true;
            yAxis.IsMarksNextToAxis = isLabelNextToAxis(xlsYAxis.getTickMarkLabel());

            // set the label style
            // set X-Axis the label style
            FontFormat xlsYFontFormat = xlsYAxis.getFontFormat();
            AlignmentFormat xlsYAxisAlign = xlsYAxis.getAlignmentFormat();
            LabelStyle lblYStyle = new LabelStyle();

            lblYStyle.Font = new Font(getObjectFontName(xlsYFontFormat, xlsTheme), xlsYFontFormat.getFontSize());
            lblYStyle.ForeColor = getObjectForegroundColor(xlsYFontFormat, xlsTheme);
            lblYStyle.Angle = xlsYAxisAlign.getTextAngle();
            lblYStyle.Enabled = true;

            if (xlsYAxis.getNumberFormat() != "General")
            {
                // check if percent
                if (xlsYAxis.getNumberFormat().Contains("%"))
                {
                    int percentPos = xlsYAxis.getNumberFormat().IndexOf("%");
                    string fixFormat = "{" + xlsYAxis.getNumberFormat().Substring(0, percentPos) + "}%";
                    lblYStyle.Format = fixFormat;
                }
                else
                {
                    lblYStyle.Format = xlsYAxis.getNumberFormat();
                }
            }

            if (lblYStyle.Angle != 0) yAxis.IsLabelAutoFit = false;

            yAxis.LabelStyle = lblYStyle;
            yAxis.LabelAutoFitStyle = LabelAutoFitStyles.DecreaseFont;
            yAxis.LabelAutoFitMinFontSize = 5;

            // get the interval of the gridlines
            if (!xlsYAxis.IsAutomaticMajorUnit())
            {
                yAxis.Interval = xlsYAxis.getMajorUnit();
            }
            //else yAxis.Interval = 1;

            // get axis color
            LineColorFormat axisLnColor = xlsYAxis.getLineColorFormat();
            yAxis.LineColor = getObjectLineColor(axisLnColor, xlsTheme, -1);

            // get axis line style
            LineStyleFormat axisLnFormat = xlsYAxis.getLineStyleFormat();
            yAxis.LineDashStyle = getChartLineStyle(axisLnFormat.getDashType());
            yAxis.LineWidth = (int)Math.Ceiling(getObjectWidth(axisLnColor, axisLnFormat, "axis"));

            //Major tickMark
            yAxis.MajorTickMark.TickMarkStyle = getTickMarkStyle(xlsYAxis.getMajorTickMarkType());
            if (yAxis.MajorTickMark.TickMarkStyle != TickMarkStyle.None)
            {
                yAxis.MajorTickMark.Enabled = true;
                yAxis.MajorTickMark.LineColor = getObjectLineColor(axisLnColor, xlsTheme, -1);
                yAxis.MajorTickMark.LineWidth = (int)Math.Ceiling(getObjectWidth(axisLnColor, axisLnFormat, "tickMarks"));
                yAxis.MajorTickMark.LineDashStyle = getChartLineStyle(axisLnFormat.getDashType());
                // get the interval of the gridlines
                if (!xlsYAxis.IsAutomaticMajorUnit())
                {
                    yAxis.MajorTickMark.Interval = xlsYAxis.getMajorUnit(); ;
                }
            }

            // Minor tickMark
            yAxis.MinorTickMark.TickMarkStyle = getTickMarkStyle(xlsYAxis.getMinorTickMarkType());
            if (yAxis.MinorTickMark.TickMarkStyle != TickMarkStyle.None)
            {
                yAxis.MinorTickMark.Enabled = true;
                yAxis.MinorTickMark.LineColor = getObjectLineColor(axisLnColor, xlsTheme, -1);
                yAxis.MinorTickMark.LineWidth = (int)Math.Ceiling(getObjectWidth(axisLnColor, axisLnFormat, "tickMarks"));
                yAxis.MinorTickMark.LineDashStyle = getChartLineStyle(axisLnFormat.getDashType());
                // get the interval of the gridlines
                if (!xlsYAxis.IsAutomaticMinorUnit())
                {
                    yAxis.MinorTickMark.Interval = xlsYAxis.getMinorUnit(); ;
                }
                else
                {
                    yAxis.MinorTickMark.Interval = 2;
                }
            }

            yAxis.IsReversed = xlsYAxis.IsReverseOrder();

            // check if maximum value is set or is automatic(=0)
            if (!xlsYAxis.IsAutomaticMaximumValue())
            {
                // set maximum value
                yAxis.Maximum = xlsYAxis.getMaximumValue();
            }
            if (xlsYAxis.IsAutomaticMinimumValue())
            {
                // set minimum value
                yAxis.Minimum = xlsYAxis.getMinimumValue();
            }

            yAxis.IsLogarithmic = xlsYAxis.IsLogarithmicScale();

            // Major Y-Grid
            ExcelChartGridlines xlsYAxisMajorGrid = excelChart.easy_getYAxisMajorGridlines();
            LineColorFormat yAxisMajColorFormat = xlsYAxisMajorGrid.getLineColorFormat();
            LineStyleFormat yAxisMajLineFormat = xlsYAxisMajorGrid.getLineStyleFormat();

            Grid majYGrid = new Grid();
            majYGrid.Enabled = xlsYAxisMajorGrid.IsVisible();
            majYGrid.Interval = yAxis.Interval;
            majYGrid.LineColor = getObjectLineColor(yAxisMajColorFormat, xlsTheme, -1);
            majYGrid.LineWidth = (int)Math.Ceiling(getObjectWidth(yAxisMajColorFormat, yAxisMajLineFormat, "gridLines"));
            majYGrid.LineDashStyle = getChartLineStyle(yAxisMajLineFormat.getDashType());
            yAxis.MajorGrid = majYGrid;

            // Minor Y-Grid
            ExcelChartGridlines xlsYAxisMinorGrid = excelChart.easy_getYAxisMinorGridlines();
            LineColorFormat yAxisMinColorFormat = xlsYAxisMinorGrid.getLineColorFormat();
            LineStyleFormat yAxisMinLineFormat = xlsYAxisMinorGrid.getLineStyleFormat();
            double minorUnit = xlsYAxis.IsAutomaticMinorUnit() ? 2 : xlsYAxis.getMinorUnit();

            Grid minYGrid = new Grid();
            minYGrid.Enabled = xlsYAxisMinorGrid.IsVisible();
            minYGrid.LineColor = getObjectLineColor(yAxisMinColorFormat, xlsTheme, -1);
            minYGrid.LineWidth = (int)Math.Ceiling(getObjectWidth(yAxisMinColorFormat, yAxisMinLineFormat, "gridLines"));
            minYGrid.LineDashStyle = getChartLineStyle(yAxisMinLineFormat.getDashType());
            minYGrid.Interval = yAxis.MinorTickMark.Interval;
            yAxis.MinorGrid = minYGrid;

            if (xlsYAxis.getNumberFormat() == "General")
            {
                // if chart type is 100% Stacked
                if (Is100Stacked(ChartType))
                {
                    yAxis.LabelStyle.Format = "{0}%";
                }
            }

            return yAxis;

        }

        #endregion

        #region [ Common Methods for charts ]

        /// <summary>
        /// GetTheMinimumPosition
        /// </summary>
        /// <param name="chartIdx"></param>
        /// <returns></returns>
        protected int GetTheMinimumPosition(Dictionary<int, int[]> chartIdx)
        {
            int minKey = 0;
            int minValKeyCol = int.MaxValue;
            int minValKeyRow = int.MaxValue;

            foreach (int key in chartIdx.Keys)
            {
                int[] keyVal = chartIdx[key];
                //if col 
                if ((keyVal[0] < minValKeyCol && keyVal[1] < minValKeyRow)
                    ||
                    (keyVal[1] < minValKeyRow)
                    ||
                    (keyVal[0] < minValKeyCol && keyVal[1] == minValKeyRow)
                    ) { minValKeyCol = keyVal[0]; minValKeyRow = keyVal[1]; minKey = key; }

            }

            return minKey;
        }

        /// <summary>
        /// FindMinimumScale
        /// </summary>
        /// <param name="array"></param>
        /// <returns></returns>
        private double FindMinimumScale(double[] array)
        {
            double MinScale = double.MaxValue;

            for (int idx = 0; idx < array.Count(); idx++)
            {
                if (array[idx] < MinScale) MinScale = array[idx];
            }
            return MinScale;

        }

        /// <summary>
        /// FindMaximumScale
        /// </summary>
        /// <param name="array"></param>
        /// <returns></returns>
        private double FindMaximumScale(double[] array)
        {
            double MaxScale = double.MinValue;

            for (int idx = 0; idx < array.Count(); idx++)
            {
                if (array[idx] > MaxScale) MaxScale = array[idx];
            }
            return MaxScale;
        }
       

        #endregion

        #region [ Build Chart and save it as Photo on the server ]

        /// <summary>
        /// get all charts one by one and save chart info and timpestamp for the chart images saved
        /// </summary>
        /// <param name="chartIndex"></param>
        /// <param name="excelDoc"></param>
        /// <returns></returns>
        public KeyValuePair<System.Web.UI.DataVisualization.Charting.Chart, string> getChartFromSheet(int chartIndex, ExcelDocument excelDoc, int SheetNumber, bool isDashboard)
        {
            KeyValuePair<System.Web.UI.DataVisualization.Charting.Chart, string> chartInfo;

            ExcelChart excelChart;
            ExcelSheet excelSheet = excelDoc.easy_getSheetAt(SheetNumber);
            ExcelTheme excelTheme = excelDoc.getTheme();

            if (excelSheet is ExcelWorksheet)
            {
                ExcelWorksheet excelWorksheet = (ExcelWorksheet)excelSheet;
                // get excel worksheet chart
                excelChart = excelWorksheet.easy_getChartAt(chartIndex);
            }
            else
            {
                ExcelChartSheet excelChartsheet = (ExcelChartSheet)excelSheet;
                // get excel worksheet chart
                excelChart = excelChartsheet.easy_getExcelChart();
            }

            ExcelChartArea excelChartArea = excelChart.easy_getChartArea();
            int ChartType = excelChart.easy_getChartType();

            // set .net chart 
            System.Web.UI.DataVisualization.Charting.Chart chart = new System.Web.UI.DataVisualization.Charting.Chart();
            ChartArea chartArea = new ChartArea();

            ExcelPlotArea xlsPlotArea = excelChart.easy_getPlotArea();
            //if (xlsPlotArea.getHeight() > 0)
           // {
                //chartArea.InnerPlotPosition = new  ElementPosition(25, 0, 80, 80);//excelChart.getHeight()-20;//xlsPlotArea.getHeight();
            //}

           // if (xlsPlotArea.getWidth() > 0)
            //{
               // chartArea.InnerPlotPosition = new ElementPosition(25, 0, 80, 80); //excelChart.getWidth() - 20;
            //}

            FillFormat chartAreaFillF = excelChartArea.getFillFormat();
            chart.BackColor = getObjectBackColor(chartAreaFillF, excelTheme, -1);

           

            #region [ Set Chart Area ]
            /*************************************  SET CHART AREA  ************************************/
            // set .net chart area
            FillFormat plotAreaFillF = xlsPlotArea.getFillFormat();
            if (plotAreaFillF.getBackground().IsEmpty)
            {
                chartArea.BackColor = chart.BackColor;
            }
            else
            {
                chartArea.BackColor = getObjectBackColor(plotAreaFillF, excelTheme, -1);
            }

            chart.BorderSkin.SkinStyle = BorderSkinStyle.None;

            //chartArea.InnerPlotPosition = new ElementPosition(0, 0, 100, 100);

            // check if dashboard
            if (isDashboard)
            {
                chart.BorderSkin.SkinStyle = BorderSkinStyle.None;
                chart.BorderSkin.BackColor = System.Drawing.Color.Gray;
                chart.BorderSkin.PageColor = System.Drawing.Color.Gray;
                chart.BorderlineDashStyle = ChartDashStyle.Solid;
                chart.BorderlineWidth = 1;
                chart.BorderlineColor = Color.Gray;
            }


            LineColorFormat lnColor = excelChartArea.getLineColorFormat();
            LineStyleFormat lnStyle = excelChartArea.getLineStyleFormat();
            chartArea.BorderColor = getObjectLineColor(lnColor, excelTheme, -1);
            //chartArea.BorderWidth = getDashType(lnStyle.getDashType());//.getBorderWeight();
            chartArea.Name = chartIndex.ToString();
            chart.AlignDataPointsByAxisLabel(PointSortOrder.Descending);

            #endregion

            #region [ Set Chart Title ]

            ExcelTitle chartTitle = excelChart.getChartTitle();
            AlignDrawingObject xlsChartTitle = chartTitle;
            Title webChartTitle = new Title();

            // alignment
            AlignmentFormat alForm = xlsChartTitle.getAlignmentFormat();
            webChartTitle.Alignment = getAlignment(xlsChartTitle.getAlignmentFormat());

            // font format
            FontFormat fontFormat = xlsChartTitle.getFontFormat();
            FontStyle titleFontStyle = getFontStyle(fontFormat);
            float fontTitleSize = CHART_TITLE_AUTOMATIC_FONT_SIZE;
            FontStyle fontTitleStyle = CHART_TITLE_AUTOMATIC_FONT_STYLE;

            if (!fontFormat.IsAutomaticFontSize()) fontTitleSize = fontFormat.getFontSize();
            if (!fontFormat.IsAutomaticFontStyle()) fontTitleStyle = titleFontStyle;

            Font titleFont = new Font(getObjectFontName(fontFormat, excelTheme), fontTitleSize, fontTitleStyle);
            webChartTitle.ForeColor = getObjectForegroundColor(fontFormat, excelTheme);
            webChartTitle.Font = titleFont;

            // fill format
            FillFormat fillFormat = xlsChartTitle.getFillFormat();
            webChartTitle.BackColor = getObjectBackColor(fillFormat, excelTheme, -1);
            if (fillFormat.getBackground().IsEmpty)
                webChartTitle.BackColor = chart.BackColor;

            webChartTitle.Text = chartTitle.getText();

            //webChartTitle.DockedToChartArea = chartArea.Name;
            chart.Titles.Add(webChartTitle);

            #endregion

            #region [ Set Chart Area 3D Style ]

            /*************************************  SET CHART AREA 3D STYLE  ************************************/
            ChartArea3DStyle areaStyle = new ChartArea3DStyle(chartArea);
            areaStyle.Enable3D = !excelChart.easy_Use3DPerspective();

            // set 3D data only for charts using 3D
            if (Is3DChart(ChartType))
            {
                //set 3D perspective
                areaStyle.Perspective = excelChart.easy_get3DPerspective() / 2;
                //set 3D inclination
                areaStyle.Inclination = excelChart.easy_get3DElevation();
                //set 3D rotation
                areaStyle.Rotation = excelChart.easy_get3DRotation();
                //enable 3D Viewing
                areaStyle.Enable3D = true;

                //when 3D set clustered false
                areaStyle.IsClustered = false;

                areaStyle.IsRightAngleAxes = !excelChart.easy_Use3DPerspective();
            }
            else
            {
                // for the rest set the clustered to true
                areaStyle.IsClustered = true;
            }

            if (IsClusteredChart(ChartType))
            {
                // for the rest set the clustered to true
                areaStyle.IsClustered = true;
            }

            // set lighting
            areaStyle.LightStyle = LightStyle.Simplistic;

            // check if pie chart
            if (!IsPieChart(ChartType) || IsDoughnutChart(ChartType))
            {
                // set depth on both dimensions
                areaStyle.PointGapDepth = excelChart.easy_getSeriesGap();// excelChart.easy_get3DHeight();
            }
            else
            {
                //set 3D perspective
                areaStyle.Perspective = excelChart.easy_get3DPerspective();
                // axis x angle
                areaStyle.Inclination = excelChart.easy_get3DElevation()*2;
                // axis y angle
                areaStyle.Rotation = excelChart.easy_get3DRotation();
            }

            // add 3D style area to chart area object
            chartArea.Area3DStyle = areaStyle;

            #endregion

            #region [ Set X Axis Properties ]
            /*************************************  Set X Axis Properties  ************************************/
            // get Axis
            ExcelAxis xlsXAxis = excelChart.easy_getCategoryXAxis();
            Axis xAxis = new Axis();
            xAxis = configureXAxis(xlsXAxis, ChartType, excelChartArea, excelChart, excelTheme);
            chartArea.AxisX = xAxis;

            #endregion

            #region [ Set Y Axis Properties ]

            /*************************************  Set Y Axis Properties  ************************************/
            ExcelAxis xlsYAxis = excelChart.easy_getValueYAxis();
            Axis yAxis = new Axis();
            yAxis = configureYAxis(xlsYAxis, ChartType, excelChartArea, excelChart, excelTheme);
            chartArea.AxisY = yAxis;
            // add chart area to the chart
            chart.ChartAreas.Add(chartArea);
            
            #endregion

            #region [ Maximum Crossing ]

            // if maximum cross 
            if (xlsYAxis.IsMaximumCross())
            {
                // set th right crossing
                yAxis.Crossing = Double.MaxValue;

                if (xAxis.MajorTickMark.TickMarkStyle == TickMarkStyle.InsideArea)
                {
                    xAxis.MajorTickMark.TickMarkStyle = TickMarkStyle.OutsideArea;
                }

                if (xAxis.MajorTickMark.TickMarkStyle == TickMarkStyle.OutsideArea)
                {
                    xAxis.MajorTickMark.TickMarkStyle = TickMarkStyle.AcrossAxis;
                }

            }

            // check if charts are NOT STOCK VOLUME Charts
            if (ChartType != EasyXLS.Constants.Chart.CHART_TYPE_STOCK_VOLUME_HIGH_LOW_CLOSE && ChartType != EasyXLS.Constants.Chart.CHART_TYPE_STOCK_VOLUME_OPEN_HIGH_LOW_CLOSE)
            {
                // if maximum cross 
                if (xlsXAxis.IsMaximumCross())
                {
                    // set th right crossing
                    xAxis.Crossing = Double.MaxValue;

                    if (yAxis.MajorTickMark.TickMarkStyle == TickMarkStyle.InsideArea)
                    {
                        yAxis.MajorTickMark.TickMarkStyle = TickMarkStyle.OutsideArea;
                    }

                    if (yAxis.MajorTickMark.TickMarkStyle == TickMarkStyle.OutsideArea)
                    {
                        yAxis.MajorTickMark.TickMarkStyle = TickMarkStyle.InsideArea;
                    }
                }
            }

            #endregion

          

            #region [ Get X Axis Data Values ]

            // get the formula interpretation
            Formula formula = new Formula();

            // get formula for X axis
            string formulaXValues = excelChart.easy_getCategoryXAxisLabels();

            // the X Values container
            object[] elementValX = new object[0];
            ExcelTable xlsTable = new ExcelTable();
            int XArrayCount = 0;


            // if x values in the document
            if (formulaXValues != "")
            {
                string SheetName = "";
                int indexOfEqual = formulaXValues.IndexOf("=") + 1;
                int indexOfExclam = formulaXValues.IndexOf("!");
                SheetName = formulaXValues.Substring(indexOfEqual, indexOfExclam - indexOfEqual);

                // check if sheetName contins special characters
                if (SheetName.Contains("'"))
                    SheetName = SheetName.Replace("'", "");

                // get the document table
                xlsTable = ((ExcelWorksheet)excelDoc.easy_getSheet(SheetName)).easy_getExcelTable();

                if (formulaXValues.StartsWith("="))
                    formulaXValues = formulaXValues.Remove(0, 1);

                // this gets the left and the right corner of the XLS series table values 
                object[] elementProp = formula.get3DRangeElements(formulaXValues);

                // start col/row
                int startCol = int.Parse(elementProp[1].ToString());
                int startRow = int.Parse(elementProp[2].ToString());

                // stop col and row
                int stopCol = int.Parse(elementProp[3].ToString());
                int stopRow = int.Parse(elementProp[4].ToString());

                // get the number of values
                XArrayCount = (stopCol - startCol + 1) * (stopRow - startRow + 1);

                // the X Values container
                elementValX = new object[XArrayCount];

                // iterate through the excel table from start to stop
                int counter = 0;
                for (int i = startRow; i <= stopRow; i++)
                {
                    for (int j = startCol; j <= stopCol; j++)
                    {
                        // check if charts are SCATTER Charts
                        if (ChartType >= EasyXLS.Constants.Chart.CHART_TYPE_SCATTER && ChartType <= EasyXLS.Constants.Chart.CHART_TYPE_SCATTER_LINES)
                        {
                            int valParse = 0;
                            int.TryParse(xlsTable.easy_getCellAt(i, j).getValue(), out valParse);
                            elementValX[counter++] = valParse;
                        }
                        else
                        {
                            // add it to the array
                            elementValX[counter++] = xlsTable.easy_getCellAt(i, j).getValue();
                        }
                    }
                }
            }
            else
            {
                // if values are automatic the interval should be 1
                xAxis.Interval = 1;
            }

            if (xAxis.LabelStyle.Format.Contains("/"))
            {
                double minVal = double.MaxValue;
                for (int idx = 0; idx < elementValX.Count(); idx++)
                {
                    DateTime x = DateTime.Parse(elementValX[idx].ToString());
                    double dateTimeToDouble = x.ToOADate();
                    elementValX[idx] = dateTimeToDouble;
                    if (minVal > dateTimeToDouble) minVal = dateTimeToDouble;
                }

                //set the minimum value
                xAxis.Minimum = minVal;

            }


            #endregion

            ExcelChartLegend xlsChartLegend = excelChart.easy_getLegend();

            #region [ Set The Chart Series ]

            Color[] myThemeColorPalette = new Color[100];
      

            /*************************************  Set the Chart Series ************************************/

            // get Y Values
            double[][] valuesY = new double[excelChart.SeriesCount()][];
            // the maximum number of points in a series
            int largestCount = 0;
            // the maximum value in a series
            double MaximumInSeries = 0;
            // the position for the Maximum
            int MaximumInSeriesPos = 0;

            Array[] yStockObj = new Array[excelChart.SeriesCount()];

            if (IsStockChart(ChartType))
            {
                MaximumInSeries = 0;
            }

            bool reverseOrder = false;
            if (IsClusteredBarChart(ChartType) 
                || ChartType == EasyXLS.Constants.Chart.CHART_TYPE_LINE_3D 
                || Is3DColumnChart(ChartType)
                || ChartType == EasyXLS.Constants.Chart.CHART_TYPE_AREA_3D
                )
            {
                reverseOrder = true;
            }

            int startIndex = excelChart.SeriesCount() - 1;
            int index = 0;
            // iterate trough excel series
            while (index < excelChart.SeriesCount())
            {
                int seriesId = index;
                if (reverseOrder) seriesId = startIndex - index;

                // excel series
                ExcelSeries xlsSeries = excelChart.easy_getSeriesAt(seriesId);
                Series series = new Series(xlsSeries.getName());

                // set the chart type
                series.ChartType = getChartType(ChartType);

                if (xAxis.LabelStyle.Format.Contains("/"))
                {
                    series.XValueType = ChartValueType.Date;
                }

                #region [ Get Y Axis Data Values ]

                // the Y Values container
                double[] elementValY = new double[0];

                // get Y Values formula
                string formulaYValues = xlsSeries.getValues();
                if (formulaYValues == "={1}")
                {
                    formulaYValues = xlsSeries.getName();
                }

                if (formulaYValues != "")
                {
                    

                    string SheetName = "";
                    int indexOfEqual = formulaYValues.IndexOf("=") + 1;
                    int indexOfExclam = formulaYValues.IndexOf("!");
                    SheetName = formulaYValues.Substring(indexOfEqual, indexOfExclam - indexOfEqual);

                    // check if sheetName contins special characters
                    if (SheetName.Contains("'"))
                        SheetName = SheetName.Replace("'", "");

                    ExcelWorksheet xlsWorkSheet = (ExcelWorksheet)excelDoc.easy_getSheet(SheetName);
                    xlsWorkSheet.easy_computeFormulas(excelDoc, true);

                    // get the document table
                    xlsTable = xlsWorkSheet.easy_getExcelTable();
                    

                    if (formulaYValues.StartsWith("="))
                        formulaYValues = formulaYValues.Remove(0, 1);

                    // this gets the left and the right corner of the XLS series table values 
                    object[] elementProp = formula.get3DRangeElements(formulaYValues);

                    // start col/row
                    int startCol = int.Parse(elementProp[1].ToString());
                    int startRow = int.Parse(elementProp[2].ToString());

                    // stop col and row
                    int stopCol = int.Parse(elementProp[3].ToString());
                    int stopRow = int.Parse(elementProp[4].ToString());

                    // get the number of values
                    int arrayCount = (stopCol - startCol + 1) * (stopRow - startRow + 1);

                    if (arrayCount < XArrayCount) elementValY = new double[XArrayCount];
                    else elementValY = new double[arrayCount];

                    // iterate through the excel table from start to stop
                    int counter = 0;
                    for (int i = startRow; i <= stopRow; i++)
                    {
                        for (int j = startCol; j <= stopCol; j++)
                        {
                            // get the excel value
                            double valParse = 0;


                            if (!double.TryParse(xlsTable.easy_getCellAt(i, j).getValue(), out valParse))
                            {
                                double.TryParse(xlsTable.easy_getCellAt(i, j).getFormulaResultValue(), out valParse);
                            }
                            

                            // add it to the array
                            elementValY[counter++] = valParse;
                        }
                    }
                }

                valuesY[seriesId] = elementValY;
                if (largestCount < valuesY[seriesId].Count()) largestCount = valuesY[seriesId].Count();

                // check if Charts are Line Stacked OR 100 Stacked OR Bar Charts
                if (!Is100Stacked(ChartType) && !IsBarChart(ChartType))
                {

                    if (MaximumInSeries < getMaxValueFrom(valuesY[seriesId]))
                    {
                        MaximumInSeries = getMaxValueFrom(valuesY[seriesId]);
                        MaximumInSeriesPos = getMaxValuePositionFrom(valuesY[seriesId]);
                    }
                    if (index + 1 == excelChart.SeriesCount())
                    {
                        int calculatedInterval = 0;
                        if (xlsYAxis.IsAutomaticMajorUnit())
                        {
                            if (IsColumnStackedChart(ChartType)
                                || ChartType == EasyXLS.Constants.Chart.CHART_TYPE_LINE_STACKED 
                                || ChartType == EasyXLS.Constants.Chart.CHART_TYPE_LINE_STACKED_MARKERS
                                || ChartType == EasyXLS.Constants.Chart.CHART_TYPE_AREA_STACKED
                                || ChartType == EasyXLS.Constants.Chart.CHART_TYPE_AREA_3D_STACKED)
                            {
                                MaximumInSeries = 0;
                                for(int seriesIdx=0;seriesIdx< excelChart.SeriesCount();seriesIdx ++)
                                {
                                    MaximumInSeries += valuesY[seriesIdx][MaximumInSeriesPos];
                                }
                            }

                            // get the height of the chart
                            double chartHeight = (double)excelChart.getHeight();

                            int noOfTickMarks = (int)Math.Floor(chartHeight/AXIS_LABEL_SPACE_PX);

                            // get interval
                            calculatedInterval = (int)MaximumInSeries/noOfTickMarks;
                            int auxCalcInterval = calculatedInterval;
                            int modCalcInterval = 0;
                            int noOfZeros = 1;

                            while (auxCalcInterval > 0)
                            {
                                noOfZeros *= 10;
                                modCalcInterval = auxCalcInterval % 10;
                                auxCalcInterval /= 10;
                            }

                            noOfZeros /= 10;
                           

                            for (int i = 0; i < 3; i++)
                            {
                                if (modCalcInterval >= 5)
                                {
                                    calculatedInterval = automaticInterval[0] * noOfZeros * 10;
                                    break;
                                }
                                if (modCalcInterval < automaticInterval[i])
                                {
                                    calculatedInterval = automaticInterval[i] * noOfZeros;
                                    break;
                                }
                            }

                           yAxis.Interval = calculatedInterval;

                            //if (MaximumInSeries < 1500) yAxis.Interval = 200;
                            //if (MaximumInSeries < 800) yAxis.Interval = 100;
                            //if (MaximumInSeries < 300) yAxis.Interval = 50;
                            //if (MaximumInSeries < 100) yAxis.Interval = 10;
                        }

                        if (xlsYAxis.IsAutomaticMaximumValue())
                        {
                            if (MaximumInSeries % yAxis.Interval == 0)
                            {
                                yAxis.Maximum = MaximumInSeries;
                            }
                            else
                            {
                                yAxis.Maximum = MaximumInSeries + yAxis.Interval;
                            }
                        }

                        if (calculatedInterval == 0 && xlsYAxis.IsAutomaticMajorUnit())
                        {
                            yAxis.IsStartedFromZero = true;
                            yAxis.Minimum = 0;
                            yAxis.IntervalType = DateTimeIntervalType.Number;

                            yAxis.Interval = 1;
                            yAxis.MajorGrid.Interval = 1;
                            yAxis.MajorTickMark.Interval = 1;
                            yAxis.Maximum = MaximumInSeries + yAxis.Interval;
                        }
                    }
                }

                #endregion

                #region [ Get The Series Name ]

                // get the name of the series
                string seriesName = series.Name;
                if (seriesName != "")
                {
                    series.Name = GetSeriesName(seriesName, excelDoc, formula, xlsTable, seriesId);

                    // keep series name in legend if not Dougnut or Pie Charts
                    if (!IsDoughnutChart(ChartType) && !IsPieChart(ChartType))
                    {
                        if (xlsChartLegend.IsVisible())
                        {
                            series.LegendText = series.Name;
                        }
                    }
                }
                #endregion


                #region [ Get Series Appearance ]

                // series fill format
                FillFormat seriesFillFormat = xlsSeries.getFillFormat();
                LineColorFormat seriesLineColFormat = xlsSeries.getLineColorFormat();

                Color backGroundColor = getObjectBackColor(seriesFillFormat, excelTheme, seriesId);
                Color lineColor = getObjectLineColor(seriesLineColFormat, excelTheme, seriesId);

                if (IsColumnChart(ChartType) || IsBarChart(ChartType)||Is3DChart(ChartType) || IsAreaChart(ChartType) || ChartType == EasyXLS.Constants.Chart.CHART_TYPE_RADAR_FILLED)
                {
                    // color series
                    if (!backGroundColor.IsEmpty)
                    {
                        series.Color = backGroundColor;
                    }
                }
                else
                {
                    // color series
                    if (!lineColor.IsEmpty)
                    {
                        series.Color = lineColor;
                    }
                }
                // line style series
                LineStyleFormat seriesLineStyleFormat = xlsSeries.getLineStyleFormat();
                series.BorderDashStyle = getChartLineStyle(seriesLineStyleFormat.getDashType());
                series.BorderWidth = (int)Unit.Point((int)Math.Ceiling(getObjectWidth(seriesLineColFormat, seriesLineStyleFormat, "border"))).Value;
                series.BorderColor = lineColor.IsEmpty ? backGroundColor : lineColor;

                #endregion

                #region [ Set Custom Properties To Chart Types ]
                /******************* CUSTOM CHART TYPES ********************/

                // CHART TYPE CYLINDER COLUMN/BAR
                if (IsCylinderChart(ChartType))
                {
                    series["DrawingStyle"] = "Cylinder";
                    series.MarkerSize = 0;
                }


                // ONLY FOR LINE CHART TYPE AND RADAR CHARTS
                if (
                        (ChartType >= EasyXLS.Constants.Chart.CHART_TYPE_LINE && ChartType < EasyXLS.Constants.Chart.CHART_TYPE_LINE_3D)
                        ||
                        ChartType == EasyXLS.Constants.Chart.CHART_TYPE_RADAR || ChartType == EasyXLS.Constants.Chart.CHART_TYPE_RADAR_MARKERS
                    )
                {
                    // color only the line chart this way
                    series.BorderColor = backGroundColor.IsEmpty ? lineColor : backGroundColor;
                    series.Color = backGroundColor.IsEmpty ? lineColor : backGroundColor;
                    series.BorderWidth = (int)Unit.Point((int)Math.Ceiling(getObjectWidth(seriesLineColFormat, seriesLineStyleFormat, "series"))).Value;
                }


                // CHART TYPE STACKED LINES AND RADAR (use Area charts and make the content transparent)
                if (
                    ChartType == EasyXLS.Constants.Chart.CHART_TYPE_LINE_STACKED
                    || ChartType == EasyXLS.Constants.Chart.CHART_TYPE_LINE_100_PERCENT_STACKED
                    || ChartType == EasyXLS.Constants.Chart.CHART_TYPE_LINE_STACKED_MARKERS
                    || ChartType == EasyXLS.Constants.Chart.CHART_TYPE_LINE_100_PERCENT_STACKED_MARKERS
                    || ChartType == EasyXLS.Constants.Chart.CHART_TYPE_RADAR
                    || ChartType == EasyXLS.Constants.Chart.CHART_TYPE_RADAR_MARKERS
                    )
                {
                    // trick for getting the stacked line charts
                    series.Color = Color.Transparent;
                }



                // CHART TYPE RADAR MARKS
                if (ChartType == EasyXLS.Constants.Chart.CHART_TYPE_RADAR_MARKERS)
                {
                    // set the marker color as the line color
                    series.MarkerColor = lineColor;
                }

                //if 3D
                if (!areaStyle.Enable3D)
                {
                    series["PointWidth"] = getPointWidthByGap(excelChart.easy_getSeriesGap()).ToString();
                }
                else
                {
                    //seriesGap * 100(percent)/500(maximum)
                    int gapBetweenPoints = excelChart.easy_getSeriesGap() / 5;
                    //series["PointWidth"] = (1-gapBetweenPoints/100.0).ToString();//getPointWidthByGap(210).ToString();
                    series["PointWidth"] = getPointWidthByGap(210).ToString();
                }

                // check if chart type
                if (IsPieChart(ChartType) || IsDoughnutChart(ChartType))
                {
                    string pieDoughnutChart = chart.Titles[0].Text;
                    if (pieDoughnutChart == "Chart Title" || pieDoughnutChart == "")
                    {
                        chart.Titles[0].Text = series.Name;
                    }

                    // get angle of first slice in pie doughnut charts
                    int angleOfFirstSlice = excelChart.getFirstSliceAngle();
                    int finalAngle = 270 + angleOfFirstSlice;
                    series["PieStartAngle"] = finalAngle.ToString();


                    if ((IsDoughnutChart(ChartType) || IsPieChart(ChartType)) && elementValX.Count() == 0)
                    {
                    }
                    else
                    {
                        // set the legend as being the value
                        series.LegendText = "#VALX";
                    }

                    // check if the data labels are disabled
                    if (xlsSeries.getDataLabels() != null)
                    {
                        // get the way it should show the labels of the pie (percentage/ values/ name)
                        series.Label = getDataLabelStyle(xlsSeries.getDataLabels());
                        //series.SmartLabelStyle.Enabled = true;
                        //series.SmartLabelStyle.AllowOutsidePlotArea = LabelOutsidePlotAreaStyle.No;
                        //series.SmartLabelStyle.IsOverlappedHidden = false;
                        series["PieLabelStyle"] = "outside";
                    }
                    else
                    {
                        series["PieLabelStyle"] = "Disabled";
                    }
                }

                // check if the data labels are disabled
                if (xlsSeries.getDataLabels() != null)
                { // get the way it should show the labels of the pie (percentage/ values/ name)
                    series.Label = getDataLabelStyle(xlsSeries.getDataLabels());
                }

                // do not show markers for the area,bar and column charts
                if (!IsAreaChart(ChartType) && !IsPieChart(ChartType) && !IsBarChart(ChartType) && !IsColumnChart(ChartType) && !IsBubbleChart(ChartType) && ChartType != EasyXLS.Constants.Chart.CHART_TYPE_LINE_3D && !IsSurfaceChart(ChartType))
                {
                    // SHOW MARKERS IF ANY
                    FillFormat markerFillFormat = xlsSeries.getMarkerFillFormat();
                    LineColorFormat markerLineColorFormat = xlsSeries.getMarkerLineColorFormat();
                    LineStyleFormat markerLineStyleFormat = xlsSeries.getMarkerLineStyleFormat();

                    // set marker color
                    Color markerColor = lineColor;

                    // check if automatic
                    if (markerFillFormat.IsAutomatic())
                        markerColor = getObjectBackColor(markerFillFormat, excelTheme, seriesId);
                    else
                        if (markerFillFormat.IsNoFill()) // check if no fill
                            markerColor = Color.Empty;
                        else 
                            if (markerFillFormat.IsSolidFill()) // check if solid fill
                            markerColor = getObjectBackColor(markerFillFormat, excelTheme, seriesId);

                    // set marker color
                    series.MarkerColor = markerColor;

                    // set marker border color
                    Color markerBorderColor = lineColor;

                    // check if automatic
                    if (markerLineColorFormat.IsAutomatic())
                        markerBorderColor = getObjectBackColor(markerFillFormat, excelTheme, seriesId);
                    else
                        if (markerLineColorFormat.IsNoLine()) // check if no fill
                            markerBorderColor = Color.Empty;
                        else
                            if (markerLineColorFormat.IsSolidLine()) // check if solid fill
                                markerBorderColor = getObjectLineColor(markerLineColorFormat, excelTheme, seriesId);

                    // set marker border color
                    series.MarkerBorderColor = markerBorderColor;
                    series.MarkerBorderWidth = (int)Math.Ceiling(getObjectWidth(markerLineColorFormat, markerLineStyleFormat, "markers"));

                    if (xlsSeries.getMarkerType() ==  EasyXLS.Constants.Chart.SERIES_MARKER_AUTOMATIC)
                    {
                        // set marker style
                        series.MarkerStyle = MarkerStyle.Diamond;
                         // set marker style
                        series.MarkerSize = 10;
                    }
                    else
                    {
                        // set marker style
                        series.MarkerStyle = getSeriesMarker(xlsSeries.getMarkerType()); 
                        series.MarkerSize = xlsSeries.getMarkerSize();
                    }
                }

               
                #endregion

                //if (IsColumnChart(ChartType) || Is3DColumnChart(ChartType) || IsBarChart(ChartType))
                //{
                //    // drae columns/bars side by side(not available for 3D)
                //    series["DrawSideBySide"] = "true";
                //}
                

                #region [ Data Bind X&Y values ]

                // data bind all chart excluding Stock and Bubble Charts
                if (!IsStockChart(ChartType) && !IsBubbleChart(ChartType))
                {
                    // if has x values as well then data bind bothe of them
                    if (elementValX.Count() > 0)
                    {
                        //data bind x and y
                        series.Points.DataBindXY(elementValX, "bu", elementValY, "cu");

                        // check if pie or doughnout chart
                        //if (IsPieChart(ChartType) || IsDoughnutChart(ChartType))
                        //{
                        //    for (int idx = 0; idx < elementValX.Count(); idx++)
                        //    {
                        //        series.Points[idx].Label = elementValX[idx].ToString();
                        //    }
                        //}
                    }
                    else
                    {
                        //data bind only the y values (the x values are automatic)
                        series.Points.DataBindY(elementValY);
                    }

                    // add the series to the chart
                    chart.Series.Add(series);
                }
                else
                {
                    yStockObj[seriesId] = elementValY;
                }

                foreach (DataPoint dp in series.Points)
                {
                    if (dp.YValues[0] == 0) dp.IsEmpty = true;
                }

                // Set empty points visual appearance attributes
                series.EmptyPointStyle.Color = series.Color;
                series.EmptyPointStyle.BorderWidth = series.BorderWidth;
                series.EmptyPointStyle.BorderDashStyle = series.BorderDashStyle;


                // check if pie of pie or bar of pie
                if (ChartType == EasyXLS.Constants.Chart.CHART_TYPE_BAR_OF_PIE || ChartType == EasyXLS.Constants.Chart.CHART_TYPE_PIE_OF_PIE)
                {
                    int NoOfPoints = chart.Series[0].Points.Count;
                    int basicColorIdx = 0;
                    double tintColor = -0.1;
                    for (int idx = 0; idx < NoOfPoints; idx++)
                    {
                        // constant for theme colors
                        const int SERIES_START_COLORS = 4;
                        // constant for theme colors
                        const int SERIES_STOP_COLORS = 5;

                        if (basicColorIdx > SERIES_STOP_COLORS) { tintColor += 0.2; basicColorIdx = 0; }

                        // get the automatic theme color
                        ThemeColor themeBackColor = new ThemeColor(SERIES_START_COLORS + basicColorIdx, tintColor);
                        // set back color
                        myThemeColorPalette[idx] = themeBackColor.getColorForTheme(excelTheme);

                        basicColorIdx++;
                    }

                    chart.Palette = ChartColorPalette.None;
                    chart.PaletteCustomColors = myThemeColorPalette;

                    // PieCollectedDataHelper is a helper class found in the samples Utilities folder. 
                    PieCollectedDataHelper pieHelper = new PieCollectedDataHelper(chart);

                    if (ChartType == EasyXLS.Constants.Chart.CHART_TYPE_BAR_OF_PIE)
                    {
                        pieHelper.UseStackedChart = true;
                    }

                    // get split TYPE
                    string collectedType = "position";

                    switch (excelChart.getSplitType())
                    {
                        case EasyXLS.Constants.Chart.SPLIT_BY_PERCENTAGE_VALUE: collectedType = "percentage"; break;
                        case EasyXLS.Constants.Chart.SPLIT_BY_POSITION: collectedType = "position"; break;
                        case EasyXLS.Constants.Chart.SPLIT_BY_VALUE: collectedType = "value"; break;
                        default: collectedType = "position"; break;
                    }

                    // get split position
                    double splitPosition = Math.Ceiling(NoOfPoints/3.0);
                    splitPosition = excelChart.getSplitPosition() != 1 ? excelChart.getSplitPosition() : splitPosition;

                    pieHelper.CollectedType = collectedType;

                    // Set the percentage of the total series values. This value determines 
                    // if the data point value is a "small" value and should be shown as collected.
                    pieHelper.CollectedValue = splitPosition;

                    // Indicates if small segments should be shown as one "collected" segment in 
                    // the original series.
                    pieHelper.ShowCollectedDataAsOneSlice = true;

                    // Size ratio between the original and supplemental chart areas.
                    // Value of 1.0f indicates that same area size will be used.
                    float secondPlotSize = 0.7f;//excelChart.getSecondPlotSize()/100.0;
                    pieHelper.SupplementedAreaSizeRatio = secondPlotSize;

                    // Set position in relative coordinates ( 0,0 - top left corner; 100,100 - bottom right corner)
                    // where original and supplemental pie charts should be placed.
                    pieHelper.ChartAreaPosition = new RectangleF(2f, 2f, 68f, 100f);

                    // Show supplemental pie for the "Default" series
                    pieHelper.ShowSmallSegmentsAsSupplementalPie(series.Name);

                    pieHelper.supplementalSeries["BarLabelStyle"] = "Center";
                }

                // CHART TYPE EXPLODED (it can only be done after adding the series)
                if (IsExplodedChart(ChartType))
                {
                    for (int i = 0; i < series.Points.Count(); i++)
                    {
                        series.Points[i]["Exploded"] = "true";
                    }
                }

                #endregion
               

                index++;
            }

            #region [ Bind Bubble Charts ]

            // check if bubble chart
            if (IsBubbleChart(ChartType))
            {
                double[][] bubbleValuesY = new double[2][];

                bubbleValuesY[0] = valuesY[0];
                bubbleValuesY[1] = valuesY[0];

                chart.ChartAreas[0].Area3DStyle.Enable3D = true;
                ExcelSeries excelSeries = excelChart.easy_getSeriesAt(0);
                FillFormat seriesFillFormat = excelSeries.getFillFormat();

                string SeriesName = GetSeriesName(excelSeries.getName(), excelDoc, formula, xlsTable, 0);
                chart.Series.Add(SeriesName);
                chart.Series[SeriesName].ChartType = SeriesChartType.Bubble;

                chart.Series[SeriesName].Color = getObjectBackColor(seriesFillFormat, excelTheme, 0);

                // Set bubble shape
                chart.Series[SeriesName].MarkerStyle = MarkerStyle.Circle;
                // set marker color
                //series.MarkerColor = Color.FromArgb(100, xlsSeries.getBackground().R, xlsSeries.getBackground().G, xlsSeries.getBackground().B);

                //// Set max bubble size
                chart.Series[SeriesName]["BubbleMaxSize"] = "15";

                //// Show bubble size as point labels
                chart.Series[SeriesName].IsValueShownAsLabel = true;
                chart.Series[SeriesName]["BubbleUseSizeForLabel"] = "true";

                //// Set bubble size scale
                chart.Series[SeriesName]["BubbleScaleMin"] = FindMinimumScale(valuesY[0]).ToString();
                chart.Series[SeriesName]["BubbleScaleMax"] = FindMaximumScale(valuesY[0]).ToString();


                // if has x values as well then data bind bothe of them
                if (elementValX.Count() > 0)
                {
                    //data bind x and y
                    chart.Series[SeriesName].Points.DataBindXY(elementValX, bubbleValuesY);
                }
                else
                {
                    //data bind only the y values (the x values are automatic)
                    chart.Series[SeriesName].Points.DataBindY(bubbleValuesY);
                }

                chart.ChartAreas[0].Area3DStyle.Enable3D = false;
                chart.ChartAreas[0].AxisX.Crossing = 0;
                chart.ChartAreas[0].AxisX.IntervalType = DateTimeIntervalType.Number;
            }

            #endregion

            #region [ Bind Stock Charts ]

            // check if stock chart
            if (IsStockChart(ChartType))
            {
                chart.Series.Add("Stock");
                chart.Series["Stock"].ChartType = SeriesChartType.Stock;

                Array yVolumeStock = new double[yStockObj[0].Length];
                Array[] yFinalStock;
                int arrVolIndex = 0;
                int arrOpenIndex = 0;

                int noOfStockArray = 4; /* constant indicating the number of arrays in a stock chart */
                const int openArr = 0; /* constant indicating the Open array */
                const int highArr = 1;/* constant indicating the High array */
                const int lowArr = 2;/* constant indicating the Low array */
                const int closeArr = 3;/* constant indicating the Close array */

                // check if STOCK VOLUME CHARTS
                if (
                    ChartType == EasyXLS.Constants.Chart.CHART_TYPE_STOCK_VOLUME_HIGH_LOW_CLOSE
                    ||
                    ChartType == EasyXLS.Constants.Chart.CHART_TYPE_STOCK_VOLUME_OPEN_HIGH_LOW_CLOSE
                    )
                {
                    yVolumeStock = yStockObj[0];
                    arrVolIndex = 1;
                }

                /* CHART_TYPE_STOCK_HIGH_LOW_CLOSE and CHART_TYPE_STOCK_VOLUME_HIGH_LOW_CLOSE */
                if (
                    ChartType == EasyXLS.Constants.Chart.CHART_TYPE_STOCK_HIGH_LOW_CLOSE
                    ||
                    ChartType == EasyXLS.Constants.Chart.CHART_TYPE_STOCK_VOLUME_HIGH_LOW_CLOSE
                    )
                {
                    arrOpenIndex = -1;
                }

                // define the final stock arrays
                yFinalStock = new Array[noOfStockArray];

                // add the Open Stock
                yFinalStock[openArr] = new double[yStockObj[openArr].Length];

                /* CHART_TYPE_STOCK_OPEN_HIGH_LOW_CLOSE and CHART_TYPE_STOCK_VOLUME_OPEN_HIGH_LOW_CLOSE */
                if (ChartType == EasyXLS.Constants.Chart.CHART_TYPE_STOCK_OPEN_HIGH_LOW_CLOSE
                    ||
                    ChartType == EasyXLS.Constants.Chart.CHART_TYPE_STOCK_VOLUME_OPEN_HIGH_LOW_CLOSE
                    )
                {
                    yFinalStock[openArr] = yStockObj[openArr + arrVolIndex + arrOpenIndex];
                }

                // add the High Stock
                yFinalStock[highArr] = new double[yStockObj[highArr + arrVolIndex + arrOpenIndex].Length];
                yFinalStock[highArr] = yStockObj[highArr + arrVolIndex + arrOpenIndex];

                // add the Low Stock
                yFinalStock[lowArr] = new double[yStockObj[lowArr + arrVolIndex + arrOpenIndex].Length];
                yFinalStock[lowArr] = yStockObj[lowArr + arrVolIndex + arrOpenIndex];

                // add the Close Stock
                yFinalStock[closeArr] = new double[yStockObj[closeArr + arrVolIndex + arrOpenIndex].Length];
                yFinalStock[closeArr] = yStockObj[closeArr + arrVolIndex + arrOpenIndex];

                // must switch low with open
                Array obj = new double[yStockObj[openArr].Length];
                obj = yFinalStock[0];
                yFinalStock[0] = yFinalStock[2];
                yFinalStock[2] = obj;


                if (ChartType == EasyXLS.Constants.Chart.CHART_TYPE_STOCK_HIGH_LOW_CLOSE) /* CHART_TYPE_STOCK_HIGH_LOW_CLOSE */
                {
                    const int THIRD_SERIES_INDEX = 2;
                    ExcelSeries xlsSeries = excelChart.easy_getSeriesAt(THIRD_SERIES_INDEX);

                    // Show both open and close marks
                    chart.Series["Stock"]["ShowOpenClose"] = "Close";
                    chart.Series["Stock"]["OpenCloseStyle"] = xlsSeries.getMarkerType() == EasyXLS.Constants.Chart.SERIES_MARKER_TRIANGLE? "Triangle": "Line";

                    chart.Series["Stock"]["PixelPointWidth"] = xlsSeries.getMarkerSize().ToString();
                    // Set point width
                    chart.Series["Stock"]["PointWidth"] = xlsSeries.getMarkerSize().ToString();

                    // get color from the third series
                    chart.Series["Stock"].Color = getObjectBackColor(xlsSeries.getFillFormat(), excelTheme, THIRD_SERIES_INDEX);
                }

                // if has x values as well then data bind bothe of them
                if (elementValX.Count() > 0)
                {

                    //data bind x and y
                    chart.Series["Stock"].Points.DataBindXY(elementValX, yFinalStock);
                }
                else
                {
                    //data bind only the y values (the x values are automatic)
                    chart.Series["Stock"].Points.DataBindY(yFinalStock);
                }

                const int CLOSE_SERIES_INDEX = 1;
                if (ChartType == EasyXLS.Constants.Chart.CHART_TYPE_STOCK_OPEN_HIGH_LOW_CLOSE) /* CHART_TYPE_STOCK_OPEN_HIGH_LOW_CLOSE */
                {
                    ExcelSeries xlsSeries = excelChart.easy_getSeriesAt(CLOSE_SERIES_INDEX);
                    // Show both open and close marks
                    chart.Series["Stock"]["ShowOpenClose"] = "Both";
                    chart.Series["Stock"]["OpenCloseStyle"] = "CandleStick";

                    // get color from the third series
                    chart.Series["Stock"].Color = getObjectBackColor(xlsSeries.getFillFormat(), excelTheme, CLOSE_SERIES_INDEX); ;
                }

                const int VOLUME_SERIES_INDEX = 0;
                ExcelSeries xSeries = excelChart.easy_getSeriesAt(VOLUME_SERIES_INDEX);

                if (ChartType == EasyXLS.Constants.Chart.CHART_TYPE_STOCK_VOLUME_HIGH_LOW_CLOSE) /* CHART_TYPE_STOCK_VOLUME_HIGH_LOW_CLOSE */
                {
                   
                    // Show both open and close marks
                    chart.Series["Stock"]["ShowOpenClose"] = "Close";
                    chart.Series["Stock"].YAxisType = AxisType.Secondary;
                    chart.ChartAreas[0].AxisY2.MajorGrid.Enabled = false;

                    // get color from the third series
                    chart.Series["Stock"].Color = getObjectBackColor(xSeries.getFillFormat(), excelTheme, CLOSE_SERIES_INDEX);

                    chart.Series.Add("Volume");
                    chart.Series["Volume"].ChartType = SeriesChartType.Column;
                    //data bind only the y values (the x values are automatic)
                    chart.Series["Volume"].Points.DataBindY(yVolumeStock);
                    Color volumeColor = getObjectBackColor(xSeries.getFillFormat(), excelTheme, VOLUME_SERIES_INDEX);
                    chart.Series["Volume"].Color = Color.FromArgb(100, volumeColor.R, volumeColor.G, volumeColor.B);

                }

                if (ChartType == EasyXLS.Constants.Chart.CHART_TYPE_STOCK_VOLUME_OPEN_HIGH_LOW_CLOSE) /* CHART_TYPE_STOCK_VOLUME_OPEN_HIGH_LOW_CLOSE */
                {
                    // Show both open and close marks
                    chart.Series["Stock"]["ShowOpenClose"] = "Both";
                    chart.Series["Stock"]["OpenCloseStyle"] = "CandleStick";

                    chart.Series["Stock"].YAxisType = AxisType.Secondary;
                    chart.ChartAreas[0].AxisY2.MajorGrid.Enabled = false;

                    // get color from the third series
                    chart.Series["Stock"].Color = getObjectBackColor(xSeries.getFillFormat(), excelTheme, CLOSE_SERIES_INDEX); ;

                    chart.Series.Add("Volume");
                    chart.Series["Volume"].ChartType = SeriesChartType.Column;
                    //data bind only the y values (the x values are automatic)
                    chart.Series["Volume"].Points.DataBindY(yVolumeStock);
                    Color volumeColor = getObjectBackColor(xSeries.getFillFormat(), excelTheme, VOLUME_SERIES_INDEX);
                    chart.Series["Volume"].Color = Color.FromArgb(100, volumeColor.R, volumeColor.G, volumeColor.B);
                }

                

            }


            #endregion

            #region [ Bind Surface Charts ]

            // check if bubble chart
            if (IsSurfaceChart(ChartType))
            {
                // get all x and y values for all points involved
                double[] allYValues = new double[chart.Series.Count * chart.Series[0].Points.Count];
                double[] allXValues = new double[chart.Series.Count * chart.Series[0].Points.Count];


                int i = 0;
                //create a huge array with all values from all series
                foreach (Series ser in chart.Series)
                {
                    for (int j = 0; j < ser.Points.Count; j++)
                    {
                        allYValues[i] = ser.Points[j].YValues[0];
                        allXValues[i] = double.Parse(elementValX[j].ToString());
                        i++;
                    }
                }

                // get the maximum number to calculate the interval
                double MaxNumber = FindMaximumScale(allYValues);
                // get the desired interval
                double interval = chartArea.AxisY.Interval;
                if (interval == 0) interval = 100;
                // get the result
                double result = Math.Ceiling(MaxNumber / interval);
                // number of intervals
                int noOfIntervals = (int)result;

                // erase all series
                while (chart.Series.Count() > 0)
                {
                    chart.Series.RemoveAt(0);
                }

                // for all required intervals
                for (int j = noOfIntervals; j > 0; j--)
                {
                    // get the series name (the intervals in this case)
                    string seriesName = ((j - 1) * interval) + " - " + (j) * interval;

                    // create new series
                    Series series = new Series(seriesName);
                    series.ChartType = SeriesChartType.Line;

                    // for all the x Values in the array
                    for (int nx = 0; nx < elementValX.Count(); nx++)
                    {
                        double yValue = 0;
                        int count = 0;
                        // for all the y values of the points
                        for (int nb = 0; nb < allYValues.Count(); nb++)
                        {
                            // check if y values are in the required interval for the given x 
                            if ((j - 1) * interval <= allYValues[nb]
                                    && allYValues[nb] < j * interval
                                        && allXValues[nb] == double.Parse(elementValX[nx].ToString()))
                            {
                                yValue = allYValues[nb];
                                count++;
                            }
                        }
                        if (yValue != 0)
                        {
                            // add the y value to the series at the required x value
                            series.Points.AddXY(elementValX[nx], yValue);
                        }
                    }

                    // add series to the chart
                    chart.Series.Add(series);
                }

            }

            #endregion


            #region [ Calculate Interval by Values  ]

            // calculate interval y only for not stacked charts
            if (!Is100Stacked(ChartType) && !IsBarChart(ChartType) && !IsColumnChart(ChartType))
            {
                // calculate the maximum stacked value
                double[] stackedVal = new double[largestCount];
                for (int i = 0; i < largestCount; i++)
                {
                    for (int j = 0; j < excelChart.SeriesCount(); j++)
                    {
                        stackedVal[i] += valuesY[j][i];
                    }
                }

                // get the interval
                double stackedMaximum = getMaxValueFrom(stackedVal);
                if (xlsYAxis.IsAutomaticMajorUnit())
                {
                    if (stackedMaximum < 100) yAxis.Interval = 10;
                }

            }

            // calculate interval y only for stacked line and column charts
            if (Is100StackedLineChart(ChartType) || Is100StackedColumnChart(ChartType))
            {
                yAxis.Interval = 10;
            }

            #endregion

            #endregion

            if (IsPieChart(ChartType) || IsDoughnutChart(ChartType))
            {
                int NoOfPoints = chart.Series[0].Points.Count;
                int basicColorIdx = 0;
                double tintColor = -0.1;
                for (int idx = 0; idx < NoOfPoints; idx++)
                {
                    // constant for theme colors
                    const int SERIES_START_COLORS = 4;
                    // constant for theme colors
                    const int SERIES_STOP_COLORS = 5;

                    if (basicColorIdx > SERIES_STOP_COLORS) { tintColor += 0.2; basicColorIdx = 0; }

                    // get the automatic theme color
                    ThemeColor themeBackColor = new ThemeColor(SERIES_START_COLORS + basicColorIdx, tintColor);
                    // set back color
                    myThemeColorPalette[idx] = themeBackColor.getColorForTheme(excelTheme);

                    basicColorIdx++;
                }

                chart.Palette = ChartColorPalette.None;
                chart.PaletteCustomColors = myThemeColorPalette;
            }


           

            #region [ Set Legend Properties ]

            /*************************************  Set Legend Properties  ************************************/

            if (xlsChartLegend.IsVisible())
            {
                Legend legend = new Legend();
                if (IsChartToReverseLegend(ChartType))
                    legend.LegendItemOrder = LegendItemOrder.ReversedSeriesOrder;
                else
                    legend.LegendItemOrder = LegendItemOrder.SameAsSeriesOrder;

                FillFormat lgndFillFormat = xlsChartLegend.getFillFormat();
                FontFormat lgndFontFormat = xlsChartLegend.getFontFormat();

                // set legend font and foreground
                FontStyle fontStyle = getFontStyle(lgndFontFormat);
                legend.Font = new Font(getObjectFontName(lgndFontFormat, excelTheme), lgndFontFormat.getFontSize(), fontStyle);
                legend.ForeColor = getObjectForegroundColor(lgndFontFormat, excelTheme);
                legend.BackColor = getObjectBackColor(lgndFillFormat, excelTheme, -1);
                if (lgndFillFormat.getBackground().IsEmpty)
                    legend.BackColor = chart.BackColor;

                // set legend borders
                LineColorFormat lgndBorderFormat = xlsChartLegend.getLineColorFormat();
                LineStyleFormat lgndBorderStyle = xlsChartLegend.getLineStyleFormat();

                legend.BorderColor = getObjectLineColor(lgndBorderFormat, excelTheme, -1);
                legend.BorderDashStyle = getChartLineStyle(lgndBorderStyle.getDashType());
                legend.BorderWidth = (int)Math.Ceiling(getObjectWidth(lgndBorderFormat, lgndBorderStyle, "legend"));


                legend.Docking = getDocking(xlsChartLegend.getPlacement());
                legend.Alignment = getAlignmentByPlacement(xlsChartLegend.getPlacement());

                //legend.LegendStyle = xlsChartLegend.getKeysArrangementDirection()==0?LegendStyle.Column:LegendStyle.Row;

                if (xlsChartLegend.getPlacement() == 7) //LEGEND_NOT_DOCKED_OR_INSIDE_THE_PLOT_AREA
                    legend.IsDockedInsideChartArea = false;
                else legend.IsDockedInsideChartArea = true;


                legend.IsTextAutoFit = true;
                legend.AutoFitMinFontSize = 30;
                legend.MaximumAutoSize = 100;

                chart.Legends.Add(legend);
            }

            #endregion

            // fix GRADIENT FILL
            if (chartAreaFillF.IsGradientFill())
            {
                // get the first gradient stop
                GradientStop firstGrStop = chartAreaFillF.getGradientStopAt(0);
                // get the first gradient stop
                GradientStop lastGrStop = chartAreaFillF.getGradientStopAt(chartAreaFillF.GradientStopCount() - 1);

                chart.BackColor = Color.Empty;
                if (firstGrStop.Color is ThemeColor)
                {
                    ThemeColor firstColor, secColor;
                    firstColor = (ThemeColor)firstGrStop.Color;
                    secColor = (ThemeColor)lastGrStop.Color;

                    chart.BackColor = firstColor.getColorForTheme(excelTheme);
                    chartArea.BackSecondaryColor = secColor.getColorForTheme(excelTheme);
                }
                else
                {
                    chart.BackColor = (Color)firstGrStop.Color;
                    chart.BackSecondaryColor = (Color)lastGrStop.Color;
                }

                chartArea.BackColor = chart.BackColor;
                chartArea.BackSecondaryColor = chart.BackSecondaryColor;

                chart.Titles[0].BackColor = chart.BackColor;
                chart.Titles[0].BackSecondaryColor = chart.BackSecondaryColor;

                chart.Legends[0].BackColor = chart.BackColor;
                chart.Legends[0].BackSecondaryColor = chart.BackSecondaryColor;

                chart.BackGradientStyle = getGradientStyle(chartAreaFillF.getGradientDirection());
                chartArea.BackGradientStyle =  chart.BackGradientStyle;
                chart.Titles[0].BackGradientStyle  =  chart.BackGradientStyle;
                chart.Legends[0].BackGradientStyle = chart.BackGradientStyle;
            }


            #region [ Save Chart as PNG & Add it to Data Source ]


            // Save the chart as a 300 x 200 image
            chart.Width = excelChart.getWidth();
            chart.Height = excelChart.getHeight(); 

            string fileTimeName = DateTime.Now.ToString("yyyyMMddHHmmssfff");

            DirectoryInfo directory = new DirectoryInfo(HttpContext.Current.Server.MapPath("~/EasyXLSChartFiles/" + UserSession));
            string filename = "";

            if (directory.Exists)
            {
                filename = directory.FullName + "/Chart" + chartIndex + "_" + fileTimeName + ".png";
            }
            else
            {
                DirectoryInfo newDir = Directory.CreateDirectory(HttpContext.Current.Server.MapPath("~/EasyXLSChartFiles/" + UserSession));
                filename = newDir.FullName + "/Chart" + chartIndex + "_" + fileTimeName + ".png";
            }

            chart.SaveImage(filename, ChartImageFormat.Png);

            chartInfo = new KeyValuePair<System.Web.UI.DataVisualization.Charting.Chart, string>(chart, fileTimeName);//filename;//[[chart;

            #endregion

            return chartInfo;
        }

        #endregion

        #region [ Build Data Table with Chart Information ]

        // Create DataTable with the correct arrangement of the charts

        /// <summary>
        /// DrawCharts
        /// </summary>
        public DataTable DrawCharts(ExcelDocument document, int sheetNo, string LeftCornerCellReference, string RightCornerCellReference, bool isDashboard)
        {
            // the data table for saving the path to the pictures
            DataTable dtCharts = new DataTable();
            // define the table's schema
            //dtCharts.Columns.Add(new DataColumn("PictureURL", typeof(string)));

            // Get the current worksheet
            ExcelSheet sheet = document.easy_getSheetAt(sheetNo);
            ExcelWorksheet excelWorksheet = new ExcelWorksheet();
            int noOfCharts = 0;
            if (sheet is ExcelWorksheet)
            {
                excelWorksheet = (ExcelWorksheet)sheet;

                ExcelTable xlsTable = excelWorksheet.easy_getExcelTable();

                // Get the number of charts
                noOfCharts = excelWorksheet.ChartCount();

                #region [ Get the IMAGES ]

                int noOfImages = sheet.ImageCount();

                string[] fileTimeStamp = new string[noOfImages];
                int[] imageHeight = new int[noOfImages];
                int[] imageWidth = new int[noOfImages];

                for (int imgIdx = 0; imgIdx < noOfImages; imgIdx++)
                {
                    ExcelImage excelImage = sheet.easy_getImageAt(imgIdx);
                    //get url for this image
                    string tmpPath = excelImage.getImagePath();

                    // check if url exists
                    if (tmpPath == "")
                    {
                        byte[] imageByte = excelImage.getImageBytes();
                        ImageConverter imgConv = new ImageConverter();
                        System.Drawing.Image xlsImage = (System.Drawing.Image)imgConv.ConvertFrom(imageByte);
                        int destWidth = excelImage.getWidth();
                        int destHeight = excelImage.getHeight();

                        Bitmap b = new Bitmap(xlsImage, destWidth, destHeight);

                        fileTimeStamp[imgIdx] = DateTime.Now.ToString("yyyyMMddHHmmssfff");

                        string imagePath = "";
                        DirectoryInfo dirApp = new DirectoryInfo(HttpContext.Current.Server.MapPath("~/EasyXLSImage/" + UserSession));

                        if (dirApp.Exists)
                        {
                            imagePath = dirApp.FullName + "\\Image" + imgIdx + "_" + fileTimeStamp[imgIdx] + ".png";
                        }
                        else
                        {
                            DirectoryInfo dirNew = Directory.CreateDirectory(HttpContext.Current.Server.MapPath("~/EasyXLSImage/" + UserSession + "/"));
                            imagePath = dirNew.FullName + "\\Image" + imgIdx + "_" + fileTimeStamp[imgIdx] + ".png";
                        }

                        b.Save(imagePath, System.Drawing.Imaging.ImageFormat.Png);

                        imageHeight[imgIdx] = excelImage.getHeight();
                        imageWidth[imgIdx] = excelImage.getWidth();
                    }
                }

                #endregion

                #region [Get the position on the excel file]

                Dictionary<int, int[]> posChart = new Dictionary<int, int[]>();

                List<int> getOrderCharts = new List<int>();

                int[] elementNameLeft = FormulaParser.get2DCellElements(LeftCornerCellReference);
                int[] elementNameRight = FormulaParser.get2DCellElements(RightCornerCellReference);

                // start col/row
                int leftCornerCol = int.Parse(elementNameLeft[0].ToString());
                int leftCornerRow = int.Parse(elementNameLeft[1].ToString());

                // start col/row
                int rightCornerCol = int.Parse(elementNameRight[0].ToString());
                int rightCornerRow = int.Parse(elementNameRight[1].ToString());


                // get all charts from worksheet to get their position
                for (int chartIndex = 0; chartIndex < noOfCharts; chartIndex++)
                {
                    int isChart = 1;

                    // get excel worksheet chart
                    ExcelChart excelChart = excelWorksheet.easy_getChartAt(chartIndex);

                    int leftUpperCornerCol = 0, leftUpperCornerRow = 0;

                    // get the column
                    int getUPCornerCol = excelChart.getLeftUpperCornerColumn();
                    //get the row
                    int getUPCornerRow = excelChart.getLeftUpperCornerRow();

                    // go to the left corner cell and check the width and height
                    ExcelColumn xlsCornerColumn = (xlsTable.ColumnCount() > getUPCornerCol) ? xlsTable.easy_getColumnAt(getUPCornerCol) : null;
                    ExcelRow xlsCornerRow = (xlsTable.RowCount() > getUPCornerRow) ? xlsTable.easy_getRowAt(getUPCornerRow) : null;

                    // store the cell height and width
                    int cellHeight, cellWidth = 0;

                    // check if row exists 
                    if (xlsCornerRow != null)
                    {
                        // if Height is set then get the value , otherwise set the empty cell Height value
                        cellHeight = (xlsCornerRow.getHeight() != -1 ? xlsCornerRow.getHeight() : emptyCellHeight);
                    }
                    else cellHeight = emptyCellHeight;

                    // check if column exists 
                    if (xlsCornerColumn != null)
                    {
                        // if Width is set then get the value , otherwise set the empty cell Width value
                        cellWidth = (xlsCornerColumn.getWidth() != -1 ? xlsCornerColumn.getWidth() : emptyCellWidth);
                    }
                    else cellWidth = emptyCellWidth;

                    // get the OFFSET PERCENT horizontaly and verticaly
                    double getHorizontalOffsetPercent = ((double)excelChart.getHorizontalOffset() * 100.00)/(double)cellWidth;
                    double getVerticalOffsetPercent = ((double)excelChart.getVerticalOffset() * 100.00)/(double)cellHeight;

                    // if the offset is more then 50 % then go to the next column
                    if (getHorizontalOffsetPercent > 50.00) leftUpperCornerCol = getUPCornerCol + 1;
                    else leftUpperCornerCol = getUPCornerCol;

                    // if the offset is more then 50 % then go to the next row
                    if (getVerticalOffsetPercent > 50.00) leftUpperCornerRow = getUPCornerRow + 1;
                    else leftUpperCornerRow = getUPCornerRow; 

                    // check if chart happens to be in the filtered range
                    if (
                        leftCornerCol <= leftUpperCornerCol && leftUpperCornerCol <= rightCornerCol
                        &&
                        leftCornerRow <= leftUpperCornerRow && leftUpperCornerRow <= rightCornerRow
                        )
                    {
                        //save it to an array
                        posChart[chartIndex] = new int[] { leftUpperCornerCol, leftUpperCornerRow, isChart};
                    }

                }

                // get all images from worksheet to get their position
                for (int imgIdx = 0; imgIdx < noOfImages; imgIdx++)
                {
                    int isChart = 0;

                    // get excel worksheet chart
                    ExcelImage excelImage = excelWorksheet.easy_getImageAt(imgIdx);

                    int leftUpperCornerCol = 0, leftUpperCornerRow = 0;

                    // get the column
                    int getUPCornerCol = excelImage.getLeftUpperCornerColumn();
                    //get the row
                    int getUPCornerRow = excelImage.getLeftUpperCornerRow();

                    // go to the left corner cell and check the width and height
                    ExcelColumn xlsCornerColumn = (xlsTable.ColumnCount() > getUPCornerCol) ? xlsTable.easy_getColumnAt(getUPCornerCol) : null;
                    ExcelRow xlsCornerRow = (xlsTable.RowCount() > getUPCornerRow) ? xlsTable.easy_getRowAt(getUPCornerRow) : null;

                    // store the cell height and width
                    int cellHeight, cellWidth = 0;

                    // check if row exists 
                    if (xlsCornerRow != null)
                    {
                        // if Height is set then get the value , otherwise set the empty cell Height value
                        cellHeight = (xlsCornerRow.getHeight() != -1 ? xlsCornerRow.getHeight() : emptyCellHeight);
                    }
                    else cellHeight = emptyCellHeight;

                    // check if column exists 
                    if (xlsCornerColumn != null)
                    {
                        // if Width is set then get the value , otherwise set the empty cell Width value
                        cellWidth = (xlsCornerColumn.getWidth() != -1 ? xlsCornerColumn.getWidth() : emptyCellWidth);
                    }
                    else cellWidth = emptyCellWidth;

                    // get the OFFSET PERCENT horizontaly and verticaly
                    double getHorizontalOffsetPercent = ((double)excelImage.getHorizontalOffset() * 100.00) / (double)cellWidth;
                    double getVerticalOffsetPercent = ((double)excelImage.getVerticalOffset() * 100.00) / (double)cellHeight;

                    // if the offset is more then 50 % then go to the next column
                    if (getHorizontalOffsetPercent > 50.00) leftUpperCornerCol = getUPCornerCol + 1;
                    else leftUpperCornerCol = getUPCornerCol;

                    // if the offset is more then 50 % then go to the next row
                    if (getVerticalOffsetPercent > 50.00) leftUpperCornerRow = getUPCornerRow + 1;
                    else leftUpperCornerRow = getUPCornerRow; 

                    // check if chart happens to be in the filtered range
                    if (
                        leftCornerCol <= leftUpperCornerCol && leftUpperCornerCol <= rightCornerCol
                        &&
                        leftCornerRow <= leftUpperCornerRow && leftUpperCornerRow <= rightCornerRow
                        )
                    {
                        //save it to an array
                        posChart[noOfCharts + imgIdx] = new int[] { leftUpperCornerCol, leftUpperCornerRow, isChart };
                    }

                }

                Dictionary<int, int[]> auxPosChart = new Dictionary<int, int[]>();

                // while pos chart
                while (posChart.Count > 0)
                {
                    int getMinPosKey = GetTheMinimumPosition(posChart);
                    getOrderCharts.Add(getMinPosKey);
                    auxPosChart.Add(getMinPosKey, posChart[getMinPosKey]);
                    posChart.Remove(getMinPosKey);

                }

                #endregion

                #region [Create the Chart Data Table]
                
                int previousChartIndex = -1;
                int countColSameRow = 0;
                DataRow drChart = dtCharts.NewRow();

                // get all charts from worksheet
                foreach (int chartIndex in getOrderCharts)
                {
                    KeyValuePair<System.Web.UI.DataVisualization.Charting.Chart, string> chartLocation = new KeyValuePair<System.Web.UI.DataVisualization.Charting.Chart, string>();
                    System.Web.UI.DataVisualization.Charting.Chart chart = null;
                    if (chartIndex < noOfCharts)
                    {
                        chartLocation = getChartFromSheet(chartIndex, document, sheetNo, isDashboard);
                        chart = chartLocation.Key;
                    }
                    

                    // check if there has been added another chart before this current one
                    if (previousChartIndex != -1)
                    {
                        // if the row of both charts is the same
                        if (auxPosChart[chartIndex][1] == auxPosChart[previousChartIndex][1])
                        {
                            // count the number of columns to be added to the data table
                            countColSameRow++;
                        }
                        else
                        {
                            //create new row in data table with the chart file
                            drChart = dtCharts.NewRow();
                            // reinitiate the columns count
                            countColSameRow = 0;
                        }
                    }

                    // check if there is not a column named that way
                    if (!dtCharts.Columns.Contains("PictureURL" + countColSameRow))
                    {
                        // create a new column
                        dtCharts.Columns.Add(new DataColumn("PictureURL" + countColSameRow, typeof(ChartData)));
                    }
                    ChartData chtData = null;

                    if (chartIndex < noOfCharts)
                    {

                        string fileTimeName = chartLocation.Value;
                        chtData = new ChartData("../EasyXLSChartFiles/" + UserSession + "/Chart" + chartIndex + "_" + fileTimeName + ".png", auxPosChart[chartIndex][1], auxPosChart[chartIndex][0], chart.Width.Value, chart.Height.Value);
                    }
                    else
                    {
                        chtData = new ChartData("../EasyXLSImage/" + UserSession + "/Image" + (chartIndex - noOfCharts) + "_" + fileTimeStamp[chartIndex - noOfCharts] + ".png", auxPosChart[chartIndex][1], auxPosChart[chartIndex][0], imageWidth[chartIndex - noOfCharts], imageHeight[chartIndex - noOfCharts]);
                    }

                    // add to the data cell the url to the picture of the chart
                    drChart["PictureURL" + countColSameRow] = chtData;

                    // add the rows to the data table
                    if (previousChartIndex == -1 || auxPosChart[chartIndex][1] != auxPosChart[previousChartIndex][1])
                    {
                        dtCharts.Rows.Add(drChart);
                    }

                    previousChartIndex = chartIndex;
                }

                #endregion
            }
            else
            {
                noOfCharts = 1;

                // get the chart from the ChartSheet
                KeyValuePair<System.Web.UI.DataVisualization.Charting.Chart, string> chartLocation = new KeyValuePair<System.Web.UI.DataVisualization.Charting.Chart, string>();
                chartLocation = getChartFromSheet(0, document, sheetNo, isDashboard);
                System.Web.UI.DataVisualization.Charting.Chart chart = chartLocation.Key;
                string fileTimeName = chartLocation.Value;
                ChartData chtData = new ChartData("../EasyXLSChartFiles/" + UserSession + "/Chart0_" + fileTimeName + ".png", 0, 0, chart.Width.Value, chart.Height.Value);
                DataRow drChart = dtCharts.NewRow();
                // create a new column
                dtCharts.Columns.Add(new DataColumn("PictureURL" + 0, typeof(ChartData)));

                drChart[0] = chtData;
                dtCharts.Rows.Add(drChart);

            }

            return dtCharts;
        }

        #endregion

    }
}