﻿/*
 Copyright (c) 2012-2014 EU Outsourcing, Inc
All rights reserved.  http://www.euoutsourcing.com
Version: 2.1.0
Disclaimer: 
Any redistribution, modification and use of this software either in source or binary forms is permitted provided that the following terms are met: 
1. This software’s source code is not modified to avoid the use of EasyXLS™. 
2.  All servers and computers, whether physical or virtual, using this software or its derivatives must be properly licensed to use EasyXLS™.
3. Any derivatives of this source code that is redistributed must include the above copyright notice, these five conditions and the included disclaimer. 
4. Any redistributions in binary form must include the above copyright notice, these five conditions and the included disclaimer in any materials that are provided with the distribution. 
5. EasyXLS™ and the name of its parent organization may not be used to endorse or promote any software products or derivative works without specific prior written permission from the copyright holders.
DISCLAIMER.  THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Data;
using EasyXLSDemoApp.EasyXLSGrid;
using EasyXLS;
using System.Web.UI.WebControls;
using System.Drawing;
using EasyXLS.Util;
using EasyXLS.Formulas.Util;
using EasyXLS.Charts;
using EasyXLS.Util.Conversion;
using System.Web.UI.DataVisualization.Charting;
using System.Web.UI.DataVisualization.Charting.Utilities;
using System.Web.UI;
using EasyXLS.Drawings;
using EasyXLS.Drawings.Formatting;
using EasyXLS.Constants;
using EasyXLS.Themes;
using EasyXLS.PivotTables;
using System.Reflection;

namespace EasyXLSDemoApp.EasyXLSData
{
    public class EasyXLSGridData: System.Web.UI.Page
    {
        #region [ Declarations ]

        public int gridViewWidth;
        // Ex : row 1, height 21 -> row 2, height (21 + 21=)42 -> .......
        Dictionary<int, double> gridViewRowHeight = new Dictionary<int, double>();
        // keep merged columns for regular cells
        public Dictionary<int, List<CellSpan>> mergedCells = new Dictionary<int, List<CellSpan>>();
        // keep merged columns for charts
        public Dictionary<int, int[]> mergedChartCols = new Dictionary<int, int[]>();


        // this datatable stores the values found in excel cells (it will be bound to the grid view)
        public DataTable dtValues = new DataTable();
        // this datatable stores the style information for formatting the grid view cell
        public DataTable dtStyles = new DataTable();

        // THE EXCEL DOCUMENT
        public ExcelDocument gridDataDocument = new ExcelDocument();
        // THE EXCEL TABLE
        public ExcelTable xlsTable = new ExcelTable();
        // THE EXCEL THEME APPLIED TO THE EXCEL DOCUMENT
        public ExcelTheme xlsTheme = new ExcelTheme();

        #endregion

        #region [ Constants ]

        // The maximum height of a row that auto resizes its self
        const double MAX_ROW_HEIGHT = 600;

        // constants for width and height of empty cells
        public const int emptyCellWidth = 65;
        public const int emptyCellHeight = 21;

        // first index in the array contains the column span
        public const int POSITION_COL_SPAN = 0;
        // second index in the array contains the row span
        public const int POSITION_ROW_SPAN = 1;
        // third index in the array contains the current row
        public const int POSITION_CURRENT_ROW = 2;

        // type that keeps the cell's row and column span
        public struct CellSpan
        {
            public int cellIndex, columnSpan, rowSpan;

            public CellSpan(int idx, int cSpan, int rSpan)
            {
                cellIndex = idx;
                columnSpan = cSpan;
                rowSpan = rSpan;
            }
        }

        #endregion

        #region[ Members ]
        
        // the top left corner of selection
        private int _cellLeftCornerX;
        private int _cellLeftCornerY;

        // the bottom right corner of selection
        private int _cellRightCornerX;
        private int _cellRightCornerY;

        // sheet number to render
        private int _sheetNumber;
        // the file path
        private string _filePath;

        // check if user wants to see all selection or just the cells populated from the selection
        // true - only cells populated from selection
        // false - all cells populated/empty from selection
        private bool _showOnlyPopulated;



    
        // get set for top left corner x coordinate
        public int CellLeftCornerX
        {
            get { return _cellLeftCornerX; }
            set { _cellLeftCornerX = value; }
        }

        // get set for top left corner y coordinate
        public int CellLeftCornerY
        {
            get { return _cellLeftCornerY; }
            set { _cellLeftCornerY = value; }
        }

        // get set for top left corner y coordinate
        public int CellRightCornerX
        {
            get { return _cellRightCornerX; }
            set { _cellRightCornerX = value; }
        }

        // get set for top left corner y coordinate
        public int CellRightCornerY
        {
            get { return _cellRightCornerY; }
            set { _cellRightCornerY = value; }
        }

        // get sheet number
        public int SheetNumber
        {
            get { return _sheetNumber; }
            set { _sheetNumber = value; }
        }

        // get set for the file path
        public string FilePath
        {
            get { return _filePath; }
            set { _filePath = value; }
        }

        // get set for the check box
        public bool ShowOnlyPopulated
        {
            get { return _showOnlyPopulated; }
            set { _showOnlyPopulated = value; }
        }

        #endregion

        #region [ Color vs Theme Color ]

        /// <summary>
        /// getObjectThemeVSForeground : get automatic/solid/theme color
        /// </summary>
        /// <returns></returns>
        public static Color getObjectThemeVSNormalColor(ThemeColor themeColor, Color color, ExcelTheme excelTheme, string xlsType)
        {
            Color automaticColor = Color.Black;

            switch (xlsType)
            {
                case "foreground": automaticColor = Color.Black; break;
                case "background": automaticColor = Color.White; break;
                case "pattern": automaticColor = Color.White; break;
                case "border": automaticColor = Color.Empty; break;
            }

            Color xlsColor = automaticColor;

            // if theme color does not exist
            if (themeColor == null)
            {
                // get normal foreground
                xlsColor = color;

                // if foreground is empty return black
                if (xlsColor.IsEmpty) xlsColor = automaticColor;
            }
            else
            {
                // get the color from the theme
                xlsColor = themeColor.getColorForTheme(excelTheme);
            }

            return xlsColor;
        }

        #endregion

        #region [ Read the Excel file ]

        /// <summary>
        /// getExcelDocument
        /// </summary>
        /// <param name="pathToFile"></param>
        /// <returns></returns>
        public ExcelDocument getExcelDocument(string pathToFile)
        {
            // Load the unformatted data
            ExcelDocument readDocument = new ExcelDocument();
            // Load the xls file
            ExcelDocument myDocument = new ExcelDocument();
            // Load the xlsx file
            ExcelDocument myXLSXDocument = new ExcelDocument();
            bool isXLSXFile = false;
            string sheetName = "";

            //try
            //{
                // check if the file is a xls/xlsx document
                if (myDocument.easy_IsExcel(pathToFile) == true)
                {
                    // get the xls document
                    myDocument.easy_LoadXLSFile(pathToFile);
                    // get current sheet number 
                    int currentSheetNo = SheetNumber;

                    if (currentSheetNo < myDocument.SheetCount())
                    {
                        // get sheet based on currentSheetNo
                        ExcelSheet currentSheet = myDocument.easy_getSheetAt(currentSheetNo);
                        // get sheet name 
                        sheetName = currentSheet.getSheetName();
                    }
                    else myDocument = null;
                }
                else
                {
                    // get the xlsx document
                    isXLSXFile = true;
                    myXLSXDocument.easy_LoadXLSXFile(pathToFile);
                    // get current sheet number
                    int currentSheetNo = SheetNumber;
                    if (currentSheetNo < myXLSXDocument.SheetCount())
                    {
                        // get cuurent sheet
                        ExcelSheet currentSheet = myXLSXDocument.easy_getSheetAt(currentSheetNo);
                        // get sheet name 
                        sheetName = currentSheet.getSheetName();
                    }
                    else myXLSXDocument = null;
                }

            //}
            //catch (Exception ex)
            //{

            //}

            // overwrite the documents
            if (isXLSXFile)
            { myDocument = myXLSXDocument; }

            if(myDocument != null)
                xlsTheme = myDocument.getTheme();

            return myDocument;
        }

       
        /// <summary>
        /// GetExcelDataTables
        /// </summary>
        /// <param name="cellLeftCornerX"></param>
        /// <param name="cellLeftCornerY"></param>
        /// <param name="cellRightCornerX"></param>
        /// <param name="cellRightCornerY"></param>
        /// <param name="pathToFile"></param>
        /// <returns></returns>
        public int GetExcelDataTables(string pathToFile)
        {
            this.FilePath = pathToFile;

            // indentation for datatable
            int grvRow = 0, grvCol = 0;

           

            // get the document
            gridDataDocument = getExcelDocument(pathToFile);

            // get first sheet
            ExcelSheet excSheet = gridDataDocument.easy_getSheetAt(SheetNumber);

            // get conditionals
            Dictionary<int[], List> listConditionals = new Dictionary<int[], List>();
           
            // check if current sheet is a WorkSheet
            if (excSheet is ExcelWorksheet)
            {
                ExcelWorksheet excWorkSheet = (ExcelWorksheet)excSheet;
                //Get the table of the first and only worksheet
                xlsTable = excWorkSheet.easy_getExcelTable();

                #region [ Get PIVOT TABLES ]

                // check if worksheet contains pivot tables
                if (excWorkSheet.PivotTableCount() > 0)
                {
                    for (int pivotIDX = 0; pivotIDX < excWorkSheet.PivotTableCount(); pivotIDX++)
                    {
                        ExcelPivotTable xlsPivotTable = excWorkSheet.easy_getPivotTableAt(pivotIDX);
                    }
                }

                #endregion

                #region [ Get Conditional Formatting ]

                // check if any conditional formatting exists
                int excCondCount = excWorkSheet.ConditionalFormattingCount();

                // go through all conditional formatting
                for (int formattingCount = 0; formattingCount < excCondCount; formattingCount++)
                {
                    // get the current conditional formatting
                    ExcelConditionalFormatting conditionalFormatting = excWorkSheet.easy_getConditionalFormattingAt(formattingCount);

                    // get the range that applies to
                    int firstCondRow = conditionalFormatting.getFirstRow();
                    int lastCondRow = conditionalFormatting.getLastRow();
                    int firstCondCol = conditionalFormatting.getFirstColumn();
                    int lastCondCol = conditionalFormatting.getLastColumn();
                    int[] position = { firstCondRow, firstCondCol, lastCondRow, lastCondCol };

                    EasyXLSGridFormula formulaParser = new EasyXLSGridFormula();

                    // go through all conditions for that specified range
                    for (int conditionCount = 0; conditionCount < conditionalFormatting.ConditionCount(); conditionCount++)
                    {
                        // get the current condition
                        ExcelCondition excCondition = conditionalFormatting.getConditionAt(conditionCount);
                        // get condition type
                        int excConditionType = excCondition.getConditionType();
                        if (excConditionType == 2)
                        {
                            // get condition formula
                            string formula = excCondition.getFirstFormula().Replace(" ", "");
                            // add the formulas to a list
                            listConditionals.Add(position, formulaParser.getDynamicReferenceFormula(firstCondRow, firstCondCol, lastCondRow, lastCondCol, formula, gridDataDocument, excWorkSheet, xlsTable));
                        }
                    }
                }

                #endregion

                // init empty datacolumns to create template datatable
                DataColumn dcol, dcolStyle;

                dtStyles.Columns.Clear();
                dtValues.Columns.Clear();

                // the maximum numer of rows
                int maxRowNumber = 0;
                maxRowNumber = (CellRightCornerY + 1 > xlsTable.RowCount() ? xlsTable.RowCount() : CellRightCornerY + 1);

                // the maximum numer of columns
                int maxColumnNumber = 0;
                maxColumnNumber = (CellRightCornerX + 1 > xlsTable.ColumnCount() ? xlsTable.ColumnCount() : CellRightCornerX + 1);

                // if user wnats to see all the range he entered then set maxColumn Number to the end of the range
                if (!ShowOnlyPopulated) { maxColumnNumber = CellRightCornerX + 1; maxRowNumber = CellRightCornerY + 1; }

                // create an empty datatable 
                for (int idx = 0; idx < maxColumnNumber - CellLeftCornerX; idx++)
                {
                    //Create a new column for each column in the excel file
                    dcol = new DataColumn();  // give header
                    dcolStyle = new DataColumn("Col" + idx, typeof(CellStyleData));
                    dtStyles.Columns.Add(dcolStyle);
                    dtValues.Columns.Add(dcol);
                }

                excWorkSheet.easy_computeFormulas(gridDataDocument, false);

                // read excel file row by row between the given coordinates
                for (int xlsRow = CellLeftCornerY; xlsRow < maxRowNumber; xlsRow++)
                {

                    // reset the column number
                    grvCol = 0;

                    // create new row for each data table
                    DataRow drow = dtValues.NewRow();
                    DataRow drowStyle = dtStyles.NewRow();
                    bool isHiddenRow = false;

                    int RowWidth = 0;

                    if (xlsRow < xlsTable.RowCount())
                    {
                        // get the current excel row in order to get its height
                        ExcelRow row = xlsTable.easy_getRowAt(xlsRow);

                        // check if row is hidden
                        if (!row.IsHiddenRow())
                        {
                            // read excel file cell by cell between the given coordinates
                            for (int xlsCol = CellLeftCornerX; xlsCol < maxColumnNumber; xlsCol++)
                            {
                                CellStyleData cellStyleData = new CellStyleData();

                                if (xlsCol < xlsTable.ColumnCount())
                                {
                                    // get the whole column of the cell in order to get its width
                                    ExcelColumn col = xlsTable.easy_getColumnAt(xlsCol);

                                    //check if column is hidden
                                    if (!col.IsHiddenColumn())
                                    {
                                        // get the current cell
                                        ExcelCell xlsCell = xlsTable.easy_getCell(xlsRow, xlsCol);

                                        // gets the data type within the cell
                                        string cellDataType = xlsCell.getDataType();

                                        ExcelStyle xlsStyle = new ExcelStyle();//new ExcelStyle(xlsCell.getStyle().getFont(), xlsCell.getStyle().getFontSize(), xlsCell.getStyle().IsBold(), xlsCell.getStyle().IsItalic(), xlsCell.getStyle().getForeground());

                                        xlsStyle = xlsCell.getStyle().Clone();

                                        // get the format of the value
                                        string format = xlsStyle.getFormat();

                                        int Pattern = xlsCell.getPattern();
                                        Color patternCol = getObjectThemeVSNormalColor(xlsCell.getPatternThemeColor(), xlsCell.getPatternColor(), xlsTheme, "pattern");

                                        Color fontCol = getObjectThemeVSNormalColor(xlsCell.getForegroundThemeColor(), xlsCell.getForeground(), xlsTheme, "foreground");

                                        ExcelComment excelComment = xlsCell.getComment();

                                        string htmlValue = "";
                                        if(xlsCell.IsHTMLUsed())
                                            htmlValue = xlsCell.getHTMLValue();
                                        

                                        // check if unreadable value
                                        if (xlsCell.getFormattedValue() == "#VALUE!")
                                        {
                                            DateTime date;

                                            // check if the value is a datetime
                                            if (DateTime.TryParse(xlsCell.getValue(), out date))
                                            {
                                                drow[grvCol] = EasyXLSGridFormat.ReturnUnreadableDateTimeValues(date, format);

                                            }
                                            else
                                            {
                                                // get the formatted data
                                                drow[grvCol] = xlsCell.getFormattedValue();
                                            }
                                        }
                                        else
                                        {
                                            // get the formatted data
                                            drow[grvCol] = xlsCell.getFormattedValue();
                                        }

                                        bool numericFormatRed = false;

                                        // check if any negative numbers formatted with red                    
                                        if (format.Contains("[Red]"))
                                        {
                                            double negNumber = 0;
                                            double.TryParse(xlsCell.getValue(), out negNumber);

                                            if (negNumber < 0)
                                            {
                                                numericFormatRed = true;
                                            }
                                        }

                                        // get cell height and width
                                        bool heightAutoSize = false;
                                        int height = row.getHeight();
                                        int width = col.getWidth();

                                        // if not available, default to 30
                                        if (height == -1)
                                        {
                                            heightAutoSize = true;
                                            height = emptyCellHeight;
                                        }
                                        if (width == -1)
                                            width = emptyCellWidth;

                                        #region [ GET CELL Height ]

                                        // check if first cell in the row
                                        if (grvCol == 0)
                                        {
                                            if (grvRow == 0)
                                                gridViewRowHeight.Add(grvRow, height);
                                            else
                                                gridViewRowHeight.Add(grvRow, gridViewRowHeight[grvRow - 1] + height);
                                        }

                                        #endregion

                                        // Add each column with to total gridViewWidth
                                        RowWidth += width;

                                        #region [ Conditional Formatting Area]

                                        // check if any conditional formatting exists
                                        ExcelConditionalFormatting excConditionalFormatting = CheckForConditionalFormatting(excWorkSheet, xlsRow, xlsCol);
                                        if (excConditionalFormatting != null)
                                        {
                                            // get the number of conditions
                                            int excCondFormCount = excConditionalFormatting.ConditionCount();

                                            // iterate trough all conditions
                                            for (int conditionCount = 0; conditionCount < excCondFormCount; conditionCount++)
                                            {
                                                // get the condition
                                                ExcelCondition excCondition = excConditionalFormatting.getConditionAt(conditionCount);

                                                // get condition type
                                                int excConditionType = excCondition.getConditionType();
                                                // get condition operator
                                                int excOperator = excCondition.getOperator();
                                                // get the first formula for the condition
                                                string firstFormula = excCondition.getFirstFormula();
                                                // get the second formula for the condition
                                                string secondFormula = excCondition.getSecondFormula();
                                                // variable to make or do not make the changes
                                                bool makeChanges = false;

                                                if (firstFormula != "" || secondFormula != "")
                                                {
                                                    // get the value in the cell
                                                    double toCompareVal = 0;
                                                    double.TryParse(xlsCell.getValue(), out toCompareVal);

                                                    // check if conditional formatting is checking cell values
                                                    if (excConditionType == EasyXLS.Constants.ConditionalFormatting.CONDITIONAL_FORMATTING_TYPE_COMPARE_CELL_VALUE)
                                                    {

                                                        // check operators
                                                        switch (excOperator)
                                                        {
                                                            case EasyXLS.Constants.ConditionalFormatting.OPERATOR_LESS_THAN_OR_EQUAL_TO:
                                                                {
                                                                    double toCompareToVal = 0;
                                                                    int lastIndexOfEq = firstFormula.LastIndexOf("=");
                                                                    if (!double.TryParse(firstFormula.Substring(lastIndexOfEq + 1), out toCompareToVal))
                                                                    {
                                                                        string getFirstRef = firstFormula.Substring(lastIndexOfEq + 1);
                                                                        int cell1BetweenX = (int)FormulaParser.get2DCellElements(getFirstRef)[0];
                                                                        int cell1BetweenY = (int)FormulaParser.get2DCellElements(getFirstRef)[1];
                                                                        double.TryParse(xlsTable.easy_getCellAt(cell1BetweenY, cell1BetweenX).getValue(), out toCompareToVal);
                                                                    }

                                                                    // if LESS_THAN_OR_EQUAL_TO make changes
                                                                    if (toCompareVal <= toCompareToVal) makeChanges = true;
                                                                    break;
                                                                }
                                                            case EasyXLS.Constants.ConditionalFormatting.OPERATOR_GREATER_THAN_OR_EQUAL_TO:
                                                                {
                                                                    double toCompareToVal = 0;
                                                                    int lastIndexOfEq = firstFormula.LastIndexOf("=");
                                                                    if (!double.TryParse(firstFormula.Substring(lastIndexOfEq + 1), out toCompareToVal))
                                                                    {
                                                                        string getFirstRef = firstFormula.Substring(lastIndexOfEq + 1);
                                                                        int cell1BetweenX = (int)FormulaParser.get2DCellElements(getFirstRef)[0];
                                                                        int cell1BetweenY = (int)FormulaParser.get2DCellElements(getFirstRef)[1];
                                                                        double.TryParse(xlsTable.easy_getCellAt(cell1BetweenY, cell1BetweenX).getValue(), out toCompareToVal);
                                                                    }

                                                                    // if GREATER_THAN_OR_EQUAL_TO LESS_THAN make changes
                                                                    if (toCompareVal >= toCompareToVal) makeChanges = true;
                                                                    break;
                                                                }
                                                            case EasyXLS.Constants.ConditionalFormatting.OPERATOR_LESS_THAN:
                                                                {
                                                                    double toCompareToVal = 0;
                                                                    int lastIndexOfEq = firstFormula.LastIndexOf("=");
                                                                    if (!double.TryParse(firstFormula.Substring(lastIndexOfEq + 1), out toCompareToVal))
                                                                    {
                                                                        string getFirstRef = firstFormula.Substring(lastIndexOfEq + 1);
                                                                        int cell1BetweenX = (int)FormulaParser.get2DCellElements(getFirstRef)[0];
                                                                        int cell1BetweenY = (int)FormulaParser.get2DCellElements(getFirstRef)[1];
                                                                        double.TryParse(xlsTable.easy_getCellAt(cell1BetweenY, cell1BetweenX).getValue(), out toCompareToVal);
                                                                    }

                                                                    // if LESS_THAN make changes
                                                                    if (toCompareVal < toCompareToVal) makeChanges = true;
                                                                    break;
                                                                }
                                                            case EasyXLS.Constants.ConditionalFormatting.OPERATOR_GREATER_THAN:
                                                                {
                                                                    double toCompareToVal = 0;
                                                                    int lastIndexOfEq = firstFormula.LastIndexOf("=");
                                                                    if (!double.TryParse(firstFormula.Substring(lastIndexOfEq + 1), out toCompareToVal))
                                                                    {
                                                                        string getFirstRef = firstFormula.Substring(lastIndexOfEq + 1);
                                                                        int cell1BetweenX = (int)FormulaParser.get2DCellElements(getFirstRef)[0];
                                                                        int cell1BetweenY = (int)FormulaParser.get2DCellElements(getFirstRef)[1];
                                                                        double.TryParse(xlsTable.easy_getCellAt(cell1BetweenY, cell1BetweenX).getValue(), out toCompareToVal);
                                                                    }

                                                                    // if GREATER_THAN make changes
                                                                    if (toCompareVal > toCompareToVal) makeChanges = true;
                                                                    break;
                                                                }
                                                            case EasyXLS.Constants.ConditionalFormatting.OPERATOR_NOT_EQUAL_TO:
                                                                {
                                                                    double toCompareToVal = 0;
                                                                    int lastIndexOfEq = firstFormula.LastIndexOf("=");
                                                                    if (!double.TryParse(firstFormula.Substring(lastIndexOfEq + 1), out toCompareToVal))
                                                                    {
                                                                        string getFirstRef = firstFormula.Substring(lastIndexOfEq + 1);
                                                                        int cell1BetweenX = (int)FormulaParser.get2DCellElements(getFirstRef)[0];
                                                                        int cell1BetweenY = (int)FormulaParser.get2DCellElements(getFirstRef)[1];
                                                                        double.TryParse(xlsTable.easy_getCellAt(cell1BetweenY, cell1BetweenX).getValue(), out toCompareToVal);
                                                                    }

                                                                    // if NOT_EQUAL_TO make changes
                                                                    if (toCompareVal != toCompareToVal) makeChanges = true;
                                                                    break;
                                                                }
                                                            case EasyXLS.Constants.ConditionalFormatting.OPERATOR_EQUAL_TO:
                                                                {
                                                                    double toCompareToVal = 0;
                                                                    int lastIndexOfEq = firstFormula.LastIndexOf("=");
                                                                    if (!double.TryParse(firstFormula.Substring(lastIndexOfEq + 1), out toCompareToVal))
                                                                    {
                                                                        string getFirstRef = firstFormula.Substring(lastIndexOfEq + 1);
                                                                        int cell1BetweenX = (int)FormulaParser.get2DCellElements(getFirstRef)[0];
                                                                        int cell1BetweenY = (int)FormulaParser.get2DCellElements(getFirstRef)[1];
                                                                        double.TryParse(xlsTable.easy_getCellAt(cell1BetweenY, cell1BetweenX).getValue(), out toCompareToVal);
                                                                    }

                                                                    // if EQUAL_TO make changes
                                                                    if (toCompareVal == toCompareToVal) makeChanges = true;
                                                                    break;
                                                                }
                                                            case EasyXLS.Constants.ConditionalFormatting.OPERATOR_NOT_BETWEEN:
                                                                {
                                                                    double toCompareToLowVal = 0;
                                                                    double toCompareToHighVal = 0;
                                                                    int lastIndexOfEq = firstFormula.LastIndexOf("=");

                                                                    if (!double.TryParse(firstFormula.Substring(lastIndexOfEq + 1), out toCompareToLowVal))
                                                                    {
                                                                        string getFirstRef = firstFormula.Substring(lastIndexOfEq + 1);
                                                                        int cell1BetweenX = (int)FormulaParser.get2DCellElements(getFirstRef)[0];
                                                                        int cell1BetweenY = (int)FormulaParser.get2DCellElements(getFirstRef)[1];
                                                                        double.TryParse(xlsTable.easy_getCellAt(cell1BetweenY, cell1BetweenX).getValue(), out toCompareToLowVal);
                                                                    }
                                                                    if (!double.TryParse(secondFormula.Substring(lastIndexOfEq + 1), out toCompareToHighVal))
                                                                    {
                                                                        string getFirstRef = secondFormula.Substring(lastIndexOfEq + 1);
                                                                        int cell2BetweenX = (int)FormulaParser.get2DCellElements(getFirstRef)[0];
                                                                        int cell2BetweenY = (int)FormulaParser.get2DCellElements(getFirstRef)[1];
                                                                        double.TryParse(xlsTable.easy_getCellAt(cell2BetweenY, cell2BetweenX).getValue(), out toCompareToHighVal);
                                                                    }

                                                                    // if NOT_BETWEEN make changes
                                                                    if (toCompareVal < toCompareToLowVal || toCompareVal > toCompareToHighVal) makeChanges = true;
                                                                    break;
                                                                }
                                                            case EasyXLS.Constants.ConditionalFormatting.OPERATOR_BETWEEN:
                                                                {
                                                                    double toCompareToLowVal = 0;
                                                                    double toCompareToHighVal = 0;
                                                                    int lastIndexOfEq = firstFormula.LastIndexOf("=");

                                                                    if (!double.TryParse(firstFormula.Substring(firstFormula.LastIndexOf("=") + 1), out toCompareToLowVal))
                                                                    {
                                                                        string getFirstRef = firstFormula.Substring(lastIndexOfEq + 1);
                                                                        int cell1BetweenX = (int)FormulaParser.get2DCellElements(getFirstRef)[0];
                                                                        int cell1BetweenY = (int)FormulaParser.get2DCellElements(getFirstRef)[1];
                                                                        double.TryParse(xlsTable.easy_getCellAt(cell1BetweenY, cell1BetweenX).getValue(), out toCompareToLowVal);
                                                                    }
                                                                    if (!double.TryParse(secondFormula.Substring(secondFormula.LastIndexOf("=") + 1), out toCompareToHighVal))
                                                                    {
                                                                        string getFirstRef = secondFormula.Substring(lastIndexOfEq + 1);
                                                                        int cell2BetweenX = (int)FormulaParser.get2DCellElements(getFirstRef)[0];
                                                                        int cell2BetweenY = (int)FormulaParser.get2DCellElements(getFirstRef)[1];
                                                                        double.TryParse(xlsTable.easy_getCellAt(cell2BetweenY, cell2BetweenX).getValue(), out toCompareToHighVal);
                                                                    }

                                                                    // if BETWEEN make changes
                                                                    if (toCompareVal >= toCompareToLowVal && toCompareVal <= toCompareToHighVal) makeChanges = true;
                                                                    break;
                                                                }
                                                        }
                                                    }

                                                    // if conditional formatting is set to evaluate formulas
                                                    if (excConditionType == EasyXLS.Constants.ConditionalFormatting.CONDITIONAL_FORMATTING_TYPE_EVALUATE_FORMULA)
                                                    {
                                                        //get the formula parser object
                                                        EasyXLSGridFormula formulaParser = new EasyXLSGridFormula();

                                                        // get the proper key for this range 
                                                        int[] properKey = formulaParser.getTheRequiredList(xlsRow, xlsCol, listConditionals);
                                                        //get the proper list for this range
                                                        List properList = listConditionals[properKey];

                                                        // get the proper formula
                                                        String properFormula;
                                                        properFormula = "";
                                                        // tokens that will build the formula
                                                        Object oToken;

                                                        // go through all the tokens
                                                        for (int nToken = 0; nToken < properList.Count; nToken++)
                                                        {
                                                            // get token at position
                                                            oToken = properList.elementAt(nToken);

                                                            //if token is string just add it to the formula
                                                            if (oToken is String)
                                                            {
                                                                // add to the formula
                                                                properFormula += oToken;
                                                            }
                                                            else//if array
                                                            {
                                                                // get the current formula (if relative reference)
                                                                properFormula += ((String[,])oToken)[xlsRow - properKey[0], xlsCol - properKey[1]];
                                                            }

                                                        }
                                                        //save the value from the first cell
                                                        string existingValue = xlsTable.easy_getCellAt(0, 0).getValue();

                                                        // set the formula to the first cell
                                                        xlsTable.easy_getCellAt(0, 0).setValue(properFormula);

                                                        // calculate formula in the first cell
                                                        xlsTable.easy_getCellAt(0, 0).calculateFormula(gridDataDocument, excWorkSheet, excWorkSheet, 0, 0, 0, 0, true);

                                                        // get the formula result
                                                        string calcFormula = xlsTable.easy_getCellAt(0, 0).getFormulaResultValue();

                                                        // set the original value
                                                        xlsTable.easy_getCellAt(0, 0).setValue(existingValue);

                                                        // bool values to check formula result
                                                        bool checkFormula = false;
                                                        bool checkEqual = false;
                                                        double doubleResultFormula = 0;
                                                        int intResultFormula = 0;


                                                        // check if formula result is BOOLEAN VALUE
                                                        if (!bool.TryParse(calcFormula, out checkFormula))
                                                        {
                                                            // get the formula value
                                                            if (double.TryParse(calcFormula, out doubleResultFormula))
                                                            {
                                                                // the formula result must be equal to the current value
                                                                checkEqual = true;
                                                            }

                                                            // get the formula value
                                                            if (int.TryParse(calcFormula, out intResultFormula))
                                                            {
                                                                // the formula result must be equal to the current value
                                                                checkFormula = true;
                                                            }
                                                        }

                                                        // get the current value
                                                        double currentVal = 0;
                                                        double.TryParse(xlsTable.easy_getCellAt(xlsRow, xlsCol).getValue(), out currentVal);

                                                        // if formula returns the value TRUE
                                                        if (checkFormula)
                                                        {
                                                            // make the conditional formatting changes
                                                            makeChanges = true;
                                                        }
                                                        // if formula returns a numeric value and formula result equals the current value
                                                        else if (checkEqual && currentVal == doubleResultFormula)
                                                        {
                                                            // make the conditional formatting changes
                                                            makeChanges = true;
                                                        }
                                                        else makeChanges = false; // DO NOT make the conditional formatting changes
                                                    }


                                                    // if changes required
                                                    if (makeChanges)
                                                    {
                                                        // check if any modification on border formatting
                                                        if (excCondition.ContainsBorderFormatting())
                                                        {
                                                            //// change top border color and style
                                                            Color topColorBorder = getObjectThemeVSNormalColor(excCondition.getTopThemeColor_Border(), excCondition.getTopColor_Border(), xlsTheme, "border");
                                                            xlsStyle.setTopColor_Border(topColorBorder);
                                                            xlsStyle.setTopStyle_Border(excCondition.getTopStyle_Border());

                                                            // change bottom border color and style
                                                            Color bottomColorBorder = getObjectThemeVSNormalColor(excCondition.getBottomThemeColor_Border(), excCondition.getBottomColor_Border(), xlsTheme, "border");
                                                            xlsStyle.setBottomColor_Border(bottomColorBorder);
                                                            xlsStyle.setBottomStyle_Border(excCondition.getBottomStyle_Border());

                                                            // change left border color and style
                                                            Color leftColorBorder = getObjectThemeVSNormalColor(excCondition.getLeftThemeColor_Border(), excCondition.getLeftColor_Border(), xlsTheme, "border");
                                                            xlsStyle.setLeftColor_Border(leftColorBorder);
                                                            xlsStyle.setLeftStyle_Border(excCondition.getLeftStyle_Border());

                                                            // change right border color and style
                                                            Color rightColorBorder = getObjectThemeVSNormalColor(excCondition.getRightThemeColor_Border(), excCondition.getRightColor_Border(), xlsTheme, "border");
                                                            xlsStyle.setRightColor_Border(rightColorBorder);
                                                            xlsStyle.setRightStyle_Border(excCondition.getRightStyle_Border());

                                                            ////xlsStyle.setHorizontalAlignment(xlsCell.getStyle().getHorizontalAlignment());
                                                        }

                                                        // check if any modification on font formatting
                                                        if (excCondition.ContainsFontFormatting())
                                                        {
                                                            // change the underlineStyle
                                                            xlsStyle.setUnderlineStyle(excCondition.getUnderlineStyle());

                                                            // change the boldness
                                                            xlsStyle.setBold(excCondition.IsBold());
                                                            // change the italic
                                                            xlsStyle.setItalic(excCondition.IsItalic());
                                                            // change the strikethrough
                                                            xlsStyle.setStrikethrough(excCondition.IsStrikethrough());

                                                            // change the foreground
                                                            Color foreground = getObjectThemeVSNormalColor(excCondition.getForegroundThemeColor(), excCondition.getForeground(), xlsTheme, "foreground");
                                                            xlsStyle.setForeground(foreground);
                                                            if (htmlValue != "")
                                                            {
                                                                int indexOfRGBStart = htmlValue.IndexOf("(");
                                                                int indexOfRGBEnd = htmlValue.IndexOf(")");
                                                                string replacedString = htmlValue.Substring(indexOfRGBStart, indexOfRGBEnd - indexOfRGBStart + 1);
                                                                htmlValue = htmlValue.Replace(replacedString, "(" + foreground.R + "," + foreground.G + "," + foreground.B + ")");
                                                            }
                                                            //xlsStyle.setHorizontalAlignment(xlsCell.getStyle().getHorizontalAlignment());
                                                        }

                                                        // check if any modification on pattern formatting
                                                        if (excCondition.ContainsPatternFormatting())
                                                        {
                                                            // change pattern colors
                                                            Color background = getObjectThemeVSNormalColor(excCondition.getBackgroundThemeColor(), excCondition.getBackground(), xlsTheme, "background");
                                                            patternCol = background;
                                                            xlsStyle.setBackground(background);
                                                            //xlsStyle.setHorizontalAlignment(xlsCell.getStyle().getHorizontalAlignment());
                                                        }
                                                    }
                                                }
                                            }
                                        }

                                        #endregion

                                        cellStyleData.PopulateValues(xlsStyle, cellDataType, width, heightAutoSize, height, numericFormatRed, patternCol, excelComment, htmlValue);
                                    }
                                }
                                else
                                {
                                    // get the formatted data
                                    drow[grvCol] = "";
                                    bool heightAutoSize = false;
                                    cellStyleData.PopulateValues(null, "numeric", emptyCellWidth, heightAutoSize, emptyCellHeight, false, Color.Empty, null, "");
                                }

                                drowStyle[grvCol] = cellStyleData;

                                // go to next column
                                grvCol++;
                            }
                        }
                        else
                        {
                            grvRow--;
                            isHiddenRow = true;
                        }
                    }
                    else
                    {
                         // read excel file cell by cell between the given coordinates
                        for (int xlsCol = CellLeftCornerX; xlsCol < CellRightCornerX + 1; xlsCol++)
                        {
                            CellStyleData cellStyleData = new CellStyleData();
                            // get the formatted data
                            drow[grvCol] = "";
                            bool heightAutoSize = false;
                            cellStyleData.PopulateValues(null, "numeric", emptyCellWidth, heightAutoSize, emptyCellHeight, false, Color.Empty, null, "");

                            drowStyle[grvCol] = cellStyleData;

                            // go to next column
                            grvCol++;
                        }
                    }

                    if (!isHiddenRow)
                    {
                        //Add the rows to the datatables
                        dtStyles.Rows.Add(drowStyle);
                        dtValues.Rows.Add(drow);

                        if (gridViewWidth < RowWidth) gridViewWidth = RowWidth;
                    }
                    // go to next row
                    grvRow++;
                }

                Session.Add("GridViewHeight", gridViewRowHeight);
                Session.Add("GridViewWidth", gridViewWidth);
                Session.Add("GridViewRowsNo", grvRow);
            }

            return gridViewWidth;
        }

        #endregion

        #region [ Bound the Excel file to the Grid View ]

        /// <summary>
        /// BoundTheGridView
        /// </summary>
        /// <param name="grvEasyXLSGrid"></param>
        public void BoundTheGridView(GridView grvEasyXLSGrid, DataTable values)
        {
            // create actual gridview from datatable
            foreach (DataColumn col in values.Columns)
            {
                //Declare the bound field and allocate memory for the bound field.
                BoundField bfield = new BoundField();

                //Initalize the DataField value.
                bfield.DataField = col.ColumnName;

                //Initialize the HeaderText field value.
                bfield.HeaderText = col.ColumnName;

                //Add the newly created bound field to the GridView.
                grvEasyXLSGrid.Columns.Add(bfield);
            }

            //Initialize the DataSource
            grvEasyXLSGrid.DataSource = values;

            //Bind the datatable with the GridView.
            grvEasyXLSGrid.DataBind();


        }

        #endregion

        #region [ Conditional Formatting ]

       
        /// <summary>
        /// CheckForConditionalFormatting
        /// </summary>
        /// <param name="excWorkSheet"></param>
        /// <param name="rowNo"></param>
        /// <param name="colNo"></param>
        /// <returns></returns>
        ExcelConditionalFormatting CheckForConditionalFormatting(ExcelWorksheet excWorkSheet, int rowNo, int colNo)
        {
            int formCount = excWorkSheet.ConditionalFormattingCount();
            for (int formattingCount = 0; formattingCount < formCount; formattingCount++)
            {
                ExcelConditionalFormatting conditionalFormatting = excWorkSheet.easy_getConditionalFormattingAt(formattingCount);
                int firstCondRow = conditionalFormatting.getFirstRow();
                int lastCondRow = conditionalFormatting.getLastRow();
                int firstCondCol = conditionalFormatting.getFirstColumn();
                int lastCondCol = conditionalFormatting.getLastColumn();

                if (
                    rowNo >= firstCondRow && rowNo <= lastCondRow 
                    && 
                    colNo >= firstCondCol && colNo <= lastCondCol
                    )
                {
                    return conditionalFormatting;
                }
            }
            return null;
        }

        #endregion

        #region [ Merging Cells ]

        /// <summary>
        /// Merge Cells
        /// </summary>
        /// <param name="grRow"></param>
        /// <param name="style"></param>
        /// <param name="cellIdx"></param>
        /// <param name="rowNo"></param>
        /// <param name="rowOnTable"></param>
        /// <returns> merged cells number </returns>
        protected int[] MergeCells(GridView grvXLS, GridViewRow grRow, ExcelStyle style, int cellIdx, int rowNo, int rowOnTable, int mergeCellsNo)
        {
            // counter for consecutive merged cells
            int[] result = new int[2];
            int internalMergeCellsNo = 0;

            // the col span variable
            int mergeCell = 0;

            // the col span variable
            int mergeRowCell = 0;

            // get the merging property of the cell
            bool IsMergeCell = style.IsMergeCell();
            // check if merged cell
            if (IsMergeCell)
            {
                // get the index of the merging range for wich the cell is the first
                int rangeIdx = getTheIndexOfMergingRange(CellLeftCornerY + rowNo + rowOnTable, CellLeftCornerX + cellIdx + mergeCellsNo, xlsTable);

                int inRangeIdx = getTheIndexOfFirstColMergingRange(CellLeftCornerY + rowNo + rowOnTable, CellLeftCornerX + cellIdx + mergeCellsNo, xlsTable);

                // if the range exists
                if (rangeIdx >= 0)
                {
                    // get the col span
                    mergeCell = getMergingAreaColumnsCount(rangeIdx, xlsTable);
                    mergeRowCell = getMergingAreaRowsCount(rangeIdx, xlsTable);
                }

                if (inRangeIdx >= 0)
                {
                    // get the col span
                    mergeCell = getMergingAreaColumnsCount(inRangeIdx, xlsTable);
                }

                //merge cells
                if (mergeRowCell > 0)
                    grRow.Cells[cellIdx].RowSpan = mergeRowCell;

                if (mergeCell > 0)
                {
                    if (rangeIdx >= 0)
                    {
                        grRow.Cells[cellIdx].ColumnSpan = mergeCell;

                        internalMergeCellsNo += mergeCell - 1;

                        //delete cells that are not visible due merging
                        int p = 1;
                        while (p < mergeCell)
                        {
                            // remove the extra cells resulted from merging
                            if (cellIdx + 1 < grRow.Cells.Count)
                            {
                                grRow.Cells.RemoveAt(cellIdx + 1);
                            }
                            // go to next cell for delete 
                            p++;
                        }
                    }
                    else
                    {
                        internalMergeCellsNo += mergeCell;

                        //delete cells that are not visible due merging
                        int p = 1;
                        while (p <= mergeCell)
                        {
                            // remove the extra cells resulted from merging
                            if (cellIdx < grRow.Cells.Count)
                            {
                                grRow.Cells.RemoveAt(cellIdx);
                            }
                            // go to next cell for delete 
                            p++;
                        }

                    }
                }
            }

            // do not allow negative values
            result[0] = internalMergeCellsNo > 0 ? internalMergeCellsNo : 0;
            result[1] = mergeRowCell > 0 ? mergeRowCell - 1 : mergeRowCell;


            // return number of merged cells
            return result;
        }

        /// <summary>
        /// getTheIndexOfMergingRange
        /// </summary>
        /// <param name="nRowNumber"></param>
        /// <param name="nColumnNumber"></param>
        /// <param name="xlsTable"></param>
        /// <returns></returns>
        public int getTheIndexOfMergingRange(int nRowNumber, int nColumnNumber, ExcelTable xlsTable)
        {
            for (int rangeIdx = 0; rangeIdx < xlsTable.MergeCellRangesCount(); rangeIdx++)
            {
                if (nRowNumber == xlsTable.easy_getCellMergingFirstRow(rangeIdx) && nColumnNumber == xlsTable.easy_getCellMergingFirstCol(rangeIdx)
                    //&&
                    //nRowNumber <= xlsTable.easy_getCellMergingLastRow(rangeIdx) && nColumnNumber <= xlsTable.easy_getCellMergingLastCol(rangeIdx)
                    ) 
                return rangeIdx;
            }
            return -1;
        }

        /// <summary>
        /// getTheIndexOfMergingRange
        /// </summary>
        /// <param name="nRowNumber"></param>
        /// <param name="nColumnNumber"></param>
        /// <param name="xlsTable"></param>
        /// <returns></returns>
        public int checkAnyColumnMerge(int nRowNumber, int nColumnNumber, ExcelTable xlsTable)
        {
            for (int rangeIdx = 0; rangeIdx < xlsTable.MergeCellRangesCount(); rangeIdx++)
            {
                if (xlsTable.easy_getCellMergingLastCol(rangeIdx) - xlsTable.easy_getCellMergingFirstCol(rangeIdx) > 0
                    &&
                    nRowNumber <= xlsTable.easy_getCellMergingLastRow(rangeIdx) && nRowNumber >= xlsTable.easy_getCellMergingFirstRow(rangeIdx)
                    &&
                    nColumnNumber <= xlsTable.easy_getCellMergingLastCol(rangeIdx) && nColumnNumber >= xlsTable.easy_getCellMergingFirstCol(rangeIdx)
                    )
                    return rangeIdx;
            }
            return -1;
        }

        /// <summary>
        /// getTheIndexOfMergingRange
        /// </summary>
        /// <param name="nRowNumber"></param>
        /// <param name="nColumnNumber"></param>
        /// <param name="xlsTable"></param>
        /// <returns></returns>
        public int getTheIndexOfFirstColMergingRange(int nRowNumber, int nColumnNumber, ExcelTable xlsTable)
        {
            for (int rangeIdx = 0; rangeIdx < xlsTable.MergeCellRangesCount(); rangeIdx++)
            {
                if (nRowNumber >= xlsTable.easy_getCellMergingFirstRow(rangeIdx) && nColumnNumber == xlsTable.easy_getCellMergingFirstCol(rangeIdx)
                    &&
                    nRowNumber <= xlsTable.easy_getCellMergingLastRow(rangeIdx) //&& nColumnNumber <= xlsTable.easy_getCellMergingLastCol(rangeIdx)
                    )
                    return rangeIdx;
            }
            return -1;
        }


        /// <summary>
        /// getMergingAreaColumnsCount
        /// </summary>
        /// <param name="nIndex"></param>
        /// <param name="xlsTable"></param>
        /// <returns></returns>
        public int getMergingAreaColumnsCount(int nIndex, ExcelTable xlsTable)
        {
            return xlsTable.easy_getCellMergingLastCol(nIndex) - xlsTable.easy_getCellMergingFirstCol(nIndex) + 1;
        }

        /// <summary>
        /// getMergingAreaRowsCount
        /// </summary>
        /// <param name="nIndex"></param>
        /// <param name="xlsTable"></param>
        /// <returns></returns>
        public int getMergingAreaRowsCount(int nIndex, ExcelTable xlsTable)
        {
            return xlsTable.easy_getCellMergingLastRow(nIndex) - xlsTable.easy_getCellMergingFirstRow(nIndex) + 1;
        }

        #endregion

        #region [ Grid View format ]

        /// <summary>
        /// RowDataBound
        /// </summary>
        /// <param name="grRow"></param>
        /// <param name="rowNo"></param>
        public void RowDataBound(GridView grvXLS, GridViewRow grRow, int rowNo, int pageIndex, DataTable dtChart)
        {
            // the row without paging
            int rowOnTable = 0;
            // if pager set then get the actual data row
            if (pageIndex > 0) rowOnTable = grvXLS.PageSize * pageIndex;

            pageIndex += 1;  // the pager index
            int mergeCellsNo = 0; // > 0 when merged cells, total number of merged cells per row, offset for CellStyle
            int mergeChartCellsNo = 0; // > 0 when merged cells, total number of merged cells per row, offset for CellStyle
            int rowMergeCell = 0;

            // keep all merging cell refrences
            List<CellSpan> listCellSpans = new List<CellSpan>();

            // go through all cells
            for (int cellIdx = 0; cellIdx < grRow.Cells.Count; cellIdx++)
            {
                int listSpanIdx = cellIdx;

                #region [ DELETE CHART'S MERGED CELLS ]

                // CHECK IF ANY Merged cells from charts
                while (CheckIfAnyMergedCellsHere(cellIdx + mergeChartCellsNo))
                {
                    // if row belongs to the chart span
                    // POSITION_CURRENT_ROW keeps the current row from the chart row span -> this increments everytime I get in this code bellow
                    // POSITION_ROW_SPAN keep the number of rows that form the row span for the chart
                    if (mergedChartCols[cellIdx + mergeChartCellsNo][POSITION_CURRENT_ROW] < mergedChartCols[cellIdx + mergeChartCellsNo][POSITION_ROW_SPAN])
                    {
                        //delete cells that are not visible due to merging
                        // POSITION_COL_SPAN -> this position in the aray keeps the no of column included in the span
                        int p = 1;
                        while (p <= mergedChartCols[cellIdx + mergeChartCellsNo][POSITION_COL_SPAN])
                        {
                            // remove the extra cells resulted from merging if it does not contain charts/images
                            if (cellIdx < grRow.Cells.Count
                                && !grRow.Cells[cellIdx].Text.Contains("EasyXLSChartFiles")
                                && !grRow.Cells[cellIdx].Text.Contains("EasyXLSImage"))
                            {
                                grRow.Cells.RemoveAt(cellIdx);
                            }
                            // go to next cell for delete 
                            p++;
                        }

                        // get the row span no
                        int rowSpan = mergedChartCols[cellIdx + mergeChartCellsNo][POSITION_ROW_SPAN];

                        // increase the current row from this cell row span
                        mergedChartCols[cellIdx + mergeChartCellsNo][POSITION_CURRENT_ROW]++;
                        // add the column span to the counters
                        mergeCellsNo += p-1;
                        mergeChartCellsNo += p-1;

                        // keep the columns span and the row span in the List -> need this for borders
                        listCellSpans.Add(new CellSpan(listSpanIdx, mergeCellsNo, rowSpan));
                        listSpanIdx++;
                    }
                    else
                    {
                        // when all the rows from the row span have beeen
                        mergedChartCols.Remove(cellIdx + mergeChartCellsNo);
                    }
                }

                #endregion

                #region [ BIND CHART TO GRIDVIEW CELL ]

                // check if cellIdx is still valid
                if (cellIdx >= 0 && cellIdx < grRow.Cells.Count)
                {
                    // check if cell contains a chart or an image
                    if (grRow.Cells[cellIdx].Text.Contains("ChartFiles") || grRow.Cells[cellIdx].Text.Contains("EasyXLSImage"))
                    {
                        // create a new image control
                        System.Web.UI.WebControls.Image imgChart = new System.Web.UI.WebControls.Image();
                        // get the url
                        imgChart.ImageUrl = grRow.Cells[cellIdx].Text;
                        imgChart.ID = "imgChart";

                        // get data about the chart that is to be bound
                        ChartData chart = ChartData.FindChartByURL(grRow.Cells[cellIdx].Text, dtChart);

                        // get the column and the row span
                        int colSpan = chart.ColSpan;
                        int rowSpan = chart.RowSpan;

                        // set the chart's column and row span in the gridview
                        grRow.Cells[cellIdx].ColumnSpan = colSpan;
                        grRow.Cells[cellIdx].RowSpan = rowSpan;

                        // addWidth keeps the width of the cells that keep the chart
                        double addWidth = grRow.Cells[cellIdx].Width.Value;

                        // if no merged items on this cell than add one
                        if (!CheckIfAnyMergedCellsHere(cellIdx + mergeChartCellsNo))
                            mergedChartCols.Add(cellIdx + mergeChartCellsNo, new int[3] { colSpan, rowSpan, 1 });


                        //delete cells that are not visible due merging
                        int p = 1;
                        while (p <= colSpan - 1)
                        {
                            // remove the extra cells resulted from merging
                            if (cellIdx + 1 < grRow.Cells.Count
                                && !grRow.Cells[cellIdx + 1].Text.Contains("EasyXLSChartFiles")
                                && !grRow.Cells[cellIdx + 1].Text.Contains("EasyXLSImage")
                                )
                            {
                                // get the width of the cell that contains the chart
                                if (rowNo + rowOnTable < dtStyles.Rows.Count && cellIdx + mergeCellsNo + p < dtStyles.Columns.Count)
                                {
                                    // get the width and height of the cell
                                    int width = ((CellStyleData)dtStyles.Rows[rowNo + rowOnTable][(cellIdx + mergeCellsNo + p)]).XlsWidth;
                                    addWidth += width;
                                }
                                else
                                {
                                    addWidth += emptyCellWidth;
                                }

                                grRow.Cells.RemoveAt(cellIdx + 1);
                            }
                            // go to next cell for delete 
                            p++;
                        }

                       
                        grRow.Cells[cellIdx].Controls.Add(imgChart);
                      
                        // set the cell width
                        grRow.Cells[cellIdx].Width = (int)addWidth;

                        imgChart.Width = Unit.Percentage(100);
                        imgChart.Height = Unit.Percentage(100);


                        // set the cell style
                        grRow.Cells[cellIdx].Style.Add("vertical-align", "middle");
                        grRow.Cells[cellIdx].Style.Add("text-align", "center");
                        grRow.Cells[cellIdx].Style.Add("horizontal-align", "center");

                        // add columns span to the counts
                        mergeCellsNo += p - 1;
                        mergeChartCellsNo += p - 1;
                    }
                }

                #endregion

                #region [ SET CELL STYLE ]

                // check if cellIdx is still valid
                if (cellIdx >= 0 && cellIdx < grRow.Cells.Count)
                {
                    // cell must not conatin charts or images
                    if (!grRow.Cells[cellIdx].Text.Contains("ChartFiles") && !grRow.Cells[cellIdx].Text.Contains("EasyXLSImage"))
                    {
                        if (dtStyles.Columns.Count > cellIdx + mergeCellsNo && dtStyles.Rows.Count > rowNo + rowOnTable)
                        {
                            // check if any row mergeing applies here
                            int inRangeIdx = getTheIndexOfFirstColMergingRange(rowNo + rowOnTable, cellIdx + mergeCellsNo, xlsTable);
                            // check if any cell merging applies here
                            int rangeIdx = getTheIndexOfMergingRange(rowNo + rowOnTable, cellIdx + mergeCellsNo, xlsTable);

                            // if only rows are mergeing
                            if (inRangeIdx >= 0 && rangeIdx < 0) rowMergeCell += 1;

                            // get the style of the cell
                            CellStyleData cellStyleData = (CellStyleData)dtStyles.Rows[rowNo + rowOnTable][(cellIdx + mergeCellsNo)];

                            // check if the cell is populated OR not
                            if (!CellStyleData.IsEmptyStyleCell(cellStyleData))
                            {
                                // if html value(different text styles in same cell)
                                if (cellStyleData.HtmlValue != "")
                                {
                                    cellStyleData.HtmlValue = cellStyleData.HtmlValue.Replace(" size=", " font-size=");
                                    //cellStyleData.HtmlValue = cellStyleData.HtmlValue.Replace(" color=", " font-color=");
                                    grRow.Cells[cellIdx].Text = "<div>" + cellStyleData.HtmlValue + "</div>";
                                }

                                // the before merge style
                                ExcelStyle beforeMergeStyle = cellStyleData.XlsStyle.Clone();
                                // the before merge comments
                                ExcelComment beforeMergeXLSComment = cellStyleData.XlsComments;

                                // check the before merge style
                                if (beforeMergeStyle != null)
                                {
                                    // merge cells if neccesary
                                    int checkMergeCells = 0;
                                    int[] mergeingRes = new int[2];
                                    int[] totalMergeingRes = new int[2];

                                    // the code must be repetitive in case of several consecutive merges
                                    do
                                    {
                                        mergeingRes = MergeCells(grvXLS, grRow, beforeMergeStyle, cellIdx, rowNo, rowOnTable, mergeCellsNo);
                                        checkMergeCells = mergeingRes[0];
                                        // set merges columns
                                        totalMergeingRes[0] += mergeingRes[0];
                                        // set merged rows
                                        totalMergeingRes[1] += mergeingRes[1];

                                        if (checkMergeCells != 0)
                                        {
                                            mergeCellsNo += checkMergeCells;
                                            listCellSpans.Add(new CellSpan(cellIdx, mergeCellsNo, mergeingRes[1]));
                                        }

                                    }
                                    while (checkMergeCells > 0);

                                    //  cehck if cellIdx Is still valid
                                    if (cellIdx + mergeCellsNo < dtStyles.Columns.Count)
                                    {
                                        // get the style after merge
                                        cellStyleData = (CellStyleData)dtStyles.Rows[rowNo + rowOnTable][(cellIdx + mergeCellsNo)];

                                        // if after merge style exists
                                        if (cellStyleData.XlsStyle != null)
                                        {
                                            // after merge style
                                            ExcelStyle style = cellStyleData.XlsStyle.Clone();

                                            // after merge data type
                                            string dataType = cellStyleData.XlsDataType;

                                            // get the width and height of the cell
                                            int width = cellStyleData.XlsWidth;
                                            int height = cellStyleData.XlsHeight;

                                            bool isNumericRed = cellStyleData.IsFormatRed;
                                            Color patternCol = cellStyleData.PatternColor;

                                            // set border color and style
                                            string styleAttributes = "";

                                            // set commants if exists
                                            ExcelComment excelComment;
                                            if (beforeMergeXLSComment != null)
                                            {
                                                excelComment = beforeMergeXLSComment;
                                            }
                                            else
                                            {
                                                /* set comments on cell tool tip */
                                                excelComment = cellStyleData.XlsComments;
                                            }
                                            if (excelComment != null)
                                            {
                                                string fontFamily = excelComment.getFont();
                                                string fontSize = excelComment.getFontSize().ToString();
                                                Color commentForeground = excelComment.getForeground();
                                                string fontColorR = commentForeground.R.ToString();
                                                string fontColorG = commentForeground.G.ToString();
                                                string fontColorB = commentForeground.B.ToString();
                                                string commentWidth = excelComment.getWidth().ToString();
                                                string commentHeight = excelComment.getHeight().ToString();
                                                string bold = excelComment.IsBold() ? "bold" : "normal";
                                                string italic = excelComment.IsItalic() ? "italic" : "normal";

                                                string toolTip = "<div class='tooltip' style='color:rgb(" + fontColorR + "," + fontColorG + "," + fontColorB + "); font-family:" + fontFamily + "; font-size:" + fontSize + "pt; font-weight:" + bold + "; font-style:" + italic + "; width:" + commentWidth + "px; height:" + commentHeight + "px;'>" + ReplaceForTooltip(excelComment.getText()) + "</div>";
                                                //grRow.Cells[cellIdx].Attributes.Add("title", toolTip);
                                                styleAttributes = "background-image: url(../EasyXLSImage/comment.gif);background-repeat:no-repeat; background-position:right top;background-size:7px 7px;";
                                                grRow.Cells[cellIdx].CssClass = "tdComment";
                                                grRow.Cells[cellIdx].Text = grRow.Cells[cellIdx].Text + toolTip;
                                            }

                                            // new style chosen from before merging and after merging styles (depending on row merging)
                                            ExcelStyle checkStyle = new ExcelStyle();

                                            // check if any row merging cells
                                            if (rowMergeCell > 0) checkStyle = style.Clone();
                                            else checkStyle = beforeMergeStyle.Clone();

                                            // background color
                                            Color background = getObjectThemeVSNormalColor(checkStyle.getBackgroundThemeColor(), checkStyle.getBackground(), xlsTheme, "background");
                                            grRow.Cells[cellIdx].BackColor = background;

                                            if (patternCol.R != 255 || patternCol.G != 255 || patternCol.B != 255)
                                            {
                                                grRow.Cells[cellIdx].BackColor = patternCol;
                                            }

                                            // span is added to keep the height of the cell in place
                                            string spanAttributes = "<span style='";

                                            double maxRowHeight = grRow.Height.Value;
                                            if (cellStyleData.XlsAutoSizeHeight)
                                            {
                                                maxRowHeight = MAX_ROW_HEIGHT;

                                                spanAttributes += "max-height:" + maxRowHeight + "px;display:inline;overflow:hidden;'>";
                                            }
                                            else
                                            {
                                                spanAttributes += "height:" + grRow.Height + ";max-height:" + maxRowHeight + "px;display:block;overflow:hidden;'>";
                                            }

                                           

                                            // add the span to the cell
                                            grRow.Cells[cellIdx].Text = spanAttributes + grRow.Cells[cellIdx].Text + "</span>";

                                          

                                            // FORMAT
                                            //string format = checkStyle.getFormat();

                                            // format the numbers
                                            //EasyXLSGridFormat.FormatTheNumbers(grRow, format, cellIdx);

                                            // format date and time
                                            //EasyXLSGridFormat.FormatTheDateAndTime(grRow, format, cellIdx);

                                            if (dtStyles.Columns.Count > cellIdx + mergeCellsNo)
                                            {
                                                //styleAttributes += "display:block;max-height:" + grRow.Height + ";";
                                                // set font attributes
                                                styleAttributes += EasyXLSGridBorder.SetFontAttributes(grRow, checkStyle, cellIdx, isNumericRed, xlsTheme);

                                                //set alignment
                                                styleAttributes += EasyXLSGridBorder.SetAlignment(grRow, checkStyle, cellIdx, dataType);

                                                // get the style of the cell after merging (if that's the case)
                                                styleAttributes += EasyXLSGridBorder.SetBorderColorStyle(grvXLS, grRow, style, beforeMergeStyle, cellIdx, rowNo, rowOnTable, mergeCellsNo, totalMergeingRes[0], totalMergeingRes[1], dtStyles, rowMergeCell, mergedCells, xlsTheme);

                                                //check if next cell is empty or not
                                                if (grRow.Cells.Count > cellIdx + 1)
                                                {
                                                    if (grRow.Cells[cellIdx + 1].Text == "&nbsp;" && grRow.Cells[cellIdx].Text != "&nbsp;")
                                                        styleAttributes += "overflow: visible;";

                                                    else
                                                        styleAttributes += "overflow: hidden;";

                                                }
                                            }

                                            // add the style
                                            grRow.Cells[cellIdx].Attributes["Style"] = styleAttributes;
                                        }
                                    }
                                }
                            }
                        }

                        if (cellIdx < grRow.Cells.Count)
                        {
                            grRow.Cells[cellIdx].Text = grRow.Cells[cellIdx].Text.Replace("\r\n", "<br/>");
                        }
                    }

                }

                #endregion

            }
            mergedCells.Add(rowNo, listCellSpans);
        }


        /// <summary>
        /// CheckIfAnyMergedCellsHere
        /// </summary>
        /// <param name="colPosition"></param>
        /// <returns></returns>
        bool CheckIfAnyMergedCellsHere(int colPosition)
        {
            foreach (int key in mergedChartCols.Keys)
            {
                if (key == colPosition)
                    return true;
            }
            return false;
        }


        #endregion

        #region [ Check if any images in document ]
           /// <summary>
        /// DoesFileContainCharts
        /// </summary>
        /// <param name="filePath">filePath</param>
        /// <returns></returns>
        public bool DoesFileContainImages(ExcelDocument document, int sheetNo)
        {
            bool fileContainsImages = false;
            int noOfImages = 0;

            // Get the current worksheet
            ExcelSheet sheet = document.easy_getSheetAt(sheetNo);
            if (sheet is ExcelWorksheet)
            {
                ExcelWorksheet excelWorksheet = (ExcelWorksheet)sheet;
                // Get the number of charts
                noOfImages = excelWorksheet.ImageCount();
                if (noOfImages > 0) fileContainsImages = true;

            }

            return fileContainsImages;
        }

        #endregion


        //keep
        #region [ Check if any Charts in document ]

        /// <summary>
        /// DoesFileContainCharts
        /// </summary>
        /// <param name="filePath">filePath</param>
        /// <returns></returns>
        public bool DoesFileContainCharts(ExcelDocument document, int sheetNo)
        {
            bool fileContainsCharts = false;
            int noOfCharts = 0;

            ExcelSheet sheet = document.easy_getSheetAt(sheetNo);
            if (sheet is ExcelWorksheet)
            {
                ExcelWorksheet excelWorksheet = (ExcelWorksheet)sheet;
                // Get the number of charts
                noOfCharts = excelWorksheet.ChartCount();
                if (noOfCharts > 0) fileContainsCharts = true;

            }

            if(sheet is ExcelChartSheet)
            {
                noOfCharts = 1;
                fileContainsCharts = true;
            }
           
            return fileContainsCharts;
        }

        #endregion

      

        /// <summary>
        /// Replaces string for the display of the tooltip
        /// </summary>
        /// <param name="strTooltip">tooltip</param>
        /// <returns>modified string</returns>
        public string ReplaceForTooltip(string strTooltip)
        {
            strTooltip = strTooltip.Replace("'", "\\'");
            strTooltip = Server.HtmlEncode(strTooltip);
            strTooltip = strTooltip.Replace("\r\n", "<br>");
            strTooltip = strTooltip.Replace("\n", "<br>");

            return strTooltip;
        }

        //keep
        #region [ Load Grid View using File ]

        /// <summary>
        /// LoadGridViewUsingFile
        /// </summary>
        /// <param name="cellLeftCornerX">cellLeftCornerX</param>
        /// <param name="cellLeftCornerY">cellLeftCornerY</param>
        /// <param name="cellRightCornerX">cellRightCornerX</param>
        /// <param name="cellRightCornerY">cellRightCornerY</param>
        /// <param name="pathToFile">pathToFile</param>
        /// <returns>void</returns>
        public DataTable LoadDataTableUsingFile(string pathToFile, GridView grvEasyXLSGrid, DataTable dtCharting)
        {
            // get the last Row and column based on the chart's height and width
            int [] NoOfMaxRowColumns = ChartData.FindTheLastRowAndColumn(dtCharting);
            int NoOfColumns = NoOfMaxRowColumns[1];
            int NoOfRows = NoOfMaxRowColumns[0];

            // get the excel data tables
            GetExcelDataTables(pathToFile);

            // for all rows in the table
            for (int rowIdx = 0; rowIdx < dtCharting.Rows.Count; rowIdx++)
            {
                // for all columns in the table
                for (int colIdx = 0; colIdx < dtCharting.Columns.Count; colIdx++)
                {
                    if (dtCharting.Rows[rowIdx][colIdx].ToString() != "")
                    {
                        ChartData chtData = (ChartData)dtCharting.Rows[rowIdx][colIdx];
                        // calculates the column span and the row span for all the charts/images
                        ((ChartData)dtCharting.Rows[rowIdx][colIdx]).ColSpan = chtData.GetChartColSpan(dtStyles, chtData.RowPosition, chtData.ColPosition, xlsTable);
                        ((ChartData)dtCharting.Rows[rowIdx][colIdx]).RowSpan = chtData.GetChartRowSpan(dtStyles, chtData.RowPosition, chtData.ColPosition, xlsTable);
                    }
                }
            }

            // dtTogether is the table that keeps both the values and the charts/images together
            DataTable dtTogether = new DataTable();
            if (dtCharting.Columns.Count > 0)
            {
                dtTogether = ChartData.CreateOne(dtValues, dtCharting, CellLeftCornerX, CellLeftCornerY);
            }
            else
            {
                dtTogether = dtValues;
            }

            for(int colIdx= dtValues.Columns.Count; colIdx< dtTogether.Columns.Count; colIdx++)
            {
                gridViewWidth += emptyCellWidth;
            }

            return dtTogether;
        }

        #endregion

        
    }
}

