﻿/*
 Copyright (c) 2012-2014 EU Outsourcing, Inc
All rights reserved.  http://www.euoutsourcing.com
Version: 2.1.0
Disclaimer: 
Any redistribution, modification and use of this software either in source or binary forms is permitted provided that the following terms are met: 
1. This software’s source code is not modified to avoid the use of EasyXLS™. 
2.  All servers and computers, whether physical or virtual, using this software or its derivatives must be properly licensed to use EasyXLS™.
3. Any derivatives of this source code that is redistributed must include the above copyright notice, these five conditions and the included disclaimer. 
4. Any redistributions in binary form must include the above copyright notice, these five conditions and the included disclaimer in any materials that are provided with the distribution. 
5. EasyXLS™ and the name of its parent organization may not be used to endorse or promote any software products or derivative works without specific prior written permission from the copyright holders.
DISCLAIMER.  THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using EasyXLS;
using EasyXLS.Constants;
using System.Drawing;
using System.Data;
using EasyXLS.Util;
using System.Web.UI.WebControls;

namespace EasyXLSDemoApp.EasyXLSData
{
    // class for storing cell number/date formats
    public class EasyXLSGridFormat
    {
        #region [ Format the Date AND Time Grid View ROW ]

        /// <summary>
        /// FormatTheDateAndTime
        /// </summary>
        /// <param name="grvEasyXLSGrid"></param>
        public static void FormatTheDateAndTime(GridViewRow grRow, string format, int colIdx)
        {
            // init dtDate for datetime parsing
            DateTime dtDate;

            // Format data like 05-Mar-12
            if (format.Contains("dd\\-mmm\\-yy;@"))
            {
                if (!DateTime.TryParse(grRow.Cells[colIdx].Text, out dtDate))
                { grRow.Cells[colIdx].Text = ""; }
                else
                { grRow.Cells[colIdx].Text = dtDate.ToString("dd-MMM-yy"); } // 05-Mar-12 

                grRow.Cells[colIdx].HorizontalAlign = HorizontalAlign.Right;
            }
            else
                // Format data like 5-Mar-12
                if (format.Contains("d\\-mmm\\-yy;@"))
                {
                    if (!DateTime.TryParse(grRow.Cells[colIdx].Text, out dtDate))
                    { grRow.Cells[colIdx].Text = ""; }
                    else
                    { grRow.Cells[colIdx].Text = dtDate.ToString("d-MMM-yy"); } // 5-Mar-12

                    grRow.Cells[colIdx].HorizontalAlign = HorizontalAlign.Right;
                }
                else
                    // Format data like 5-Mar-2012
                    if (format.Contains("d\\-mmm\\-yyyy;@"))
                    {
                        if (!DateTime.TryParse(grRow.Cells[colIdx].Text, out dtDate))
                        { grRow.Cells[colIdx].Text = ""; }
                        else
                        { grRow.Cells[colIdx].Text = dtDate.ToString("d-MMM-yyyy"); } // 5-Mar-2012

                        grRow.Cells[colIdx].HorizontalAlign = HorizontalAlign.Right;
                    }
                    else
                        // Format data like M-12
                        if (format.Contains("mmmmm\\-yy;@"))
                        {
                            if (!DateTime.TryParse(grRow.Cells[colIdx].Text, out dtDate))
                            { grRow.Cells[colIdx].Text = ""; }
                            else
                            { grRow.Cells[colIdx].Text = dtDate.ToString("MMMMM-yy"); } // M-12

                            grRow.Cells[colIdx].HorizontalAlign = HorizontalAlign.Right;
                        }
                        else
                            // Format data like March-12
                            if (format.Contains("mmmm\\-yy;@"))
                            {
                                if (!DateTime.TryParse(grRow.Cells[colIdx].Text, out dtDate))
                                { grRow.Cells[colIdx].Text = ""; }
                                else
                                { grRow.Cells[colIdx].Text = dtDate.ToString("MMMM-yy"); } // March-12

                                grRow.Cells[colIdx].HorizontalAlign = HorizontalAlign.Right;
                            }
                            else
                                // Format data like Mar-12
                                if (format.Contains("mmm\\-yy;@"))
                                {
                                    if (!DateTime.TryParse(grRow.Cells[colIdx].Text, out dtDate))
                                    { grRow.Cells[colIdx].Text = ""; }
                                    else
                                    { grRow.Cells[colIdx].Text = dtDate.ToString("MMM-yy"); } // Mar-12

                                    grRow.Cells[colIdx].HorizontalAlign = HorizontalAlign.Right;
                                }
                                else
                                    // Format data like 3/5/2012
                                    if (format == "m/d/yyyy")
                                    {
                                        if (!DateTime.TryParse(grRow.Cells[colIdx].Text, out dtDate))
                                        { grRow.Cells[colIdx].Text = ""; }
                                        else
                                        { grRow.Cells[colIdx].Text = dtDate.ToString("M/d/yyyy"); }

                                        grRow.Cells[colIdx].HorizontalAlign = HorizontalAlign.Right;
                                    }
                                    else
                                        // Format data like Monday, March 5, 2012
                                        if (format.Contains("dddd\\,\\ mmmm\\ dd\\,\\ yyyy"))
                                        {

                                            if (!DateTime.TryParse(grRow.Cells[colIdx].Text, out dtDate))
                                            { grRow.Cells[colIdx].Text = ""; }
                                            else
                                            { grRow.Cells[colIdx].Text = dtDate.ToString("D"); }// Monday, March 5, 2012

                                            grRow.Cells[colIdx].HorizontalAlign = HorizontalAlign.Right;
                                        }
                                        else
                                            // Format data like 3/5
                                            if (format == "m/d;@")
                                            {
                                                if (!DateTime.TryParse(grRow.Cells[colIdx].Text, out dtDate))
                                                { grRow.Cells[colIdx].Text = ""; }
                                                else
                                                { grRow.Cells[colIdx].Text = dtDate.ToString("M/dd"); } // 3/5 

                                                grRow.Cells[colIdx].HorizontalAlign = HorizontalAlign.Right;
                                            }
                                            else
                                                // Format data like 3/5/12
                                                if (format == "m/d/yy;@")
                                                {
                                                    if (!DateTime.TryParse(grRow.Cells[colIdx].Text, out dtDate))
                                                    { grRow.Cells[colIdx].Text = ""; }
                                                    else
                                                    { grRow.Cells[colIdx].Text = dtDate.ToString("M/d/yy"); } // 3/5/12 

                                                    grRow.Cells[colIdx].HorizontalAlign = HorizontalAlign.Right;
                                                }
                                                else
                                                    // Format data like 03/05/12
                                                    if (format == "mm/dd/yy;@")
                                                    {
                                                        if (!DateTime.TryParse(grRow.Cells[colIdx].Text, out dtDate))
                                                        { grRow.Cells[colIdx].Text = ""; }
                                                        else
                                                        { grRow.Cells[colIdx].Text = dtDate.ToString("MM/dd/yy"); } // 03/05/12

                                                        grRow.Cells[colIdx].HorizontalAlign = HorizontalAlign.Right;
                                                    }
                                                    else
                                                        // Format data like 5-Mar
                                                        if (format.Contains("d\\-mmm;@"))
                                                        {
                                                            if (!DateTime.TryParse(grRow.Cells[colIdx].Text, out dtDate))
                                                            { grRow.Cells[colIdx].Text = ""; }
                                                            else
                                                            { grRow.Cells[colIdx].Text = dtDate.ToString("d-MMM"); } // 5-Mar 

                                                            grRow.Cells[colIdx].HorizontalAlign = HorizontalAlign.Right;
                                                        }
                                                        else
                                                            // Format data like March 5, 2012
                                                            if (format.Contains("mmmm\\ d\\,\\ yyyy;@"))
                                                            {
                                                                if (!DateTime.TryParse(grRow.Cells[colIdx].Text, out dtDate))
                                                                { grRow.Cells[colIdx].Text = ""; }
                                                                else
                                                                { grRow.Cells[colIdx].Text = dtDate.ToString("MMMM d, yyyy"); } // March 5, 2012

                                                                grRow.Cells[colIdx].HorizontalAlign = HorizontalAlign.Right;
                                                            }
                                                            else
                                                                // Format data like 3/5/12 3:05 AM
                                                                if (format.Contains("m/d/yy\\ h:mm\\ AM/PM;@"))
                                                                {
                                                                    if (!DateTime.TryParse(grRow.Cells[colIdx].Text, out dtDate))
                                                                    { grRow.Cells[colIdx].Text = ""; }
                                                                    else
                                                                    { grRow.Cells[colIdx].Text = dtDate.ToString("M/d/yy h:mm tt"); } // 3/5/12 3:05 AM

                                                                    grRow.Cells[colIdx].HorizontalAlign = HorizontalAlign.Right;
                                                                }
                                                                else
                                                                    // Format data like M
                                                                    if (format.Contains("mmmmm;@"))
                                                                    {
                                                                        if (!DateTime.TryParse(grRow.Cells[colIdx].Text, out dtDate))
                                                                        { grRow.Cells[colIdx].Text = ""; }
                                                                        else
                                                                        { grRow.Cells[colIdx].Text = dtDate.ToString("MMMMM"); } // M 

                                                                        grRow.Cells[colIdx].HorizontalAlign = HorizontalAlign.Right;
                                                                    }
                                                                    else
                                                                        // Format data like 3/5/12 3:05
                                                                        if (format.Contains("m/d/yy\\ h:mm;@"))
                                                                        {
                                                                            if (!DateTime.TryParse(grRow.Cells[colIdx].Text, out dtDate))
                                                                            { grRow.Cells[colIdx].Text = ""; }
                                                                            else
                                                                            { grRow.Cells[colIdx].Text = dtDate.ToString("M/d/yy h:mm"); } // 3/5/12 3:05

                                                                            grRow.Cells[colIdx].HorizontalAlign = HorizontalAlign.Right;
                                                                        }
                                                                        else
                                                                            // Format data like 3/5/2012
                                                                            if (format.Contains("m/d/yyyy;@"))
                                                                            {
                                                                                if (!DateTime.TryParse(grRow.Cells[colIdx].Text, out dtDate))
                                                                                { grRow.Cells[colIdx].Text = ""; }
                                                                                else
                                                                                { grRow.Cells[colIdx].Text = dtDate.ToString("M/d/yyyy"); } // 3/5/2012

                                                                                grRow.Cells[colIdx].HorizontalAlign = HorizontalAlign.Right;
                                                                            }
                                                                            else
                                                                                // Format time like 3:56:23 AM/PM
                                                                                if (format.Contains("h:mm:ss"))
                                                                                {
                                                                                    if (!DateTime.TryParse(grRow.Cells[colIdx].Text, out dtDate))
                                                                                    { grRow.Cells[colIdx].Text = ""; }
                                                                                    else
                                                                                    { grRow.Cells[colIdx].Text = dtDate.ToString("hh:mm:ss tt"); }

                                                                                    grRow.Cells[colIdx].HorizontalAlign = HorizontalAlign.Right;
                                                                                }
        }

#endregion

        #region [ Format Date AND Time string ]

        /// <summary>
        /// FormatTheDateAndTime
        /// </summary>
        /// <param name="format"></param>
        public static string FormatTheDateAndTime(string xlsFormat)
        {
            // init dtDate for datetime parsing
            string webFormat = "";

            // Format data like 05-Mar-12
            if (xlsFormat.Contains("dd\\-mmm\\-yy;@"))
            {
                webFormat = "dd-MMM-yy";
            }
            else
                // Format data like 5-Mar-12
                if (xlsFormat.Contains("d\\-mmm\\-yy;@"))
                {
                    webFormat = "d-MMM-yy"; // 5-Mar-12
                }
                else
                    // Format data like 5-Mar-2012
                    if (xlsFormat.Contains("d\\-mmm\\-yyyy;@"))
                    {
                        webFormat = "d-MMM-yyyy"; // 5-Mar-2012
                    }
                    else
                        // Format data like M-12
                        if (xlsFormat.Contains("mmmmm\\-yy;@"))
                        {
                            webFormat = "MMMMM-yy"; // M-12
                        }
                        else
                            // Format data like March-12
                            if (xlsFormat.Contains("mmmm\\-yy;@"))
                            {
                                webFormat = "MMMM-yy"; // March-12
                            }
                            else
                                // Format data like Mar-12
                                if (xlsFormat.Contains("mmm\\-yy;@"))
                                {
                                    webFormat = "MMM-yy"; // Mar-12
                                }
                                else
                                    // Format data like 3/5/2012
                                    if (xlsFormat == "m/d/yyyy")
                                    {
                                        webFormat = "M/d/yyyy";
                                    }
                                    else
                                        // Format data like Monday, March 5, 2012
                                        if (xlsFormat.Contains("dddd\\,\\ mmmm\\ dd\\,\\ yyyy"))
                                        {
                                            webFormat = "D"; // Monday, March 5, 2012
                                        }
                                        else
                                            // Format data like 3/5
                                            if (xlsFormat == "m/d;@")
                                            {
                                                webFormat = "M/dd"; // 3/5 
                                            }
                                            else
                                                // Format data like 3/5/12
                                                if (xlsFormat == "m/d/yy;@")
                                                {
                                                    webFormat = "M/d/yy"; // 3/5/12 
                                                }
                                                else
                                                    // Format data like 03/05/12
                                                    if (xlsFormat == "mm/dd/yy;@")
                                                    {
                                                        webFormat = "MM/dd/yy"; // 03/05/12
                                                    }
                                                    else
                                                        // Format data like 5-Mar
                                                        if (xlsFormat.Contains("d\\-mmm;@"))
                                                        {
                                                            webFormat = "d-MMM"; // 5-Mar 
                                                        }
                                                        else
                                                            // Format data like March 5, 2012
                                                            if (xlsFormat.Contains("mmmm\\ d\\,\\ yyyy;@"))
                                                            {
                                                                webFormat = "MMMM d, yyyy"; // March 5, 2012
                                                            }
                                                            else
                                                                // Format data like 3/5/12 3:05 AM
                                                                if (xlsFormat.Contains("m/d/yy\\ h:mm\\ AM/PM;@"))
                                                                {
                                                                    webFormat = "M/d/yy h:mm tt"; // 3/5/12 3:05 AM
                                                                }
                                                                else
                                                                    // Format data like M
                                                                    if (xlsFormat.Contains("mmmmm;@"))
                                                                    {
                                                                        webFormat = "MMMMM"; // M 
                                                                    }
                                                                    else
                                                                        // Format data like 3/5/12 3:05
                                                                        if (xlsFormat.Contains("m/d/yy\\ h:mm;@"))
                                                                        {
                                                                            webFormat = "M/d/yy h:mm"; // 3/5/12 3:05
                                                                        }
                                                                        else
                                                                            // Format data like 3/5/2012
                                                                            if (xlsFormat.Contains("m/d/yyyy;@"))
                                                                            {
                                                                                webFormat = "M/d/yyyy"; // 3/5/2012
                                                                            }
                                                                            else
                                                                                // Format time like 3:56:23 AM/PM
                                                                                if (xlsFormat.Contains("h:mm:ss"))
                                                                                {
                                                                                    webFormat = "hh:mm:ss tt";
                                                                                }
            return webFormat;
        }
#endregion

        #region [Format TIME string]

        /// <summary>
        /// ReturnUnreadableDateTimeValues
        /// </summary>
        /// <returns></returns>
        public static string ReturnUnreadableDateTimeValues(DateTime value, string format)
        {
            string dateFormat = "";

            if (format.Contains("[h]:mm:ss"))
            {
                // get the formatted data
                dateFormat = "HH:mm:ss";
            }
            else
                if (format.Contains("hh:mm:ss"))
                {
                    string tt = "";
                    if (format.Contains("AM/PM")) tt = " tt";

                    if (tt == "")
                    {
                        // get the formatted data
                        dateFormat = "HH:mm:ss" + tt;
                    }
                    else
                    {
                        // get the formatted data
                        dateFormat = "hh:mm:ss" + tt;
                    }
                }
                else
                    if (format.Contains("h:mm:ss"))
                    {

                        string tt = "";
                        if (format.Contains("AM/PM")) tt = " tt";

                        if (tt == "")
                        {
                            // get the formatted data
                            dateFormat = "H:mm:ss" + tt;
                        }
                        else
                        {
                            // get the formatted data
                            dateFormat = "h:mm:ss" + tt;
                        }
                    }
                    else
                        if (format.Contains("hh:mm"))
                        {

                            string tt = "";
                            if (format.Contains("AM/PM")) tt = " tt";

                            if (tt == "")
                            {
                                // get the formatted data
                                dateFormat = "HH:mm" + tt;
                            }
                            else
                            {
                                // get the formatted data
                                dateFormat = "hh:mm" + tt;
                            }
                        }
                        else
                            if (format.Contains("h:mm"))
                            {

                                string tt = "";
                                if (format.Contains("AM/PM")) tt = " tt";

                                if (tt == "")
                                {
                                    // get the formatted data
                                    dateFormat = "H:mm" + tt;
                                }
                                else
                                {
                                    // get the formatted data
                                    dateFormat += "h:mm" + tt;
                                }
                            }
                            else
                                if (format.StartsWith("mm:ss"))
                                {
                                    dateFormat += "mm:ss.f";
                                }

            return value.ToString(dateFormat);

        }


        #endregion

        #region [ Format the Numbers ]

        /// <summary>
        /// FormatTheNumbers
        /// </summary>
        /// <param name="grvRow"></param>
        /// <param name="format"></param>
        /// <param name="i"></param>
        public static void FormatTheNumbers(GridViewRow grRow, string format, int colIdx)
        {
            // get cell text, for check if filled with blank only
            string strCellText = grRow.Cells[colIdx].Text;
            // remove HTML space &nbsp;
            strCellText = strCellText.Replace("&nbsp;", "");
            // remove any other space char
            strCellText = strCellText.Trim();

            double tryParse = 0.0;

            if (double.TryParse(strCellText, out tryParse))
            {

                // if cell has general formatting
                if (format == "General")
                {
                    double checkIfNumber;
                    // if the cell value can be converted to a numeric type
                    if (double.TryParse(grRow.Cells[colIdx].Text, out checkIfNumber))
                    {
                        // align text to the right
                        grRow.Cells[colIdx].HorizontalAlign = HorizontalAlign.Right;
                    }
                }

                // format as percent
                if (format.Contains("%"))
                {
                    // if cell not empty
                    if (strCellText != "")
                    {
                        // check how many 0 after dot
                        int noOfZeros = format.Substring(format.LastIndexOf(".") + 1).Length - 1;

                        double percent = double.Parse(grRow.Cells[colIdx].Text);
                        double add = Math.Round(percent * 100, noOfZeros);

                        string doubleFormat = format.Substring(0, format.Length - 1); // exclude the "%"
                        string theFormatedNumber = add.ToString(doubleFormat);

                        grRow.Cells[colIdx].Text = theFormatedNumber + "%";
                        grRow.Cells[colIdx].HorizontalAlign = HorizontalAlign.Right;
                    }
                }
                else
                    // format as number
                    if (format.StartsWith("0.0"))
                    {
                        // if cell not empty
                        if (strCellText != "")
                        {
                            // check how many 0 after dot
                            int noOfDecimals = format.Substring(format.LastIndexOf(".") + 1).Length;

                            //convert to double
                            double theNumber = 0;
                            if (double.TryParse(grRow.Cells[colIdx].Text, out theNumber))
                            {
                                grRow.Cells[colIdx].Text = String.Format(System.Globalization.CultureInfo.CreateSpecificCulture("en-us"), "{0:N" + noOfDecimals.ToString() + "}", theNumber);
                            }
                            grRow.Cells[colIdx].HorizontalAlign = HorizontalAlign.Right;
                        }
                    }

                // format as integer number
                if (format == "0")
                {
                    // if cell not empty
                    if (strCellText != "")
                    {
                        //convert to int
                        double intNumber = 0;
                        if (double.TryParse(grRow.Cells[colIdx].Text, out intNumber))
                        {
                            grRow.Cells[colIdx].Text = String.Format(System.Globalization.CultureInfo.CreateSpecificCulture("en-us"), "{0:N" + 0 + "}", intNumber);
                        }
                        grRow.Cells[colIdx].HorizontalAlign = HorizontalAlign.Right;
                    }
                }


                // format as currency
                if (format.Contains("\"$\"#,##0"))
                {
                    // if cell not empty
                    if (strCellText != "")
                    {
                        // check how many 0 after dot
                        int lastindexOfDot = format.LastIndexOf(".");
                        int noOfdecimals = 0;
                        if (lastindexOfDot != -1)
                        {
                            noOfdecimals = format.Substring(lastindexOfDot + 1).Length;
                        }

                        //convert to currency
                        double dbNumber = 0;
                        if (double.TryParse(grRow.Cells[colIdx].Text, out dbNumber))
                        {
                            grRow.Cells[colIdx].Text = String.Format(System.Globalization.CultureInfo.CreateSpecificCulture("en-us"), "{0:C" + noOfdecimals.ToString() + "}", dbNumber);
                        }
                        grRow.Cells[colIdx].HorizontalAlign = HorizontalAlign.Right;
                    }
                }
            }
        }

        #endregion
    }
}