﻿/*
 Copyright (c) 2012-2014 EU Outsourcing, Inc
All rights reserved.  http://www.euoutsourcing.com
Version: 2.1.0
Disclaimer: 
Any redistribution, modification and use of this software either in source or binary forms is permitted provided that the following terms are met: 
1. This software’s source code is not modified to avoid the use of EasyXLS™. 
2.  All servers and computers, whether physical or virtual, using this software or its derivatives must be properly licensed to use EasyXLS™.
3. Any derivatives of this source code that is redistributed must include the above copyright notice, these five conditions and the included disclaimer. 
4. Any redistributions in binary form must include the above copyright notice, these five conditions and the included disclaimer in any materials that are provided with the distribution. 
5. EasyXLS™ and the name of its parent organization may not be used to endorse or promote any software products or derivative works without specific prior written permission from the copyright holders.
DISCLAIMER.  THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using EasyXLS;
using EasyXLS.Constants;
using System.Drawing;
using System.Data;
using EasyXLS.Util;

namespace EasyXLSDemoApp.EasyXLSData
{
    // class for storing cell style, width and height
    public class EasyXLSGridFormula
    {
        const int FirstRowRange = 0;
        const int FirstColRange = 1;
        const int LastRowRange = 2;
        const int LastColRange = 3;

       /// <summary>
        /// getTheRequiredList
       /// </summary>
       /// <param name="currentRow"></param>
       /// <param name="currentColumn"></param>
       /// <param name="conditionalsList"></param>
       /// <returns></returns>
        public int[] getTheRequiredList(int currentRow, int currentColumn, Dictionary<int[], List> conditionalsList)
        {
            foreach (int[] array in conditionalsList.Keys)
            {
                if (
                    array[FirstRowRange] <= currentRow && currentRow <= array[LastRowRange]
                    &&
                    array[FirstColRange] <= currentColumn && currentColumn <= array[LastColRange]
                   )
                {
                    return array;
                }

            }

            return null;
        }

        /// <summary>
        /// getDynamicReferenceFormula
        /// </summary>
        /// <param name="nFirstRow">nFirstRow</param>
        /// <param name="nFirstColumn">nFirstColumn</param>
        /// <param name="nLastRow">nLastRow</param>
        /// <param name="nLastColumn">nLastColumn</param>
        /// <param name="sValue">sValue</param>
        /// <returns>result</returns>
        public List getDynamicReferenceFormula(int nFirstRow, int nFirstColumn, int nLastRow, int nLastColumn, string sValue, ExcelDocument gridDataDocument, ExcelWorksheet excWorkSheet, ExcelTable xlsTable)
        {
            Formula parseFormula = new Formula();
			List lstTokens = FormulaParser.getTokens(sValue);
			int nTokens = lstTokens.size();
			String sToken;
						for (int nToken=0; nToken<nTokens; nToken++)
						{
							sToken = (String)lstTokens.elementAt(nToken);
							if (Formula.Is2DRange(sToken))
							{
								String[,] aTokens = new String[nLastRow-nFirstRow+1, nLastColumn-nFirstColumn+1];
								int[] a2DRangeElements = FormulaParser.get2DRangeElements(sToken);
								bool isRowAbsolute1 = a2DRangeElements[5] == 1;
								bool isColumnAbsolute1 = a2DRangeElements[4] == 1;
								bool isRowAbsolute2 = a2DRangeElements[7] == 1;
								bool isColumnAbsolute2 = a2DRangeElements[6] == 1;
								
								for (int nRowIteration=0; nRowIteration<nLastRow-nFirstRow+1; nRowIteration++)
								{
									//First Row
									int nRowNumber1;
									if (isRowAbsolute1)
									{
										nRowNumber1 = a2DRangeElements[1] + 1;
									}
									else
									{
										nRowNumber1 = (int)(a2DRangeElements[1]+nRowIteration)%Table.MAX_ROWS + 1;
									}

									//End Row
									int nRowNumber2;
									if (isRowAbsolute2)
									{
										nRowNumber2 = a2DRangeElements[3] + 1;
									}
									else
									{
										nRowNumber2 = (int)(a2DRangeElements[3]+nRowIteration)%Table.MAX_ROWS + 1;
									}

									int nColumnNumber1;
									String sColumn1;
									int nColumnNumber2;
									String sColumn2;
									for (int nColumnIteration=0; nColumnIteration<nLastColumn-nFirstColumn+1; nColumnIteration++)
									{
										//First Column 
										if (isColumnAbsolute1)
										{
											nColumnNumber1 = (a2DRangeElements[0] & 0x3FFF) + 1;
										}
										else
										{
											nColumnNumber1 = (a2DRangeElements[0] & 0x3FFF);
											nColumnNumber1 = (nColumnNumber1 + nColumnIteration )%Table.MAX_COLUMNS +1;
										}
										sColumn1 = Formula.getLettersFromColumnNumber(nColumnNumber1);

										//End Column 
										if (isColumnAbsolute2)
										{
											nColumnNumber2 = (a2DRangeElements[2] & 0x3FFF) + 1;
										}
										else
										{
											nColumnNumber2 = (a2DRangeElements[2] & 0x3FFF);
											nColumnNumber2 = (nColumnNumber2 + nColumnIteration )%Table.MAX_COLUMNS +1;
										}
										sColumn2 = Formula.getLettersFromColumnNumber(nColumnNumber2);
										String s2DReference;
                                        if (nRowNumber1 == 0x01 && nRowNumber2 == Table.MAX_ROWS)
										{
											s2DReference = (isColumnAbsolute1?"$":"") + sColumn1 +
												":" +
												(isColumnAbsolute2?"$":"") + sColumn2;
										}
										else if (sColumn1.Equals("A") && sColumn2.Equals("XFD"))
										{
											s2DReference = (isRowAbsolute1?"$":"") + nRowNumber1 +
												":" +
												(isRowAbsolute2?"$":"") + nRowNumber2;
										}
										else
										{
											s2DReference = (isColumnAbsolute1?"$":"") + sColumn1 +
												(isRowAbsolute1?"$":"") + nRowNumber1 +
												":" +
												(isColumnAbsolute2?"$":"") + sColumn2 +
												(isRowAbsolute2?"$":"") + nRowNumber2;
										}
										aTokens[nRowIteration, nColumnIteration] = s2DReference;
									}
								}
								lstTokens.setElementAt(aTokens, nToken);
							}
							
							//2D cell reference
							else if (Formula.Is2DCellReference(sToken))
							{
								String[,] aTokens = new String[nLastRow-nFirstRow+1, nLastColumn-nFirstColumn+1];
								
								int[] a2DCellElements = FormulaParser.get2DCellElements(sToken);
								bool isRowAbsolute = a2DCellElements[3] == 1;
								bool isColumnAbsolute = a2DCellElements[2] == 1;
								for (int nRowIteration=0; nRowIteration<nLastRow-nFirstRow+1; nRowIteration++)
								{
									//Row
									int nRowNumber;
									if (isRowAbsolute)
									{
										nRowNumber = a2DCellElements[1] + 1;
									}
									else
									{
										nRowNumber = (int)(a2DCellElements[1]+nRowIteration)%Table.MAX_ROWS + 1;
									}

									int nColumnNumber;
									String sColumn;
									for (int nColumnIteration=0; nColumnIteration<nLastColumn-nFirstColumn+1; nColumnIteration++)
									{
										//Column 
										if (isColumnAbsolute)
										{
											nColumnNumber = (a2DCellElements[0] & 0x3FFF) + 1;
										}
										else
										{
											nColumnNumber = (a2DCellElements[0] & 0x3FFF);
											nColumnNumber = (nColumnNumber + nColumnIteration )%Table.MAX_COLUMNS +1;
										}
									
										sColumn = Formula.getLettersFromColumnNumber(nColumnNumber);
										aTokens[nRowIteration, nColumnIteration] = (isColumnAbsolute?"$":"") + sColumn +
												(isRowAbsolute?"$":"") + nRowNumber;
									}
								}
								lstTokens.setElementAt(aTokens, nToken);
							}
							
							
						}

                        return lstTokens;

        }
    }
}
