﻿/*
 Copyright (c) 2012-2014 EU Outsourcing, Inc
All rights reserved.  http://www.euoutsourcing.com
Version: 2.1.0
Disclaimer: 
Any redistribution, modification and use of this software either in source or binary forms is permitted provided that the following terms are met: 
1. This software’s source code is not modified to avoid the use of EasyXLS™. 
2.  All servers and computers, whether physical or virtual, using this software or its derivatives must be properly licensed to use EasyXLS™.
3. Any derivatives of this source code that is redistributed must include the above copyright notice, these five conditions and the included disclaimer. 
4. Any redistributions in binary form must include the above copyright notice, these five conditions and the included disclaimer in any materials that are provided with the distribution. 
5. EasyXLS™ and the name of its parent organization may not be used to endorse or promote any software products or derivative works without specific prior written permission from the copyright holders.
DISCLAIMER.  THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using EasyXLS;
using System.Data;
using System.Drawing;
using EasyXLSDemoApp.EasyXLSData;
using System.IO;
using EasyXLS.Util;
using System.Diagnostics;
using System.Collections;
using System.Web.UI.HtmlControls;

using EasyXLS.Charts;
using EasyXLS.Formulas.Util;
using System.Web.UI.DataVisualization.Charting;
using iTextSharp.text;
using iTextSharp.text.html.simpleparser;

namespace EasyXLSDemoApp.EasyXLSGrid
{
    public partial class EasyXLSGridDemo : System.Web.UI.Page
    {
        #region [ Declaration ]

        //the left corner of selection 
        public string leftCornerReference;

        //the right corner of selection
        public string rightCornerReference;

        // file path
        public string filePath;

        // sheet number
        public int sheetNumber;

        // create an object of grid Data
        EasyXLSGridData gridData = new EasyXLSGridData();
        EasyXLSGridChart gridChart = new EasyXLSGridChart();

        // the data table that keeps style and format for all excel cells
        DataTable dtValues = new DataTable();
        DataTable dtCharting = new DataTable();
        DataTable dtAll = new DataTable();

        int isPrinting = 0;
        string isFileName = "";
        string isLeftCorner = "";
        string isRightCorner = "";
        int isSheetNumber = 0;
        string isShowPopulated = "";
        int isPageSize = 0;
        int isPageNo = 0;
        string isDashboard = "";
        bool checkDashboard = false;

        #endregion

        #region [ Load the EXCEL File ]


        /// <summary>
        /// Page_Load
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void Page_Load(object sender, EventArgs e)
        {

            // check if received URL Parameter -> How to render the page (Dashboard style or not)
            if (Request.QueryString["IsDashboard"] != null)
            {
                isDashboard = Request.QueryString["IsDashboard"].ToString();
                checkDashboard = bool.Parse(isDashboard);
            }

            // if page is not postaback
            if (!Page.IsPostBack || checkDashboard)
            {
                DeleteExpiredSessionFolders();

                // check if received URL Parameter -> Printing option
                if (Request.QueryString["Print"] != null)
                    isPrinting = int.Parse(Request.QueryString["Print"].ToString());

                // check if received URL Parameter -> File Name
                if (Request.QueryString["FileName"] != null)
                    isFileName = Request.QueryString["FileName"].ToString();
                // check if received URL Parameter -> Sheet Number
                if (Request.QueryString["SheetNumber"] != null)
                    isSheetNumber = int.Parse(Request.QueryString["SheetNumber"].ToString());
                // check if received URL Parameter -> Selection Left Corner
                if (Request.QueryString["LeftCorner"] != null)
                    isLeftCorner = Request.QueryString["LeftCorner"].ToString();
                // check if received URL Parameter -> Selection Right Corner
                if (Request.QueryString["RightCorner"] != null)
                    isRightCorner = Request.QueryString["RightCorner"].ToString();
                // check if received URL Parameter -> Show only populated area
                if (Request.QueryString["ShowPopulated"] != null)
                    isShowPopulated = Request.QueryString["ShowPopulated"].ToString();
                // check if received URL Parameter -> The size of the page
                if (Request.QueryString["PageSize"] != null)
                    isPageSize = int.Parse(Request.QueryString["PageSize"].ToString());
                // check if received URL Parameter -> The numberof the page to render
                if (Request.QueryString["PageNo"] != null)
                    isPageNo = int.Parse(Request.QueryString["PageNo"].ToString());
                // check if received URL Parameter -> The numberof the page to render
                if (Request.QueryString["IsDashboard"] != null)
                {
                    isDashboard = Request.QueryString["IsDashboard"].ToString();
                    checkDashboard = bool.Parse(isDashboard);
                }


                // add gridview settings
                grvEasyXLSGrid.PageIndex = 0;
                grvEasyXLSGrid.PageSize = 50;
            }


            // load file
            if (!IsPostBack)
            {
               
                // load the excel file
                LoadFile(grvEasyXLSGrid.PageSize, grvEasyXLSGrid.PageIndex, checkDashboard);
     
            }
        }

        /// <summary>
        ///  Load the excel file
        /// </summary>
        protected void LoadFile(int pageSize, int pageIndex, bool isDashboard)
        {
            // check if any file selected
            if (hidFileName.Value != "" || isFileName != "")
            {
                // delete existing charts and images
                DeleteAllImages_and_Charts();

                // check if dashboard style chosen
                if (!isDashboard && cbHideGridlines.Checked) isDashboard = true;

                // show the pager 
                divPager.Visible = true;

                // get the file path
                filePath = HttpContext.Current.Server.MapPath(hidFileName.Value);

                if (filePath.Substring(filePath.LastIndexOf(".") + 1) != "xls" && filePath.Substring(filePath.LastIndexOf(".") + 1) != "xlsx")
                {
                    setError("Please select only xls or xlsx files ! ");
                    return;
                }

                int.TryParse(hidSheetName.Value, out sheetNumber);
                sheetNumber--;

                // get the selection to render
                leftCornerReference = hidLeftCorner.Value;
                rightCornerReference = hidRightCorner.Value;
                
                // check if user is printing
                if (isPrinting == 1)
                {
                    /*########### hide controls for printing #########*/
                    //divSelector.Visible = false;
                    //divFilter.Visible = false;
                    // hide Easy XLS Image
                    divImage.Visible = false;
                    // hide the pager
                    divPager.Visible = false;
                    divScroll.Attributes["Style"] = "";
                    // do not allow paging
                    grvEasyXLSGrid.AllowPaging = true;

                    /*########### set all needed data for printing #########*/
                    // get the file path
                    filePath = isFileName;
                    //get sheet number
                    sheetNumber = isSheetNumber;

                    // get the selection
                    leftCornerReference = isLeftCorner;
                    rightCornerReference = isRightCorner;
                    cbShowPopulated.Checked = bool.Parse(isShowPopulated);
                  
                    grvEasyXLSGrid.PageSize = isPageSize;
                    pageSize = isPageSize;
                    grvEasyXLSGrid.PageIndex = isPageNo;
                }

                // transform cell reference to int positions
                int cellRightCornerX, cellRightCornerY, cellLeftCornerX, cellLeftCornerY;

                cellLeftCornerX = (int)FormulaParser.get2DCellElements(leftCornerReference)[0];
                cellLeftCornerY = (int)FormulaParser.get2DCellElements(leftCornerReference)[1];
                cellRightCornerX = (int)FormulaParser.get2DCellElements(rightCornerReference)[0];
                cellRightCornerY = (int)FormulaParser.get2DCellElements(rightCornerReference)[1];

               
                // set properties for grid data
                gridData.CellLeftCornerX = cellLeftCornerX;
                gridData.CellLeftCornerY = cellLeftCornerY;
                gridData.CellRightCornerX = cellRightCornerX;
                gridData.CellRightCornerY = cellRightCornerY;

                gridData.SheetNumber = sheetNumber;
                gridData.FilePath = filePath;
                gridData.ShowOnlyPopulated = cbShowPopulated.Checked;

                try
                {
                    ExcelDocument xlsDocument = gridData.getExcelDocument(filePath);
                    //check if sheet number is correct and it exists
                    if (xlsDocument != null)
                    {
                        // check if any charts in the file
                        bool fileContainsCharts = gridData.DoesFileContainCharts(xlsDocument, sheetNumber);
                        // check if any images in the file
                        bool fileContainsImages = gridData.DoesFileContainImages(xlsDocument, sheetNumber);

                        // if document contains charts check appearence
                        if (fileContainsCharts || fileContainsImages)
                        {
                            gridChart.UserSession = Session.SessionID;
                            // get the charts and images from the excel file
                            dtCharting = gridChart.DrawCharts(xlsDocument, sheetNumber, leftCornerReference, rightCornerReference, isDashboard);
                        }

                        // get the data to be loaded in the gridview
                        dtAll = gridData.LoadDataTableUsingFile(filePath, grvEasyXLSGrid, dtCharting);

                        // set the horizontal width attributes for the gridview depending on the dynamic excel columns width
                        grvEasyXLSGrid.Width = gridData.gridViewWidth;

                        // set the fixed layout for the table
                        grvEasyXLSGrid.Attributes["Style"] = "table-layout: fixed;";

                        int newSize = pageSize;

                        // if document contains charts check appearence
                        if (fileContainsCharts && !isDashboard)
                        {
                            // check if the pager has to split up a chart and if so, hide that chart
                            HideChartsATPager(pageSize, pageIndex);
                        }

                        // get new page size
                        grvEasyXLSGrid.PageSize = pageSize;
                        grvEasyXLSGrid.PageIndex = grvEasyXLSGrid.PageIndex;

                        // if the file that loads is dashboard
                        if (isDashboard)
                        {
                            // get new page size
                            grvEasyXLSGrid.AllowPaging = false;
                            divPager.Visible = false;
                        }


                        gridData.BoundTheGridView(grvEasyXLSGrid, dtAll);

                        // send grid view information to the pager
                        ucPager.PublicPagerElements(grvEasyXLSGrid, dtAll.Rows.Count, grvEasyXLSGrid.PageIndex, grvEasyXLSGrid.PageSize);
                    }
                    else
                    {
                        setError("Please provide an existing Sheet Number.");
                    }

                }
                catch (Exception e)
                {
                    divScroll.Visible = false;
                    string bu = e.Message;
                    setError("Please review your settings above. The error you encountered is : '" + e.Message + "'.");
                }


                // if the file that loads is dashboard
                if (isDashboard)
                {
                    //change the css for dashboard
                    changeCSSForDashboard();
                }
            }
            else
            {
                divPager.Visible = false;
            }

           
        }


        #endregion

        #region [ Events ]

        protected override void OnPreInit(EventArgs e)
        {
            Response.AddHeader("X-UA-Compatible", "IE=8");

            base.OnPreInit(e);
        }

        /// <summary>
        /// grvEasyXLSGrid_RowDataBound
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void grvEasyXLSGrid_RowDataBound(Object sender, GridViewRowEventArgs e)
        {
            // WHEN HEADER ROW -> SET THE COLUMN'S WIDTH
            if (e.Row.RowType == DataControlRowType.Header)
            {
                int cellIndex = 0;
                // get all table cells in grid view
                foreach(TableCell tc in e.Row.Cells)
                {
                    // set column width as empty cell width
                    int columnWidth = EasyXLSGridData.emptyCellWidth;

                    // if cell has style
                    if (cellIndex < gridData.dtStyles.Columns.Count)
                    {
                        // set column width according to the style data table
                        columnWidth = ((CellStyleData)gridData.dtStyles.Rows[0][cellIndex]).XlsWidth;
                    }

                    // set table cell width
                    tc.Width = Unit.Pixel(columnWidth);

                    // set table cell text to empty
                    tc.Text = "";

                    // got to next cell
                    cellIndex++;
                }

                // hide the header row
                e.Row.Height = Unit.Pixel(0);

                e.Row.BorderStyle = BorderStyle.None;
                e.Row.BorderColor = Color.White;

            }

            /* WHEN DATA ROW -> SET THE ROW'S HEIGHT*/
            if(e.Row.RowType == DataControlRowType.DataRow)
            {
                // get the grid view reference
                GridView grdView = (GridView)sender;

                // set column width as empty cell width
                int rowHeight = EasyXLSGridData.emptyCellHeight;
                  // if cell has style
                if (e.Row.RowIndex < gridData.dtStyles.Rows.Count && 0 < gridData.dtStyles.Columns.Count)
                {
                    rowHeight = ((CellStyleData)gridData.dtStyles.Rows[e.Row.RowIndex][0]).XlsHeight;
                }
                e.Row.Height = Unit.Pixel(rowHeight);

                // bound data
                gridData.RowDataBound(grvEasyXLSGrid, e.Row, e.Row.RowIndex, grdView.PageIndex, dtCharting);
            }
        }


        //Confirms that an HtmlForm control is rendered for the
        //specified ASP.NET server control at run time.
        public override void VerifyRenderingInServerForm(Control control)
        { }


        /// <summary>
        /// MoveFileOnTheServer
        /// </summary>
        /// <param name="fileURL">fileURL</param>
        /// <returns></returns>
        private string MoveFileOnTheServer(string fileURL)
        {
            string filePath = fileURL;
            string fileExt = filePath.Substring(filePath.LastIndexOf(".") + 1);

            string fileName = fileURL.Substring(fileURL.LastIndexOf("\\") + 1);
            string fileNameTrimm = fileName.Replace(" ", "");
            string appPath = "";
            DirectoryInfo dirApp = new DirectoryInfo(Server.MapPath("~/EasyXLSFile/" + Session.SessionID + "/"));

            if(dirApp.Exists)
            {
                appPath = dirApp.FullName + fileNameTrimm;
            }
            else
            {
                DirectoryInfo dirNew =  Directory.CreateDirectory(Server.MapPath("~/EasyXLSFile/" + Session.SessionID + "/"));
                appPath = dirNew.FullName + fileNameTrimm;
            }

            if (txtBoxFileUrl.PostedFile.FileName != "")
            {
                txtBoxFileUrl.PostedFile.SaveAs(appPath);
            }
            else
            {
                FileInfo fileInf = new FileInfo(fileURL);
                fileInf.CopyTo(appPath, true);
            }

            return fileNameTrimm;
        }

        private int SaveFileOnTheServer(string fileURL, string leftCorner, string rightCorner, string sheetNo, bool showPopulated, bool isDashboard)
        {
            int SheetNo  = -1;
            // check if all text boxes populated
            if (fileURL != "" // file url populated
                    && leftCorner != ""             // left reference populated
                    && rightCorner != ""            // right reference populated
                    && sheetNo != ""                // sheet number populated
                )
            {
                // check if the sheet to render exists
                if (int.TryParse(sheetNo, out SheetNo))
                {
                    // clear columns
                    grvEasyXLSGrid.Columns.Clear();

                    string fileNameTrimm ="";

                    if (txtBoxFileUrl.PostedFile.FileName != "")
                    {
                        fileNameTrimm = MoveFileOnTheServer(fileURL);
                    }
                    else
                    {
                        fileNameTrimm = fileURL;
                    }

                    hidFileName.Value = "~/EasyXLSFile/" + Session.SessionID + "/" + fileNameTrimm;
                    hidSheetName.Value = sheetNo;
                    hidLeftCorner.Value = leftCorner;
                    hidRightCorner.Value = rightCorner;
                    hidShowPopulated.Value = showPopulated.ToString();
                    grvEasyXLSGrid.Visible = true;
                    divPager.Visible = true;
                    lblErrorURL.Visible = false;

                    // add gridview settings
                    grvEasyXLSGrid.PageIndex = 0;
                    grvEasyXLSGrid.PageSize = 50;

                    // load the excel file
                    LoadFile(grvEasyXLSGrid.PageSize, grvEasyXLSGrid.PageIndex, isDashboard);
                }
                else
                {
                    setError("Please insert a valid number for the Sheet Number.");
                }
            }
            else
            {
                setError("Please insert all 4 relevant values !");
            }

            return SheetNo;
        }

         /// <summary>
        /// lbnSaveURL_Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbnSaveURL_Click(Object sender, EventArgs e)
        {
            // check if sheetName is a number
            int SheetNo = -1;

            divError.Visible = false;
            divImgError.Visible = false;
            // delete existing files
            DeleteAllFiles();

            SheetNo = SaveFileOnTheServer(txtBoxFileUrl.PostedFile.FileName, txtLeftCorner.Text, txtRightCorner.Text, txtSheetNo.Text, cbShowPopulated.Checked, false);
        }


        /// <summary>
        /// changeCSSForDashboard
        /// </summary>
        protected void changeCSSForDashboard()
        {
            grvEasyXLSGrid.CssClass = "table_DASH";

            foreach (GridViewRow gr in grvEasyXLSGrid.Rows)
            {
                foreach (TableCell tbCell in gr.Cells)
                {
                    tbCell.CssClass = "table_td_DASH";
                }
            }
        }

        #endregion

        #region [ Methods ]

        /// <summary>
        /// get the server root path for javascript redirect
        /// </summary>
        public string ServerRootPath
        {
            get
            {
                return this.Server.MapPath(hidFileName.Value).Replace("\\", "/");
            }
        }

        /// <summary>
        /// setError
        /// </summary>
        /// <param name="errorMessage"></param>
        protected void setError(string errorMessage)
        {
            // set the error
            lblErrorURL.Text = errorMessage;

            // hide controls
            grvEasyXLSGrid.Visible = false;
            divPager.Visible = false;
            lblErrorURL.Visible = true;
            divError.Visible = true;
            divImgError.Visible = true;
        }

        /// <summary>
        /// DeleteAllImages_and_Charts
        /// </summary>
        void DeleteAllImages_and_Charts()
        {
            //get the folder that holds the chart
            string chartFolder = Server.MapPath("~/EasyXLSChartFiles/" + Session.SessionID + "/");
            DirectoryInfo directory = new DirectoryInfo(chartFolder);

            // check if directory exists
            if (directory.Exists)
            {
                // get the files from the directory
                FileInfo [] files = directory.GetFiles();

                // take each file and delete it
                foreach (FileInfo file in files)
                {
                    file.Delete();
                }
            }

            //get the folder that holds the images
            string imageFolder = Server.MapPath("~/EasyXLSImage/" + Session.SessionID + "/");
            directory = new DirectoryInfo(imageFolder);

            // check if directory exists
            if (directory.Exists)
            {
                // get the files from the directory
                FileInfo[] files = directory.GetFiles();

                // take each image file and delete it
                foreach (FileInfo file in files)
                {
                    if (file.Name.Contains("Image"))
                    {
                        file.Delete();
                    }
                }
            }
        }

        /// <summary>
        /// DeleteAllFiles
        /// </summary>
        void DeleteAllFiles()
        {
            //get the folder that holds the chart
            string FileFolder = Server.MapPath("~/EasyXLSFile/" + Session.SessionID + "/");
            DirectoryInfo FileDirectory = new DirectoryInfo(FileFolder);

            // check if directory exists
            if (FileDirectory.Exists)
            {
                // get the files from the directory
                FileInfo[] files = FileDirectory.GetFiles();

                // take each image file and delete it
                foreach (FileInfo file in files)
                {
                    if (file.Extension.Contains("xls") || file.Extension.Contains("xlsx"))
                        file.Delete();
                }
            }
        }


        /// <summary>
        /// DeleteExpiredSessionFolders
        /// </summary>
        void DeleteExpiredSessionFolders()
        {
            //get the folder that holds the chart
            string FileSessionFolder = Server.MapPath("~/EasyXLSFile/");
            string ChartSessionFolder = Server.MapPath("~/EasyXLSChartFiles/");
            string ImageSessionFolder = Server.MapPath("~/EasyXLSImage/");
            DirectoryInfo FileSessionDirectory = new DirectoryInfo(FileSessionFolder);
            DirectoryInfo ChartSessionDirectory = new DirectoryInfo(ChartSessionFolder);
            DirectoryInfo ImageSessionDirectory = new DirectoryInfo(ImageSessionFolder);

            // check if directory exists
            if (FileSessionDirectory.Exists)
            {
                DirectoryInfo[] SessionDirectories = FileSessionDirectory.GetDirectories();

                foreach (DirectoryInfo directory in SessionDirectories)
                {
                    DateTime dirCreateTime = directory.CreationTime;
                    TimeSpan timeSpan = DateTime.Now.Subtract(dirCreateTime);

                    if (timeSpan.Days > 1)
                    {
                        directory.Delete(true);
                    }
                }
            }

            // check if directory exists
            if (ChartSessionDirectory.Exists)
            {
                DirectoryInfo[] SessionDirectories = ChartSessionDirectory.GetDirectories();

                foreach (DirectoryInfo directory in SessionDirectories)
                {
                    DateTime dirCreateTime = directory.CreationTime;
                    TimeSpan timeSpan = DateTime.Now.Subtract(dirCreateTime);

                    if (timeSpan.Days > 1)
                    {
                        directory.Delete(true);
                    }
                }
            }

            // check if directory exists
            if (ImageSessionDirectory.Exists)
            {
                DirectoryInfo[] SessionDirectories = ImageSessionDirectory.GetDirectories();

                foreach (DirectoryInfo directory in SessionDirectories)
                {
                    DateTime dirCreateTime = directory.CreationTime;
                    TimeSpan timeSpan = DateTime.Now.Subtract(dirCreateTime);

                    if (timeSpan.Days > 1)
                    {
                        directory.Delete(true);
                    }
                }
            }
        }


        #endregion

        #region [ Custom Pager Settings ]

        /// <summary>
        /// HideChartsATPager
        /// If pager cuts through a chart then that chart will be hidden
        /// </summary>
        /// <param name="theSize">theSize</param>
        /// <param name="theIndex">theIndex</param>
        protected void HideChartsATPager(int thePageSize, int thePageIndex)
        {
            // the aux is 0 when fist page is loading and 1 for all the other
            int aux = (thePageIndex > 0) ? 1 : 0;

            // index for counting rows
            int row = 0;
            // get the rows that need to be shown in the gridview
            for (int rowsIdx = thePageSize * thePageIndex + aux; rowsIdx < dtAll.Rows.Count; rowsIdx++)
            {
                // check only the usefull number of rows 
                if (row <= thePageSize)
                {
                    // get all columns
                    for (int cellsIdx = 0; cellsIdx < dtAll.Columns.Count; cellsIdx++)
                    {
                        // check if cell contains a chart
                        if (dtAll.Rows[rowsIdx][cellsIdx].ToString().Contains("EasyXLSChartFiles"))
                        {
                            // get the ChartData
                            ChartData chData = ChartData.FindChartByURL(dtAll.Rows[rowsIdx][cellsIdx].ToString(), dtCharting);
                            // get the rowspan for this chart 
                            int rowSpan = chData.RowSpan;
                            // check how many rows separates the chart from the pager
                            int diffRow = thePageSize - row;

                            // if chart needs more space 
                            if (diffRow < rowSpan)
                            {
                                // add the needed rows to the gridview size
                                dtAll.Rows[rowsIdx][cellsIdx] = "";
                            }
                        }
                    }
                }
                else
                {
                    // no need to search for more
                    break;
                }

                // increment row number
                row++;
            }

        }


        /// <summary>
        /// RelodTheGridView
        /// </summary>
        public void ReloadTheGridView(int PageSize, int PageIndex)
        {
            string cssClass = grvEasyXLSGrid.CssClass;
            bool isDashboard = false;
            if (cssClass == "table_DASH") isDashboard = true;
            // empty the grid view and the style data table
            grvEasyXLSGrid.Columns.Clear();
            dtAll.Clear();

            // set the current page index
            grvEasyXLSGrid.PageIndex = PageIndex;

            // load the file into gridview
            LoadFile(PageSize, PageIndex, isDashboard);
        }

        #endregion
    }
}
