﻿/*
 Copyright (c) 2012-2014 EU Outsourcing, Inc
All rights reserved.  http://www.euoutsourcing.com
Version: 2.1.0
Disclaimer: 
Any redistribution, modification and use of this software either in source or binary forms is permitted provided that the following terms are met: 
1. This software’s source code is not modified to avoid the use of EasyXLS™. 
2.  All servers and computers, whether physical or virtual, using this software or its derivatives must be properly licensed to use EasyXLS™.
3. Any derivatives of this source code that is redistributed must include the above copyright notice, these five conditions and the included disclaimer. 
4. Any redistributions in binary form must include the above copyright notice, these five conditions and the included disclaimer in any materials that are provided with the distribution. 
5. EasyXLS™ and the name of its parent organization may not be used to endorse or promote any software products or derivative works without specific prior written permission from the copyright holders.
DISCLAIMER.  THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Reflection;

namespace EasyXLSDemoApp.EasyXLSGrid
{
    public partial class EasyXLSGridPager : System.Web.UI.UserControl
    {
        #region [ Declaration ]

        // SET IN LIST SCREEN
        public int PagerRecordsPerPage = 50; // to select no of records / page
        public string PagerScreenName; // screen name for user settings
        public string PagerListURL; // URL to redirect to the list screen
        public int PagerPageNo = 1; // to display Go to Page No
        public int PagerUserPageNo = 1; // to select Go to Page No
        public int PagerTotalNoRecords;
        // SET IN LIST SCREEN

        #endregion


        #region [ Load Data ]

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                // reload pager
                ShowCustomPager();
            }
        }

        #endregion


        #region [ Set and calculate pager data ]

        /// <summary>
        /// ShowCustomPager
        /// </summary>
        public void ShowCustomPager()
        {
            // populate Records per page ddl from 5 to 500
            int intCounter = 0;
            int intFor = 0;
            bool addIndexDone = false;
            ddlPagerRecordsPerPage.Items.Clear();
            for (intFor = 5; intFor <= 50; intFor += 5)
            {
                if (PagerRecordsPerPage < intFor && !addIndexDone)
                {
                    ddlPagerRecordsPerPage.Items.Insert(intCounter, new System.Web.UI.WebControls.ListItem(PagerRecordsPerPage.ToString(), PagerRecordsPerPage.ToString()));
                    intCounter++;
                    addIndexDone = true;
                }

                ddlPagerRecordsPerPage.Items.Insert(intCounter, new System.Web.UI.WebControls.ListItem(intFor.ToString(), intFor.ToString()));
                intCounter = intCounter + 1;
            }
            for (intFor = 100; intFor <= 500; intFor += 400)
            {
                if (PagerRecordsPerPage < intFor && !addIndexDone)
                {
                    ddlPagerRecordsPerPage.Items.Insert(intCounter, new System.Web.UI.WebControls.ListItem(PagerRecordsPerPage.ToString(), PagerRecordsPerPage.ToString()));
                    intCounter++;
                    addIndexDone = true;
                }

                ddlPagerRecordsPerPage.Items.Insert(intCounter, new System.Web.UI.WebControls.ListItem(intFor.ToString(), intFor.ToString()));
                intCounter = intCounter + 1;
            }

            // set by default PagerRecordsPerPage as declared in line 33
            ddlPagerRecordsPerPage.SelectedIndex = ddlPagerRecordsPerPage.Items.IndexOf(ddlPagerRecordsPerPage.Items.FindByValue(PagerRecordsPerPage.ToString()));

            // populate number of pages ddl from 1 to PagerPageNo
            intCounter = 0;
            intFor = 0;
            ddlPagerPage.Items.Clear();
            for (intFor = 1; intFor <= PagerPageNo; intFor++)
            {
                ddlPagerPage.Items.Insert(intCounter, new System.Web.UI.WebControls.ListItem(intFor.ToString(), intFor.ToString()));
                intCounter = intCounter + 1;
            }
            // set by default the current page number (at first is  1)
            ddlPagerPage.SelectedIndex = ddlPagerPage.Items.IndexOf(ddlPagerPage.Items.FindByValue(PagerUserPageNo.ToString()));

            // Hide Previous button
            if (PagerUserPageNo == 1)
            {
                lbnFirst.Enabled = false;
                imgPagerFirst.ImageUrl = "../EasyXLSImage/firstD.gif";

                lbnPrevious.Enabled = false;
                imgPagerPrevious.ImageUrl = "../EasyXLSImage/previousD.gif";
            }
            else // Show Previous button
            {
                lbnFirst.Enabled = true;
                imgPagerFirst.ImageUrl = "../EasyXLSImage/first.gif";

                lbnPrevious.Enabled = true;
                imgPagerPrevious.ImageUrl = "../EasyXLSImage/previous.gif";
            }

            // Hide Next button
            if (PagerUserPageNo == PagerPageNo)
            {
                lbnNext.Enabled = false;
                imgPagerNext.ImageUrl = "../EasyXLSImage/nextD.gif";

                lbnLast.Enabled = false;
                imgPagerLast.ImageUrl = "../EasyXLSImage/lastD.gif";
            }
            else // Show Next button
            {
                lbnNext.Enabled = true;
                imgPagerNext.ImageUrl = "../EasyXLSImage/next.gif";

                lbnLast.Enabled = true;
                imgPagerLast.ImageUrl = "../EasyXLSImage/last.gif";
            }

            // populate :  x(FirstDisplay) TO y(LastDisplay) OUT OF z(PagerRecordsPerPage)
            int FirstDisplay = (PagerUserPageNo - 1) * PagerRecordsPerPage + 1;
            int LastDisplay = 0;

            // if current page is the last one
            if (PagerUserPageNo == PagerPageNo)
            {
                LastDisplay = PagerTotalNoRecords;
            }
            else
            {
                LastDisplay = PagerUserPageNo * PagerRecordsPerPage;
            }

            // populate the labels
            lblFirstDisplay.Text = FirstDisplay.ToString();
            lblLastDisplay.Text = LastDisplay.ToString();
            lblTotalNoRecords.Text = PagerTotalNoRecords.ToString();
            lblPageNo.Text = PagerPageNo.ToString();
        }

        /// <summary>
        /// PublicPagerElements
        /// </summary>
        /// <param name="ScreenName">ScreenName</param>
        /// <param name="ListURL">ListURL</param>
        /// <param name="PublicTotalNoRecords">PublicTotalNoRecords</param>
        /// <param name="PublicNoOfRec">PublicNoOfRec</param>
        /// <param name="PublicPageNo">PublicPageNo</param>
        public void PublicPagerElements(GridView grvXLS, int PublicTotalNoRecords, int PublicUserPage, int PublicRecordsPerPage)
        {
            PagerRecordsPerPage = PublicRecordsPerPage;
            //calculate the number of pages
            int intTotalPage = PublicTotalNoRecords / PagerRecordsPerPage;

            if (PublicTotalNoRecords % PagerRecordsPerPage > 0)
            { intTotalPage = intTotalPage + 1; }

            // set total page number
            PagerPageNo = intTotalPage;
            // set total number of records
            PagerTotalNoRecords = PublicTotalNoRecords;
            // set current page number
            PagerUserPageNo = PublicUserPage + 1;

            ShowCustomPager();
        }

        #endregion


        #region [ Events ]

        /// <summary>
        /// ddlPagerRecordsPerPage_SelectedIndexChanged
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void ddlPagerRecordsPerPage_SelectedIndexChanged(object sender, EventArgs e)
        {
            // get the selected no of records per page
            int noOfRecPerPage = 100000;
            int.TryParse(ddlPagerRecordsPerPage.SelectedItem.Value.ToString(), out noOfRecPerPage);

            // set the no of records per page
            PagerRecordsPerPage = noOfRecPerPage;

            // reload the gridview
            MethodInfo mi = this.Page.GetType().GetMethod("ReloadTheGridView", BindingFlags.Public | BindingFlags.Instance);
            if (mi != null) mi.Invoke(this.Page, new Object[] { PagerRecordsPerPage, PagerUserPageNo - 1 });

            // reclaculate the number of pages
            int intTotalPage = PagerTotalNoRecords / PagerRecordsPerPage;
            if (PagerTotalNoRecords % PagerRecordsPerPage > 0)
            { intTotalPage = intTotalPage + 1; }
            PagerPageNo = intTotalPage;

            // get the current page of te user
            PagerUserPageNo = 1;
                //int.Parse(ddlPagerPage.SelectedItem.Value.ToString());

            // if the current page of the user is greater then the new number of pages then go to page 1
            if (PagerPageNo < PagerUserPageNo) PagerUserPageNo = 1;

            // reload pager
            ShowCustomPager();
        }

        /// <summary>
        /// ddlPagerPage_SelectedIndexChanged
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void ddlPagerPage_SelectedIndexChanged(object sender, EventArgs e)
        {
            // get the new page number
            int noOfPagerPage = 0;
            int.TryParse(ddlPagerPage.SelectedItem.Value.ToString(), out noOfPagerPage);

            //  set the records per page
            PagerRecordsPerPage = int.Parse(ddlPagerRecordsPerPage.SelectedItem.Value.ToString());

            // reload the gridview
            MethodInfo mi = this.Page.GetType().GetMethod("ReloadTheGridView", BindingFlags.Public | BindingFlags.Instance);
            if (mi != null) mi.Invoke(this.Page, new Object[] { PagerRecordsPerPage, noOfPagerPage - 1});

            // set the current page number 
            PagerUserPageNo = int.Parse(ddlPagerPage.SelectedItem.Value.ToString());

            // reload pager
            ShowCustomPager();
        }

        /// <summary>
        /// lbnFirst_Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbnFirst_Click(object sender, EventArgs e)
        {
            // set current page and records per page
            PagerUserPageNo = 1;
            PagerRecordsPerPage = int.Parse(ddlPagerRecordsPerPage.SelectedItem.Value.ToString());

            // reload the gridview
            MethodInfo mi = this.Page.GetType().GetMethod("ReloadTheGridView", BindingFlags.Public | BindingFlags.Instance);
            if (mi != null) mi.Invoke(this.Page, new Object[] { PagerRecordsPerPage, PagerUserPageNo - 1 });

            // reload pager
            ShowCustomPager();
        }

        /// <summary>
        /// lbnPrevious_Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbnPrevious_Click(object sender, EventArgs e)
        {
            // go to the previous page
            PagerUserPageNo = int.Parse(ddlPagerPage.SelectedItem.Value.ToString());
            int previousPage = PagerUserPageNo - 1;

            // set current page and records per page
            PagerUserPageNo = previousPage;
            PagerRecordsPerPage = int.Parse(ddlPagerRecordsPerPage.SelectedItem.Value.ToString());

            // reload the gridview
            MethodInfo mi = this.Page.GetType().GetMethod("ReloadTheGridView", BindingFlags.Public | BindingFlags.Instance);
            if (mi != null) mi.Invoke(this.Page, new Object[] { PagerRecordsPerPage, PagerUserPageNo - 1 });

            // reload pager
            ShowCustomPager();
        }

        /// <summary>
        /// lbnNext_Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbnNext_Click(object sender, EventArgs e)
        {
            // go to the previous page
            PagerUserPageNo = int.Parse(ddlPagerPage.SelectedItem.Value.ToString());
            int nextPage = PagerUserPageNo + 1;

            // set current page and records per page
            PagerUserPageNo = nextPage;
            PagerRecordsPerPage = int.Parse(ddlPagerRecordsPerPage.SelectedItem.Value.ToString());

            // reload the gridview
            MethodInfo mi = this.Page.GetType().GetMethod("ReloadTheGridView", BindingFlags.Public | BindingFlags.Instance);
            if (mi != null) mi.Invoke(this.Page, new Object[] { PagerRecordsPerPage, PagerUserPageNo - 1 });

            // reload pager
            ShowCustomPager();
        }

        /// <summary>
        /// lbnLast_Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbnLast_Click(object sender, EventArgs e)
        {
            // go to the last page
            PagerPageNo = ddlPagerPage.Items.Count;
            int lastPage = PagerPageNo;

            // set current page and records per page
            PagerUserPageNo = lastPage;
            PagerRecordsPerPage = int.Parse(ddlPagerRecordsPerPage.SelectedItem.Value.ToString());

            // reload the gridview
            MethodInfo mi = this.Page.GetType().GetMethod("ReloadTheGridView", BindingFlags.Public | BindingFlags.Instance);
            if (mi != null) mi.Invoke(this.Page, new Object[] { PagerRecordsPerPage, PagerUserPageNo - 1 });

            // reload pager
            ShowCustomPager();
        }


#endregion

    }
}